// This code conforms with the UFC specification version 1.0
// and was automatically generated by FFC version 0.5.1.

#ifndef __QUADRATUREELEMENT_H
#define __QUADRATUREELEMENT_H

#include <cmath>
#include <stdexcept>
#include <ufc.h>

/// This class defines the interface for a finite element.

class QuadratureElementBilinearForm_finite_element_0: public ufc::finite_element
{
public:

  /// Constructor
  QuadratureElementBilinearForm_finite_element_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~QuadratureElementBilinearForm_finite_element_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 2 on a triangle";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::triangle;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 6;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute inverse of Jacobian
    
    // Get coordinates and map to the reference (UFC) element
    double x = (element_coordinates[0][1]*element_coordinates[2][0] -\
                element_coordinates[0][0]*element_coordinates[2][1] +\
                J_11*coordinates[0] - J_01*coordinates[1]) / detJ;
    double y = (element_coordinates[1][1]*element_coordinates[0][0] -\
                element_coordinates[1][0]*element_coordinates[0][1] -\
                J_10*coordinates[0] + J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-08)
      x = -1.0;
    else
      x = 2.0 *x/(1.0 - y) - 1.0;
    y = 2.0*y - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111*psitilde_bs_0_1 + 1.66666667*y*psitilde_bs_0_1 - 0.555555556*psitilde_bs_0_0;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_2_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.707106781*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    const double basisvalue1 = 1.73205081*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
    const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
    const double basisvalue3 = 2.73861279*psitilde_a_2*scalings_y_2*psitilde_bs_2_0;
    const double basisvalue4 = 2.12132034*psitilde_a_1*scalings_y_1*psitilde_bs_1_1;
    const double basisvalue5 = 1.22474487*psitilde_a_0*scalings_y_0*psitilde_bs_0_2;
    
    // Table(s) of coefficients
    const static double coefficients0[6][6] = \
    {{0, -0.173205081, -0.1, 0.121716124, 0.0942809042, 0.0544331054},
    {0, 0.173205081, -0.1, 0.121716124, -0.0942809042, 0.0544331054},
    {0, 0, 0.2, 0, 0, 0.163299316},
    {0.471404521, 0.230940108, 0.133333333, 0, 0.188561808, -0.163299316},
    {0.471404521, -0.230940108, 0.133333333, 0, -0.188561808, -0.163299316},
    {0.471404521, 0, -0.266666667, -0.243432248, 0, 0.0544331054}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    const double coeff0_4 = coefficients0[dof][4];
    const double coeff0_5 = coefficients0[dof][5];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5;
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute inverse of Jacobian
    
    // Get coordinates and map to the reference (UFC) element
    double x = (element_coordinates[0][1]*element_coordinates[2][0] -\
                element_coordinates[0][0]*element_coordinates[2][1] +\
                J_11*coordinates[0] - J_01*coordinates[1]) / detJ;
    double y = (element_coordinates[1][1]*element_coordinates[0][0] -\
                element_coordinates[1][0]*element_coordinates[0][1] -\
                J_10*coordinates[0] + J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-08)
      x = -1.0;
    else
      x = 2.0 *x/(1.0 - y) - 1.0;
    y = 2.0*y - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 2;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 1)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[2][2] =  {{J_11 / detJ, -J_01 / detJ}, {-J_10 / detJ, J_00 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111*psitilde_bs_0_1 + 1.66666667*y*psitilde_bs_0_1 - 0.555555556*psitilde_bs_0_0;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_2_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.707106781*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    const double basisvalue1 = 1.73205081*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
    const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
    const double basisvalue3 = 2.73861279*psitilde_a_2*scalings_y_2*psitilde_bs_2_0;
    const double basisvalue4 = 2.12132034*psitilde_a_1*scalings_y_1*psitilde_bs_1_1;
    const double basisvalue5 = 1.22474487*psitilde_a_0*scalings_y_0*psitilde_bs_0_2;
    
    // Table(s) of coefficients
    const static double coefficients0[6][6] = \
    {{0, -0.173205081, -0.1, 0.121716124, 0.0942809042, 0.0544331054},
    {0, 0.173205081, -0.1, 0.121716124, -0.0942809042, 0.0544331054},
    {0, 0, 0.2, 0, 0, 0.163299316},
    {0.471404521, 0.230940108, 0.133333333, 0, 0.188561808, -0.163299316},
    {0.471404521, -0.230940108, 0.133333333, 0, -0.188561808, -0.163299316},
    {0.471404521, 0, -0.266666667, -0.243432248, 0, 0.0544331054}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[6][6] = \
    {{0, 0, 0, 0, 0, 0},
    {4.89897949, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0},
    {0, 9.48683298, 0, 0, 0, 0},
    {4, 0, 7.07106781, 0, 0, 0},
    {0, 0, 0, 0, 0, 0}};
    
    const static double dmats1[6][6] = \
    {{0, 0, 0, 0, 0, 0},
    {2.44948974, 0, 0, 0, 0, 0},
    {4.24264069, 0, 0, 0, 0, 0},
    {2.5819889, 4.74341649, -0.912870929, 0, 0, 0},
    {2, 6.12372436, 3.53553391, 0, 0, 0},
    {-2.30940108, 0, 8.16496581, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    double coeff0_4 = 0;
    double coeff0_5 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    double new_coeff0_4 = 0;
    double new_coeff0_5 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
      new_coeff0_4 = coefficients0[dof][4];
      new_coeff0_5 = coefficients0[dof][5];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
        coeff0_4 = new_coeff0_4;
        coeff0_5 = new_coeff0_5;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3];
          new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4];
          new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3];
          new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4];
          new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    const static double X[6][1][2] = {{{0, 0}}, {{1, 0}}, {{0, 1}}, {{0.5, 0.5}}, {{0, 0.5}}, {{0.5, 0}}};
    const static double W[6][1] = {{1}, {1}, {1}, {1}, {1}, {1}};
    const static double D[6][1][1] = {{{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}};
    
    const double * const * x = c.coordinates;
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    
    // Compute affine mapping y = F(X)
    double y[2];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1];
    
    // Evaluate function at physical points
    double values[1];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Affine map: Do nothing
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 1; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new QuadratureElementBilinearForm_finite_element_0();
  }

};

/// This class defines the interface for a finite element.

class QuadratureElementBilinearForm_finite_element_1: public ufc::finite_element
{
public:

  /// Constructor
  QuadratureElementBilinearForm_finite_element_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~QuadratureElementBilinearForm_finite_element_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 2 on a triangle";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::triangle;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 6;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute inverse of Jacobian
    
    // Get coordinates and map to the reference (UFC) element
    double x = (element_coordinates[0][1]*element_coordinates[2][0] -\
                element_coordinates[0][0]*element_coordinates[2][1] +\
                J_11*coordinates[0] - J_01*coordinates[1]) / detJ;
    double y = (element_coordinates[1][1]*element_coordinates[0][0] -\
                element_coordinates[1][0]*element_coordinates[0][1] -\
                J_10*coordinates[0] + J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-08)
      x = -1.0;
    else
      x = 2.0 *x/(1.0 - y) - 1.0;
    y = 2.0*y - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111*psitilde_bs_0_1 + 1.66666667*y*psitilde_bs_0_1 - 0.555555556*psitilde_bs_0_0;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_2_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.707106781*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    const double basisvalue1 = 1.73205081*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
    const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
    const double basisvalue3 = 2.73861279*psitilde_a_2*scalings_y_2*psitilde_bs_2_0;
    const double basisvalue4 = 2.12132034*psitilde_a_1*scalings_y_1*psitilde_bs_1_1;
    const double basisvalue5 = 1.22474487*psitilde_a_0*scalings_y_0*psitilde_bs_0_2;
    
    // Table(s) of coefficients
    const static double coefficients0[6][6] = \
    {{0, -0.173205081, -0.1, 0.121716124, 0.0942809042, 0.0544331054},
    {0, 0.173205081, -0.1, 0.121716124, -0.0942809042, 0.0544331054},
    {0, 0, 0.2, 0, 0, 0.163299316},
    {0.471404521, 0.230940108, 0.133333333, 0, 0.188561808, -0.163299316},
    {0.471404521, -0.230940108, 0.133333333, 0, -0.188561808, -0.163299316},
    {0.471404521, 0, -0.266666667, -0.243432248, 0, 0.0544331054}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    const double coeff0_4 = coefficients0[dof][4];
    const double coeff0_5 = coefficients0[dof][5];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5;
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute inverse of Jacobian
    
    // Get coordinates and map to the reference (UFC) element
    double x = (element_coordinates[0][1]*element_coordinates[2][0] -\
                element_coordinates[0][0]*element_coordinates[2][1] +\
                J_11*coordinates[0] - J_01*coordinates[1]) / detJ;
    double y = (element_coordinates[1][1]*element_coordinates[0][0] -\
                element_coordinates[1][0]*element_coordinates[0][1] -\
                J_10*coordinates[0] + J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-08)
      x = -1.0;
    else
      x = 2.0 *x/(1.0 - y) - 1.0;
    y = 2.0*y - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 2;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 1)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[2][2] =  {{J_11 / detJ, -J_01 / detJ}, {-J_10 / detJ, J_00 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111*psitilde_bs_0_1 + 1.66666667*y*psitilde_bs_0_1 - 0.555555556*psitilde_bs_0_0;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_2_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.707106781*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    const double basisvalue1 = 1.73205081*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
    const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
    const double basisvalue3 = 2.73861279*psitilde_a_2*scalings_y_2*psitilde_bs_2_0;
    const double basisvalue4 = 2.12132034*psitilde_a_1*scalings_y_1*psitilde_bs_1_1;
    const double basisvalue5 = 1.22474487*psitilde_a_0*scalings_y_0*psitilde_bs_0_2;
    
    // Table(s) of coefficients
    const static double coefficients0[6][6] = \
    {{0, -0.173205081, -0.1, 0.121716124, 0.0942809042, 0.0544331054},
    {0, 0.173205081, -0.1, 0.121716124, -0.0942809042, 0.0544331054},
    {0, 0, 0.2, 0, 0, 0.163299316},
    {0.471404521, 0.230940108, 0.133333333, 0, 0.188561808, -0.163299316},
    {0.471404521, -0.230940108, 0.133333333, 0, -0.188561808, -0.163299316},
    {0.471404521, 0, -0.266666667, -0.243432248, 0, 0.0544331054}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[6][6] = \
    {{0, 0, 0, 0, 0, 0},
    {4.89897949, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0},
    {0, 9.48683298, 0, 0, 0, 0},
    {4, 0, 7.07106781, 0, 0, 0},
    {0, 0, 0, 0, 0, 0}};
    
    const static double dmats1[6][6] = \
    {{0, 0, 0, 0, 0, 0},
    {2.44948974, 0, 0, 0, 0, 0},
    {4.24264069, 0, 0, 0, 0, 0},
    {2.5819889, 4.74341649, -0.912870929, 0, 0, 0},
    {2, 6.12372436, 3.53553391, 0, 0, 0},
    {-2.30940108, 0, 8.16496581, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    double coeff0_4 = 0;
    double coeff0_5 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    double new_coeff0_4 = 0;
    double new_coeff0_5 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
      new_coeff0_4 = coefficients0[dof][4];
      new_coeff0_5 = coefficients0[dof][5];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
        coeff0_4 = new_coeff0_4;
        coeff0_5 = new_coeff0_5;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3];
          new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4];
          new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3];
          new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4];
          new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    const static double X[6][1][2] = {{{0, 0}}, {{1, 0}}, {{0, 1}}, {{0.5, 0.5}}, {{0, 0.5}}, {{0.5, 0}}};
    const static double W[6][1] = {{1}, {1}, {1}, {1}, {1}, {1}};
    const static double D[6][1][1] = {{{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}};
    
    const double * const * x = c.coordinates;
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    
    // Compute affine mapping y = F(X)
    double y[2];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1];
    
    // Evaluate function at physical points
    double values[1];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Affine map: Do nothing
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 1; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new QuadratureElementBilinearForm_finite_element_1();
  }

};

/// This class defines the interface for a finite element.

class QuadratureElementBilinearForm_finite_element_2: public ufc::finite_element
{
public:

  /// Constructor
  QuadratureElementBilinearForm_finite_element_2() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~QuadratureElementBilinearForm_finite_element_2()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 2 on a triangle";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::triangle;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 6;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute inverse of Jacobian
    
    // Get coordinates and map to the reference (UFC) element
    double x = (element_coordinates[0][1]*element_coordinates[2][0] -\
                element_coordinates[0][0]*element_coordinates[2][1] +\
                J_11*coordinates[0] - J_01*coordinates[1]) / detJ;
    double y = (element_coordinates[1][1]*element_coordinates[0][0] -\
                element_coordinates[1][0]*element_coordinates[0][1] -\
                J_10*coordinates[0] + J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-08)
      x = -1.0;
    else
      x = 2.0 *x/(1.0 - y) - 1.0;
    y = 2.0*y - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111*psitilde_bs_0_1 + 1.66666667*y*psitilde_bs_0_1 - 0.555555556*psitilde_bs_0_0;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_2_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.707106781*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    const double basisvalue1 = 1.73205081*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
    const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
    const double basisvalue3 = 2.73861279*psitilde_a_2*scalings_y_2*psitilde_bs_2_0;
    const double basisvalue4 = 2.12132034*psitilde_a_1*scalings_y_1*psitilde_bs_1_1;
    const double basisvalue5 = 1.22474487*psitilde_a_0*scalings_y_0*psitilde_bs_0_2;
    
    // Table(s) of coefficients
    const static double coefficients0[6][6] = \
    {{0, -0.173205081, -0.1, 0.121716124, 0.0942809042, 0.0544331054},
    {0, 0.173205081, -0.1, 0.121716124, -0.0942809042, 0.0544331054},
    {0, 0, 0.2, 0, 0, 0.163299316},
    {0.471404521, 0.230940108, 0.133333333, 0, 0.188561808, -0.163299316},
    {0.471404521, -0.230940108, 0.133333333, 0, -0.188561808, -0.163299316},
    {0.471404521, 0, -0.266666667, -0.243432248, 0, 0.0544331054}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    const double coeff0_4 = coefficients0[dof][4];
    const double coeff0_5 = coefficients0[dof][5];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5;
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute inverse of Jacobian
    
    // Get coordinates and map to the reference (UFC) element
    double x = (element_coordinates[0][1]*element_coordinates[2][0] -\
                element_coordinates[0][0]*element_coordinates[2][1] +\
                J_11*coordinates[0] - J_01*coordinates[1]) / detJ;
    double y = (element_coordinates[1][1]*element_coordinates[0][0] -\
                element_coordinates[1][0]*element_coordinates[0][1] -\
                J_10*coordinates[0] + J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-08)
      x = -1.0;
    else
      x = 2.0 *x/(1.0 - y) - 1.0;
    y = 2.0*y - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 2;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 1)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[2][2] =  {{J_11 / detJ, -J_01 / detJ}, {-J_10 / detJ, J_00 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111*psitilde_bs_0_1 + 1.66666667*y*psitilde_bs_0_1 - 0.555555556*psitilde_bs_0_0;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_2_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.707106781*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    const double basisvalue1 = 1.73205081*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
    const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
    const double basisvalue3 = 2.73861279*psitilde_a_2*scalings_y_2*psitilde_bs_2_0;
    const double basisvalue4 = 2.12132034*psitilde_a_1*scalings_y_1*psitilde_bs_1_1;
    const double basisvalue5 = 1.22474487*psitilde_a_0*scalings_y_0*psitilde_bs_0_2;
    
    // Table(s) of coefficients
    const static double coefficients0[6][6] = \
    {{0, -0.173205081, -0.1, 0.121716124, 0.0942809042, 0.0544331054},
    {0, 0.173205081, -0.1, 0.121716124, -0.0942809042, 0.0544331054},
    {0, 0, 0.2, 0, 0, 0.163299316},
    {0.471404521, 0.230940108, 0.133333333, 0, 0.188561808, -0.163299316},
    {0.471404521, -0.230940108, 0.133333333, 0, -0.188561808, -0.163299316},
    {0.471404521, 0, -0.266666667, -0.243432248, 0, 0.0544331054}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[6][6] = \
    {{0, 0, 0, 0, 0, 0},
    {4.89897949, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0},
    {0, 9.48683298, 0, 0, 0, 0},
    {4, 0, 7.07106781, 0, 0, 0},
    {0, 0, 0, 0, 0, 0}};
    
    const static double dmats1[6][6] = \
    {{0, 0, 0, 0, 0, 0},
    {2.44948974, 0, 0, 0, 0, 0},
    {4.24264069, 0, 0, 0, 0, 0},
    {2.5819889, 4.74341649, -0.912870929, 0, 0, 0},
    {2, 6.12372436, 3.53553391, 0, 0, 0},
    {-2.30940108, 0, 8.16496581, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    double coeff0_4 = 0;
    double coeff0_5 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    double new_coeff0_4 = 0;
    double new_coeff0_5 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
      new_coeff0_4 = coefficients0[dof][4];
      new_coeff0_5 = coefficients0[dof][5];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
        coeff0_4 = new_coeff0_4;
        coeff0_5 = new_coeff0_5;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3];
          new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4];
          new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3];
          new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4];
          new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    const static double X[6][1][2] = {{{0, 0}}, {{1, 0}}, {{0, 1}}, {{0.5, 0.5}}, {{0, 0.5}}, {{0.5, 0}}};
    const static double W[6][1] = {{1}, {1}, {1}, {1}, {1}, {1}};
    const static double D[6][1][1] = {{{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}};
    
    const double * const * x = c.coordinates;
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    
    // Compute affine mapping y = F(X)
    double y[2];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1];
    
    // Evaluate function at physical points
    double values[1];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Affine map: Do nothing
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 1; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new QuadratureElementBilinearForm_finite_element_2();
  }

};

/// This class defines the interface for a finite element.

class QuadratureElementBilinearForm_finite_element_3: public ufc::finite_element
{
public:

  /// Constructor
  QuadratureElementBilinearForm_finite_element_3() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~QuadratureElementBilinearForm_finite_element_3()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Quadrature element with 9 quadrature point(s) (3x3) on a triangle";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::triangle;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 9;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    throw std::runtime_error("evaluate_basis() is not supported for QuadratureElement");
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("evaluate_basis_all() is not supported for QuadratureElement.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    throw std::runtime_error("evaluate_basis_derivatives() is not supported for QuadratureElement");
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("evaluate_basis_derivatives_all() is not supported for QuadratureElement.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    const static double X[9][1][2] = {{{0.102717655, 0.0885879595}}, {{0.0665540678, 0.409466864}}, {{0.0239311323, 0.787659462}}, {{0.45570602, 0.0885879595}}, {{0.295266568, 0.409466864}}, {{0.106170269, 0.787659462}}, {{0.808694386, 0.0885879595}}, {{0.523979068, 0.409466864}}, {{0.188409406, 0.787659462}}};
    const static double W[9][1] = {{1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}};
    const static double D[9][1][1] = {{{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}};
    
    const double * const * x = c.coordinates;
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    
    // Compute affine mapping y = F(X)
    double y[2];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1];
    
    // Evaluate function at physical points
    double values[1];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Affine map: Do nothing
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 1; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    throw std::runtime_error("interpolate_vertex_values() is not supported for QuadratureElement");
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new QuadratureElementBilinearForm_finite_element_3();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class QuadratureElementBilinearForm_dof_map_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  QuadratureElementBilinearForm_dof_map_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~QuadratureElementBilinearForm_dof_map_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 2 on a triangle";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0] + m.num_entities[1];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 6;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    return 2;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    unsigned int offset = m.num_entities[0];
    dofs[3] = offset + c.entity_indices[1][0];
    dofs[4] = offset + c.entity_indices[1][1];
    dofs[5] = offset + c.entity_indices[1][2];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 4;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 5;
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[3][0] = 0.5*x[1][0] + 0.5*x[2][0];
    coordinates[3][1] = 0.5*x[1][1] + 0.5*x[2][1];
    coordinates[4][0] = 0.5*x[0][0] + 0.5*x[2][0];
    coordinates[4][1] = 0.5*x[0][1] + 0.5*x[2][1];
    coordinates[5][0] = 0.5*x[0][0] + 0.5*x[1][0];
    coordinates[5][1] = 0.5*x[0][1] + 0.5*x[1][1];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new QuadratureElementBilinearForm_dof_map_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class QuadratureElementBilinearForm_dof_map_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  QuadratureElementBilinearForm_dof_map_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~QuadratureElementBilinearForm_dof_map_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 2 on a triangle";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0] + m.num_entities[1];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 6;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    return 2;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    unsigned int offset = m.num_entities[0];
    dofs[3] = offset + c.entity_indices[1][0];
    dofs[4] = offset + c.entity_indices[1][1];
    dofs[5] = offset + c.entity_indices[1][2];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 4;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 5;
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[3][0] = 0.5*x[1][0] + 0.5*x[2][0];
    coordinates[3][1] = 0.5*x[1][1] + 0.5*x[2][1];
    coordinates[4][0] = 0.5*x[0][0] + 0.5*x[2][0];
    coordinates[4][1] = 0.5*x[0][1] + 0.5*x[2][1];
    coordinates[5][0] = 0.5*x[0][0] + 0.5*x[1][0];
    coordinates[5][1] = 0.5*x[0][1] + 0.5*x[1][1];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new QuadratureElementBilinearForm_dof_map_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class QuadratureElementBilinearForm_dof_map_2: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  QuadratureElementBilinearForm_dof_map_2() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~QuadratureElementBilinearForm_dof_map_2()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 2 on a triangle";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0] + m.num_entities[1];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 6;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    return 2;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    unsigned int offset = m.num_entities[0];
    dofs[3] = offset + c.entity_indices[1][0];
    dofs[4] = offset + c.entity_indices[1][1];
    dofs[5] = offset + c.entity_indices[1][2];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 4;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 5;
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[3][0] = 0.5*x[1][0] + 0.5*x[2][0];
    coordinates[3][1] = 0.5*x[1][1] + 0.5*x[2][1];
    coordinates[4][0] = 0.5*x[0][0] + 0.5*x[2][0];
    coordinates[4][1] = 0.5*x[0][1] + 0.5*x[2][1];
    coordinates[5][0] = 0.5*x[0][0] + 0.5*x[1][0];
    coordinates[5][1] = 0.5*x[0][1] + 0.5*x[1][1];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new QuadratureElementBilinearForm_dof_map_2();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class QuadratureElementBilinearForm_dof_map_3: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  QuadratureElementBilinearForm_dof_map_3() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~QuadratureElementBilinearForm_dof_map_3()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Quadrature element with 9 quadrature point(s) (3x3) on a triangle";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = 9*m.num_entities[2];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 9;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    return 2;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = 9*c.entity_indices[2][0];
    dofs[1] = 9*c.entity_indices[2][0] + 1;
    dofs[2] = 9*c.entity_indices[2][0] + 2;
    dofs[3] = 9*c.entity_indices[2][0] + 3;
    dofs[4] = 9*c.entity_indices[2][0] + 4;
    dofs[5] = 9*c.entity_indices[2][0] + 5;
    dofs[6] = 9*c.entity_indices[2][0] + 6;
    dofs[7] = 9*c.entity_indices[2][0] + 7;
    dofs[8] = 9*c.entity_indices[2][0] + 8;
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.808694386*x[0][0] + 0.102717655*x[1][0] + 0.0885879595*x[2][0];
    coordinates[0][1] = 0.808694386*x[0][1] + 0.102717655*x[1][1] + 0.0885879595*x[2][1];
    coordinates[1][0] = 0.523979068*x[0][0] + 0.0665540678*x[1][0] + 0.409466864*x[2][0];
    coordinates[1][1] = 0.523979068*x[0][1] + 0.0665540678*x[1][1] + 0.409466864*x[2][1];
    coordinates[2][0] = 0.188409406*x[0][0] + 0.0239311323*x[1][0] + 0.787659462*x[2][0];
    coordinates[2][1] = 0.188409406*x[0][1] + 0.0239311323*x[1][1] + 0.787659462*x[2][1];
    coordinates[3][0] = 0.45570602*x[0][0] + 0.45570602*x[1][0] + 0.0885879595*x[2][0];
    coordinates[3][1] = 0.45570602*x[0][1] + 0.45570602*x[1][1] + 0.0885879595*x[2][1];
    coordinates[4][0] = 0.295266568*x[0][0] + 0.295266568*x[1][0] + 0.409466864*x[2][0];
    coordinates[4][1] = 0.295266568*x[0][1] + 0.295266568*x[1][1] + 0.409466864*x[2][1];
    coordinates[5][0] = 0.106170269*x[0][0] + 0.106170269*x[1][0] + 0.787659462*x[2][0];
    coordinates[5][1] = 0.106170269*x[0][1] + 0.106170269*x[1][1] + 0.787659462*x[2][1];
    coordinates[6][0] = 0.102717655*x[0][0] + 0.808694386*x[1][0] + 0.0885879595*x[2][0];
    coordinates[6][1] = 0.102717655*x[0][1] + 0.808694386*x[1][1] + 0.0885879595*x[2][1];
    coordinates[7][0] = 0.0665540678*x[0][0] + 0.523979068*x[1][0] + 0.409466864*x[2][0];
    coordinates[7][1] = 0.0665540678*x[0][1] + 0.523979068*x[1][1] + 0.409466864*x[2][1];
    coordinates[8][0] = 0.0239311323*x[0][0] + 0.188409406*x[1][0] + 0.787659462*x[2][0];
    coordinates[8][1] = 0.0239311323*x[0][1] + 0.188409406*x[1][1] + 0.787659462*x[2][1];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new QuadratureElementBilinearForm_dof_map_3();
  }

};

/// This class defines the interface for the tabulation of the cell
/// tensor corresponding to the local contribution to a form from
/// the integral over a cell.

class QuadratureElementBilinearForm_cell_integral_0: public ufc::cell_integral
{
public:

  /// Constructor
  QuadratureElementBilinearForm_cell_integral_0() : ufc::cell_integral()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~QuadratureElementBilinearForm_cell_integral_0()
  {
    // Do nothing
  }

  /// Tabulate the tensor for the contribution from a local cell
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
      
    // Compute determinant of Jacobian
    double detJ = J_00*J_11 - J_01*J_10;
      
    // Compute inverse of Jacobian
    const double Jinv_00 =  J_11 / detJ;
    const double Jinv_01 = -J_01 / detJ;
    const double Jinv_10 = -J_10 / detJ;
    const double Jinv_11 =  J_00 / detJ;
    
    // Set scale factor
    const double det = std::abs(detJ);
    
    // Number of operations to compute element tensor = 8614
    // Compute coefficients
    const double c1_0_0_0 = w[1][0];
    const double c1_0_0_1 = w[1][1];
    const double c1_0_0_2 = w[1][2];
    const double c1_0_0_3 = w[1][3];
    const double c1_0_0_4 = w[1][4];
    const double c1_0_0_5 = w[1][5];
    const double c1_0_0_6 = w[1][6];
    const double c1_0_0_7 = w[1][7];
    const double c1_0_0_8 = w[1][8];
    const double c0_1_0_0 = w[0][0];
    const double c0_1_0_1 = w[0][1];
    const double c0_1_0_2 = w[0][2];
    const double c0_1_0_3 = w[0][3];
    const double c0_1_0_4 = w[0][4];
    const double c0_1_0_5 = w[0][5];
    const double c0_1_1_0 = w[0][0];
    const double c0_1_1_1 = w[0][1];
    const double c0_1_1_2 = w[0][2];
    const double c0_1_1_3 = w[0][3];
    const double c0_1_1_4 = w[0][4];
    const double c0_1_1_5 = w[0][5];
    
    // Compute geometry tensors
    // Number of operations to compute decalrations = 900
    const double G0_0_0_0 = det*c1_0_0_0*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G0_0_0_1 = det*c1_0_0_0*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G0_0_1_0 = det*c1_0_0_0*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G0_0_1_1 = det*c1_0_0_0*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G0_1_0_0 = det*c1_0_0_1*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G0_1_0_1 = det*c1_0_0_1*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G0_1_1_0 = det*c1_0_0_1*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G0_1_1_1 = det*c1_0_0_1*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G0_2_0_0 = det*c1_0_0_2*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G0_2_0_1 = det*c1_0_0_2*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G0_2_1_0 = det*c1_0_0_2*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G0_2_1_1 = det*c1_0_0_2*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G0_3_0_0 = det*c1_0_0_3*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G0_3_0_1 = det*c1_0_0_3*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G0_3_1_0 = det*c1_0_0_3*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G0_3_1_1 = det*c1_0_0_3*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G0_4_0_0 = det*c1_0_0_4*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G0_4_0_1 = det*c1_0_0_4*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G0_4_1_0 = det*c1_0_0_4*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G0_4_1_1 = det*c1_0_0_4*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G0_5_0_0 = det*c1_0_0_5*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G0_5_0_1 = det*c1_0_0_5*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G0_5_1_0 = det*c1_0_0_5*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G0_5_1_1 = det*c1_0_0_5*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G0_6_0_0 = det*c1_0_0_6*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G0_6_0_1 = det*c1_0_0_6*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G0_6_1_0 = det*c1_0_0_6*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G0_6_1_1 = det*c1_0_0_6*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G0_7_0_0 = det*c1_0_0_7*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G0_7_0_1 = det*c1_0_0_7*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G0_7_1_0 = det*c1_0_0_7*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G0_7_1_1 = det*c1_0_0_7*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G0_8_0_0 = det*c1_0_0_8*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G0_8_0_1 = det*c1_0_0_8*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G0_8_1_0 = det*c1_0_0_8*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G0_8_1_1 = det*c1_0_0_8*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_0_0_0_0 = det*c0_1_0_0*c0_1_1_0*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_0_0_0_1 = det*c0_1_0_0*c0_1_1_0*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_0_0_1_0 = det*c0_1_0_0*c0_1_1_1*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_0_0_2_1 = det*c0_1_0_0*c0_1_1_2*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_0_0_3_0 = det*c0_1_0_0*c0_1_1_3*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_0_0_3_1 = det*c0_1_0_0*c0_1_1_3*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_0_0_4_0 = det*c0_1_0_0*c0_1_1_4*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_0_0_4_1 = det*c0_1_0_0*c0_1_1_4*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_0_0_5_0 = det*c0_1_0_0*c0_1_1_5*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_0_0_5_1 = det*c0_1_0_0*c0_1_1_5*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_0_1_0_0 = det*c0_1_0_0*c0_1_1_0*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_0_1_0_1 = det*c0_1_0_0*c0_1_1_0*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_0_1_1_0 = det*c0_1_0_0*c0_1_1_1*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_0_1_2_1 = det*c0_1_0_0*c0_1_1_2*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_0_1_3_0 = det*c0_1_0_0*c0_1_1_3*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_0_1_3_1 = det*c0_1_0_0*c0_1_1_3*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_0_1_4_0 = det*c0_1_0_0*c0_1_1_4*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_0_1_4_1 = det*c0_1_0_0*c0_1_1_4*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_0_1_5_0 = det*c0_1_0_0*c0_1_1_5*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_0_1_5_1 = det*c0_1_0_0*c0_1_1_5*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_1_0_0_0 = det*c0_1_0_1*c0_1_1_0*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_1_0_0_1 = det*c0_1_0_1*c0_1_1_0*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_1_0_1_0 = det*c0_1_0_1*c0_1_1_1*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_1_0_2_1 = det*c0_1_0_1*c0_1_1_2*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_1_0_3_0 = det*c0_1_0_1*c0_1_1_3*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_1_0_3_1 = det*c0_1_0_1*c0_1_1_3*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_1_0_4_0 = det*c0_1_0_1*c0_1_1_4*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_1_0_4_1 = det*c0_1_0_1*c0_1_1_4*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_1_0_5_0 = det*c0_1_0_1*c0_1_1_5*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_1_0_5_1 = det*c0_1_0_1*c0_1_1_5*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_1_1_0_0 = det*c0_1_0_1*c0_1_1_0*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_1_1_0_1 = det*c0_1_0_1*c0_1_1_0*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_1_1_1_0 = det*c0_1_0_1*c0_1_1_1*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_1_1_2_1 = det*c0_1_0_1*c0_1_1_2*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_1_1_3_0 = det*c0_1_0_1*c0_1_1_3*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_1_1_3_1 = det*c0_1_0_1*c0_1_1_3*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_1_1_4_0 = det*c0_1_0_1*c0_1_1_4*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_1_1_4_1 = det*c0_1_0_1*c0_1_1_4*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_1_1_5_0 = det*c0_1_0_1*c0_1_1_5*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_1_1_5_1 = det*c0_1_0_1*c0_1_1_5*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_2_0_0_0 = det*c0_1_0_2*c0_1_1_0*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_2_0_0_1 = det*c0_1_0_2*c0_1_1_0*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_2_0_1_0 = det*c0_1_0_2*c0_1_1_1*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_2_0_2_1 = det*c0_1_0_2*c0_1_1_2*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_2_0_3_0 = det*c0_1_0_2*c0_1_1_3*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_2_0_3_1 = det*c0_1_0_2*c0_1_1_3*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_2_0_4_0 = det*c0_1_0_2*c0_1_1_4*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_2_0_4_1 = det*c0_1_0_2*c0_1_1_4*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_2_0_5_0 = det*c0_1_0_2*c0_1_1_5*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_2_0_5_1 = det*c0_1_0_2*c0_1_1_5*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_2_1_0_0 = det*c0_1_0_2*c0_1_1_0*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_2_1_0_1 = det*c0_1_0_2*c0_1_1_0*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_2_1_1_0 = det*c0_1_0_2*c0_1_1_1*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_2_1_2_1 = det*c0_1_0_2*c0_1_1_2*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_2_1_3_0 = det*c0_1_0_2*c0_1_1_3*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_2_1_3_1 = det*c0_1_0_2*c0_1_1_3*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_2_1_4_0 = det*c0_1_0_2*c0_1_1_4*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_2_1_4_1 = det*c0_1_0_2*c0_1_1_4*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_2_1_5_0 = det*c0_1_0_2*c0_1_1_5*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_2_1_5_1 = det*c0_1_0_2*c0_1_1_5*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_3_0_0_0 = det*c0_1_0_3*c0_1_1_0*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_3_0_0_1 = det*c0_1_0_3*c0_1_1_0*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_3_0_1_0 = det*c0_1_0_3*c0_1_1_1*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_3_0_2_1 = det*c0_1_0_3*c0_1_1_2*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_3_0_3_0 = det*c0_1_0_3*c0_1_1_3*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_3_0_3_1 = det*c0_1_0_3*c0_1_1_3*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_3_0_4_0 = det*c0_1_0_3*c0_1_1_4*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_3_0_4_1 = det*c0_1_0_3*c0_1_1_4*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_3_0_5_0 = det*c0_1_0_3*c0_1_1_5*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_3_0_5_1 = det*c0_1_0_3*c0_1_1_5*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_3_1_0_0 = det*c0_1_0_3*c0_1_1_0*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_3_1_0_1 = det*c0_1_0_3*c0_1_1_0*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_3_1_1_0 = det*c0_1_0_3*c0_1_1_1*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_3_1_2_1 = det*c0_1_0_3*c0_1_1_2*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_3_1_3_0 = det*c0_1_0_3*c0_1_1_3*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_3_1_3_1 = det*c0_1_0_3*c0_1_1_3*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_3_1_4_0 = det*c0_1_0_3*c0_1_1_4*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_3_1_4_1 = det*c0_1_0_3*c0_1_1_4*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_3_1_5_0 = det*c0_1_0_3*c0_1_1_5*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_3_1_5_1 = det*c0_1_0_3*c0_1_1_5*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_4_0_0_0 = det*c0_1_0_4*c0_1_1_0*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_4_0_0_1 = det*c0_1_0_4*c0_1_1_0*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_4_0_1_0 = det*c0_1_0_4*c0_1_1_1*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_4_0_2_1 = det*c0_1_0_4*c0_1_1_2*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_4_0_3_0 = det*c0_1_0_4*c0_1_1_3*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_4_0_3_1 = det*c0_1_0_4*c0_1_1_3*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_4_0_4_0 = det*c0_1_0_4*c0_1_1_4*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_4_0_4_1 = det*c0_1_0_4*c0_1_1_4*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_4_0_5_0 = det*c0_1_0_4*c0_1_1_5*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_4_0_5_1 = det*c0_1_0_4*c0_1_1_5*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_4_1_0_0 = det*c0_1_0_4*c0_1_1_0*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_4_1_0_1 = det*c0_1_0_4*c0_1_1_0*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_4_1_1_0 = det*c0_1_0_4*c0_1_1_1*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_4_1_2_1 = det*c0_1_0_4*c0_1_1_2*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_4_1_3_0 = det*c0_1_0_4*c0_1_1_3*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_4_1_3_1 = det*c0_1_0_4*c0_1_1_3*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_4_1_4_0 = det*c0_1_0_4*c0_1_1_4*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_4_1_4_1 = det*c0_1_0_4*c0_1_1_4*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_4_1_5_0 = det*c0_1_0_4*c0_1_1_5*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_4_1_5_1 = det*c0_1_0_4*c0_1_1_5*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_5_0_0_0 = det*c0_1_0_5*c0_1_1_0*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_5_0_0_1 = det*c0_1_0_5*c0_1_1_0*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_5_0_1_0 = det*c0_1_0_5*c0_1_1_1*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_5_0_2_1 = det*c0_1_0_5*c0_1_1_2*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_5_0_3_0 = det*c0_1_0_5*c0_1_1_3*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_5_0_3_1 = det*c0_1_0_5*c0_1_1_3*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_5_0_4_0 = det*c0_1_0_5*c0_1_1_4*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_5_0_4_1 = det*c0_1_0_5*c0_1_1_4*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_5_0_5_0 = det*c0_1_0_5*c0_1_1_5*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G1_5_0_5_1 = det*c0_1_0_5*c0_1_1_5*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G1_5_1_0_0 = det*c0_1_0_5*c0_1_1_0*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_5_1_0_1 = det*c0_1_0_5*c0_1_1_0*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_5_1_1_0 = det*c0_1_0_5*c0_1_1_1*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_5_1_2_1 = det*c0_1_0_5*c0_1_1_2*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_5_1_3_0 = det*c0_1_0_5*c0_1_1_3*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_5_1_3_1 = det*c0_1_0_5*c0_1_1_3*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_5_1_4_0 = det*c0_1_0_5*c0_1_1_4*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_5_1_4_1 = det*c0_1_0_5*c0_1_1_4*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    const double G1_5_1_5_0 = det*c0_1_0_5*c0_1_1_5*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G1_5_1_5_1 = det*c0_1_0_5*c0_1_1_5*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    
    // Compute element tensor
    // Number of operations to compute tensor = 7714
    A[0] = 0.27875009*G0_0_0_0 + 0.27875009*G0_0_0_1 + 0.27875009*G0_0_1_0 + 0.27875009*G0_0_1_1 + 0.076479448*G0_1_0_0 + 0.076479448*G0_1_0_1 + 0.076479448*G0_1_1_0 + 0.076479448*G0_1_1_1 + 0.00117725224*G0_2_0_0 + 0.00117725224*G0_2_0_1 + 0.00117725224*G0_2_1_0 + 0.00117725224*G0_2_1_1 + 0.0604617049*G0_3_0_0 + 0.0604617049*G0_3_0_1 + 0.0604617049*G0_3_1_0 + 0.0604617049*G0_3_1_1 + 0.00334029708*G0_4_0_0 + 0.00334029708*G0_4_0_1 + 0.00334029708*G0_4_1_0 + 0.00334029708*G0_4_1_1 + 0.0102720721*G0_5_0_0 + 0.0102720721*G0_5_0_1 + 0.0102720721*G0_5_1_0 + 0.0102720721*G0_5_1_1 + 0.0193717022*G0_6_0_0 + 0.0193717022*G0_6_0_1 + 0.0193717022*G0_6_1_0 + 0.0193717022*G0_6_1_1 + 0.0342867365*G0_7_0_0 + 0.0342867365*G0_7_0_1 + 0.0342867365*G0_7_1_0 + 0.0342867365*G0_7_1_1 + 0.0158606966*G0_8_0_0 + 0.0158606966*G0_8_0_1 + 0.0158606966*G0_8_1_0 + 0.0158606966*G0_8_1_1 + 0.147619048*G1_0_0_0_0 + 0.147619048*G1_0_0_0_1 + 0.0428571429*G1_0_0_1_0 + 0.0428571429*G1_0_0_2_1 - 0.019047619*G1_0_0_3_0 - 0.019047619*G1_0_0_3_1 + 0.019047619*G1_0_0_4_0 - 0.19047619*G1_0_0_4_1 - 0.19047619*G1_0_0_5_0 + 0.019047619*G1_0_0_5_1 + 0.147619048*G1_0_1_0_0 + 0.147619048*G1_0_1_0_1 + 0.0428571429*G1_0_1_1_0 + 0.0428571429*G1_0_1_2_1 - 0.019047619*G1_0_1_3_0 - 0.019047619*G1_0_1_3_1 + 0.019047619*G1_0_1_4_0 - 0.19047619*G1_0_1_4_1 - 0.19047619*G1_0_1_5_0 + 0.019047619*G1_0_1_5_1 - 0.0182539683*G1_1_0_0_0 - 0.0182539683*G1_1_0_0_1 - 0.00714285714*G1_1_0_1_0 - 0.00396825397*G1_1_0_2_1 + 0.00317460317*G1_1_0_3_0 - 0.00317460317*G1_1_0_4_0 + 0.0222222222*G1_1_0_4_1 + 0.0253968254*G1_1_0_5_0 - 0.0182539683*G1_1_1_0_0 - 0.0182539683*G1_1_1_0_1 - 0.00714285714*G1_1_1_1_0 - 0.00396825397*G1_1_1_2_1 + 0.00317460317*G1_1_1_3_0 - 0.00317460317*G1_1_1_4_0 + 0.0222222222*G1_1_1_4_1 + 0.0253968254*G1_1_1_5_0 - 0.0182539683*G1_2_0_0_0 - 0.0182539683*G1_2_0_0_1 - 0.00396825397*G1_2_0_1_0 - 0.00714285714*G1_2_0_2_1 + 0.00317460317*G1_2_0_3_1 + 0.0253968254*G1_2_0_4_1 + 0.0222222222*G1_2_0_5_0 - 0.00317460317*G1_2_0_5_1 - 0.0182539683*G1_2_1_0_0 - 0.0182539683*G1_2_1_0_1 - 0.00396825397*G1_2_1_1_0 - 0.00714285714*G1_2_1_2_1 + 0.00317460317*G1_2_1_3_1 + 0.0253968254*G1_2_1_4_1 + 0.0222222222*G1_2_1_5_0 - 0.00317460317*G1_2_1_5_1 + 0.00317460317*G1_3_0_0_0 + 0.00317460317*G1_3_0_0_1 - 0.00317460317*G1_3_0_1_0 - 0.00317460317*G1_3_0_2_1 - 0.0126984127*G1_3_0_3_0 - 0.0126984127*G1_3_0_3_1 + 0.0126984127*G1_3_0_4_0 + 0.0126984127*G1_3_0_5_1 + 0.00317460317*G1_3_1_0_0 + 0.00317460317*G1_3_1_0_1 - 0.00317460317*G1_3_1_1_0 - 0.00317460317*G1_3_1_2_1 - 0.0126984127*G1_3_1_3_0 - 0.0126984127*G1_3_1_3_1 + 0.0126984127*G1_3_1_4_0 + 0.0126984127*G1_3_1_5_1 + 0.0761904762*G1_4_0_0_0 + 0.0761904762*G1_4_0_0_1 + 0.0253968254*G1_4_0_1_0 + 0.0126984127*G1_4_0_2_1 - 0.0253968254*G1_4_0_3_0 - 0.0126984127*G1_4_0_3_1 + 0.0253968254*G1_4_0_4_0 - 0.0888888889*G1_4_0_4_1 - 0.101587302*G1_4_0_5_0 + 0.0126984127*G1_4_0_5_1 + 0.0761904762*G1_4_1_0_0 + 0.0761904762*G1_4_1_0_1 + 0.0253968254*G1_4_1_1_0 + 0.0126984127*G1_4_1_2_1 - 0.0253968254*G1_4_1_3_0 - 0.0126984127*G1_4_1_3_1 + 0.0253968254*G1_4_1_4_0 - 0.0888888889*G1_4_1_4_1 - 0.101587302*G1_4_1_5_0 + 0.0126984127*G1_4_1_5_1 + 0.0761904762*G1_5_0_0_0 + 0.0761904762*G1_5_0_0_1 + 0.0126984127*G1_5_0_1_0 + 0.0253968254*G1_5_0_2_1 - 0.0126984127*G1_5_0_3_0 - 0.0253968254*G1_5_0_3_1 + 0.0126984127*G1_5_0_4_0 - 0.101587302*G1_5_0_4_1 - 0.0888888889*G1_5_0_5_0 + 0.0253968254*G1_5_0_5_1 + 0.0761904762*G1_5_1_0_0 + 0.0761904762*G1_5_1_0_1 + 0.0126984127*G1_5_1_1_0 + 0.0253968254*G1_5_1_2_1 - 0.0126984127*G1_5_1_3_0 - 0.0253968254*G1_5_1_3_1 + 0.0126984127*G1_5_1_4_0 - 0.101587302*G1_5_1_4_1 - 0.0888888889*G1_5_1_5_0 + 0.0253968254*G1_5_1_5_1;
    A[1] = 0.0734837652*G0_0_0_0 + 0.0734837652*G0_0_1_0 + 0.0512077209*G0_1_0_0 + 0.0512077209*G0_1_1_0 - 0.00432111568*G0_2_0_0 - 0.00432111568*G0_2_1_0 - 0.0604617049*G0_3_0_0 - 0.0604617049*G0_3_1_0 - 0.00334029708*G0_4_0_0 - 0.00334029708*G0_4_1_0 - 0.0102720721*G0_5_0_0 - 0.0102720721*G0_5_1_0 + 0.0734837652*G0_6_0_0 + 0.0734837652*G0_6_1_0 + 0.0512077209*G0_7_0_0 + 0.0512077209*G0_7_1_0 - 0.00432111568*G0_8_0_0 - 0.00432111568*G0_8_1_0 - 0.0182539683*G1_0_0_0_0 - 0.0182539683*G1_0_0_0_1 - 0.00714285714*G1_0_0_1_0 - 0.00396825397*G1_0_0_2_1 + 0.00317460317*G1_0_0_3_0 - 0.00317460317*G1_0_0_4_0 + 0.0222222222*G1_0_0_4_1 + 0.0253968254*G1_0_0_5_0 - 0.0182539683*G1_0_1_0_0 - 0.0182539683*G1_0_1_0_1 - 0.00714285714*G1_0_1_1_0 - 0.00396825397*G1_0_1_2_1 + 0.00317460317*G1_0_1_3_0 - 0.00317460317*G1_0_1_4_0 + 0.0222222222*G1_0_1_4_1 + 0.0253968254*G1_0_1_5_0 + 0.0206349206*G1_1_0_0_0 + 0.0206349206*G1_1_0_0_1 + 0.0428571429*G1_1_0_1_0 - 0.00793650794*G1_1_0_2_1 + 0.00634920635*G1_1_0_3_0 + 0.0571428571*G1_1_0_3_1 - 0.00634920635*G1_1_0_4_0 - 0.0126984127*G1_1_0_4_1 - 0.0634920635*G1_1_0_5_0 - 0.0571428571*G1_1_0_5_1 + 0.0206349206*G1_1_1_0_0 + 0.0206349206*G1_1_1_0_1 + 0.0428571429*G1_1_1_1_0 - 0.00793650794*G1_1_1_2_1 + 0.00634920635*G1_1_1_3_0 + 0.0571428571*G1_1_1_3_1 - 0.00634920635*G1_1_1_4_0 - 0.0126984127*G1_1_1_4_1 - 0.0634920635*G1_1_1_5_0 - 0.0571428571*G1_1_1_5_1 + 0.000793650794*G1_2_0_0_0 + 0.000793650794*G1_2_0_0_1 - 0.00396825397*G1_2_0_1_0 - 0.00396825397*G1_2_0_2_1 - 0.0126984127*G1_2_0_3_0 - 0.0126984127*G1_2_0_3_1 + 0.0126984127*G1_2_0_4_0 + 0.00317460317*G1_2_0_4_1 + 0.00317460317*G1_2_0_5_0 + 0.0126984127*G1_2_0_5_1 + 0.000793650794*G1_2_1_0_0 + 0.000793650794*G1_2_1_0_1 - 0.00396825397*G1_2_1_1_0 - 0.00396825397*G1_2_1_2_1 - 0.0126984127*G1_2_1_3_0 - 0.0126984127*G1_2_1_3_1 + 0.0126984127*G1_2_1_4_0 + 0.00317460317*G1_2_1_4_1 + 0.00317460317*G1_2_1_5_0 + 0.0126984127*G1_2_1_5_1 + 0.0253968254*G1_3_0_1_0 - 0.0126984127*G1_3_0_2_1 + 0.0380952381*G1_3_0_3_1 + 0.0126984127*G1_3_0_4_1 - 0.0253968254*G1_3_0_5_0 - 0.0380952381*G1_3_0_5_1 + 0.0253968254*G1_3_1_1_0 - 0.0126984127*G1_3_1_2_1 + 0.0380952381*G1_3_1_3_1 + 0.0126984127*G1_3_1_4_1 - 0.0253968254*G1_3_1_5_0 - 0.0380952381*G1_3_1_5_1 - 0.0222222222*G1_4_0_0_0 - 0.0222222222*G1_4_0_0_1 - 0.00317460317*G1_4_0_1_0 - 0.00317460317*G1_4_0_2_1 + 0.0126984127*G1_4_0_3_0 + 0.0126984127*G1_4_0_3_1 - 0.0126984127*G1_4_0_4_0 + 0.0253968254*G1_4_0_4_1 + 0.0253968254*G1_4_0_5_0 - 0.0126984127*G1_4_0_5_1 - 0.0222222222*G1_4_1_0_0 - 0.0222222222*G1_4_1_0_1 - 0.00317460317*G1_4_1_1_0 - 0.00317460317*G1_4_1_2_1 + 0.0126984127*G1_4_1_3_0 + 0.0126984127*G1_4_1_3_1 - 0.0126984127*G1_4_1_4_0 + 0.0253968254*G1_4_1_4_1 + 0.0253968254*G1_4_1_5_0 - 0.0126984127*G1_4_1_5_1 - 0.0253968254*G1_5_0_0_0 - 0.0253968254*G1_5_0_0_1 + 0.0126984127*G1_5_0_1_0 - 0.0126984127*G1_5_0_2_1 + 0.0126984127*G1_5_0_3_0 + 0.0380952381*G1_5_0_3_1 - 0.0126984127*G1_5_0_4_0 + 0.0380952381*G1_5_0_4_1 + 0.0126984127*G1_5_0_5_0 - 0.0380952381*G1_5_0_5_1 - 0.0253968254*G1_5_1_0_0 - 0.0253968254*G1_5_1_0_1 + 0.0126984127*G1_5_1_1_0 - 0.0126984127*G1_5_1_2_1 + 0.0126984127*G1_5_1_3_0 + 0.0380952381*G1_5_1_3_1 - 0.0126984127*G1_5_1_4_0 + 0.0380952381*G1_5_1_4_1 + 0.0126984127*G1_5_1_5_0 - 0.0380952381*G1_5_1_5_1;
    A[2] = 0.0805335117*G0_0_0_1 + 0.0805335117*G0_0_1_1 - 0.0445141224*G0_1_0_1 - 0.0445141224*G0_1_1_1 + 0.010276907*G0_2_0_1 + 0.010276907*G0_2_1_1 + 0.047442691*G0_3_0_1 + 0.047442691*G0_3_1_1 - 0.0117673313*G0_4_0_1 - 0.0117673313*G0_4_1_1 + 0.0383987144*G0_5_0_1 + 0.0383987144*G0_5_1_1 - 0.021230148*G0_6_0_1 - 0.021230148*G0_6_1_1 + 0.0298049583*G0_7_0_1 + 0.0298049583*G0_7_1_1 + 0.037721486*G0_8_0_1 + 0.037721486*G0_8_1_1 - 0.0182539683*G1_0_0_0_0 - 0.0182539683*G1_0_0_0_1 - 0.00396825397*G1_0_0_1_0 - 0.00714285714*G1_0_0_2_1 + 0.00317460317*G1_0_0_3_1 + 0.0253968254*G1_0_0_4_1 + 0.0222222222*G1_0_0_5_0 - 0.00317460317*G1_0_0_5_1 - 0.0182539683*G1_0_1_0_0 - 0.0182539683*G1_0_1_0_1 - 0.00396825397*G1_0_1_1_0 - 0.00714285714*G1_0_1_2_1 + 0.00317460317*G1_0_1_3_1 + 0.0253968254*G1_0_1_4_1 + 0.0222222222*G1_0_1_5_0 - 0.00317460317*G1_0_1_5_1 + 0.000793650794*G1_1_0_0_0 + 0.000793650794*G1_1_0_0_1 - 0.00396825397*G1_1_0_1_0 - 0.00396825397*G1_1_0_2_1 - 0.0126984127*G1_1_0_3_0 - 0.0126984127*G1_1_0_3_1 + 0.0126984127*G1_1_0_4_0 + 0.00317460317*G1_1_0_4_1 + 0.00317460317*G1_1_0_5_0 + 0.0126984127*G1_1_0_5_1 + 0.000793650794*G1_1_1_0_0 + 0.000793650794*G1_1_1_0_1 - 0.00396825397*G1_1_1_1_0 - 0.00396825397*G1_1_1_2_1 - 0.0126984127*G1_1_1_3_0 - 0.0126984127*G1_1_1_3_1 + 0.0126984127*G1_1_1_4_0 + 0.00317460317*G1_1_1_4_1 + 0.00317460317*G1_1_1_5_0 + 0.0126984127*G1_1_1_5_1 + 0.0206349206*G1_2_0_0_0 + 0.0206349206*G1_2_0_0_1 - 0.00793650794*G1_2_0_1_0 + 0.0428571429*G1_2_0_2_1 + 0.0571428571*G1_2_0_3_0 + 0.00634920635*G1_2_0_3_1 - 0.0571428571*G1_2_0_4_0 - 0.0634920635*G1_2_0_4_1 - 0.0126984127*G1_2_0_5_0 - 0.00634920635*G1_2_0_5_1 + 0.0206349206*G1_2_1_0_0 + 0.0206349206*G1_2_1_0_1 - 0.00793650794*G1_2_1_1_0 + 0.0428571429*G1_2_1_2_1 + 0.0571428571*G1_2_1_3_0 + 0.00634920635*G1_2_1_3_1 - 0.0571428571*G1_2_1_4_0 - 0.0634920635*G1_2_1_4_1 - 0.0126984127*G1_2_1_5_0 - 0.00634920635*G1_2_1_5_1 - 0.0126984127*G1_3_0_1_0 + 0.0253968254*G1_3_0_2_1 + 0.0380952381*G1_3_0_3_0 - 0.0380952381*G1_3_0_4_0 - 0.0253968254*G1_3_0_4_1 + 0.0126984127*G1_3_0_5_0 - 0.0126984127*G1_3_1_1_0 + 0.0253968254*G1_3_1_2_1 + 0.0380952381*G1_3_1_3_0 - 0.0380952381*G1_3_1_4_0 - 0.0253968254*G1_3_1_4_1 + 0.0126984127*G1_3_1_5_0 - 0.0253968254*G1_4_0_0_0 - 0.0253968254*G1_4_0_0_1 - 0.0126984127*G1_4_0_1_0 + 0.0126984127*G1_4_0_2_1 + 0.0380952381*G1_4_0_3_0 + 0.0126984127*G1_4_0_3_1 - 0.0380952381*G1_4_0_4_0 + 0.0126984127*G1_4_0_4_1 + 0.0380952381*G1_4_0_5_0 - 0.0126984127*G1_4_0_5_1 - 0.0253968254*G1_4_1_0_0 - 0.0253968254*G1_4_1_0_1 - 0.0126984127*G1_4_1_1_0 + 0.0126984127*G1_4_1_2_1 + 0.0380952381*G1_4_1_3_0 + 0.0126984127*G1_4_1_3_1 - 0.0380952381*G1_4_1_4_0 + 0.0126984127*G1_4_1_4_1 + 0.0380952381*G1_4_1_5_0 - 0.0126984127*G1_4_1_5_1 - 0.0222222222*G1_5_0_0_0 - 0.0222222222*G1_5_0_0_1 - 0.00317460317*G1_5_0_1_0 - 0.00317460317*G1_5_0_2_1 + 0.0126984127*G1_5_0_3_0 + 0.0126984127*G1_5_0_3_1 - 0.0126984127*G1_5_0_4_0 + 0.0253968254*G1_5_0_4_1 + 0.0253968254*G1_5_0_5_0 - 0.0126984127*G1_5_0_5_1 - 0.0222222222*G1_5_1_0_0 - 0.0222222222*G1_5_1_0_1 - 0.00317460317*G1_5_1_1_0 - 0.00317460317*G1_5_1_2_1 + 0.0126984127*G1_5_1_3_0 + 0.0126984127*G1_5_1_3_1 - 0.0126984127*G1_5_1_4_0 + 0.0253968254*G1_5_1_4_1 + 0.0253968254*G1_5_1_5_0 - 0.0126984127*G1_5_1_5_1;
    A[3] = -0.0441993017*G0_0_0_0 - 0.0512490483*G0_0_0_1 - 0.0441993017*G0_0_1_0 - 0.0512490483*G0_0_1_1 - 0.114299972*G0_1_0_0 - 0.0185781287*G0_1_0_1 - 0.114299972*G0_1_1_0 - 0.0185781287*G0_1_1_1 + 0.0150554461*G0_2_0_0 + 0.000457423404*G0_2_0_1 + 0.0150554461*G0_2_1_0 + 0.000457423404*G0_2_1_1 - 0.0260380278*G0_3_0_0 - 0.133942424*G0_3_0_1 - 0.0260380278*G0_3_1_0 - 0.133942424*G0_3_1_1 - 0.0302152568*G0_4_0_0 - 0.0217882225*G0_4_0_1 - 0.0302152568*G0_4_1_0 - 0.0217882225*G0_4_1_1 + 0.0562532846*G0_5_0_0 + 0.00758249808*G0_5_0_1 + 0.0562532846*G0_5_1_0 + 0.00758249808*G0_5_1_1 + 0.011651767*G0_6_0_0 + 0.10636568*G0_6_0_1 + 0.011651767*G0_6_1_0 + 0.10636568*G0_6_1_1 + 0.076530901*G0_7_0_0 + 0.0979336636*G0_7_0_1 + 0.076530901*G0_7_1_0 + 0.0979336636*G0_7_1_1 + 0.0552611596*G0_8_0_0 + 0.013218558*G0_8_0_1 + 0.0552611596*G0_8_1_0 + 0.013218558*G0_8_1_1 + 0.00317460317*G1_0_0_0_0 + 0.00317460317*G1_0_0_0_1 - 0.00317460317*G1_0_0_1_0 - 0.00317460317*G1_0_0_2_1 - 0.0126984127*G1_0_0_3_0 - 0.0126984127*G1_0_0_3_1 + 0.0126984127*G1_0_0_4_0 + 0.0126984127*G1_0_0_5_1 + 0.00317460317*G1_0_1_0_0 + 0.00317460317*G1_0_1_0_1 - 0.00317460317*G1_0_1_1_0 - 0.00317460317*G1_0_1_2_1 - 0.0126984127*G1_0_1_3_0 - 0.0126984127*G1_0_1_3_1 + 0.0126984127*G1_0_1_4_0 + 0.0126984127*G1_0_1_5_1 + 0.0253968254*G1_1_0_1_0 - 0.0126984127*G1_1_0_2_1 + 0.0380952381*G1_1_0_3_1 + 0.0126984127*G1_1_0_4_1 - 0.0253968254*G1_1_0_5_0 - 0.0380952381*G1_1_0_5_1 + 0.0253968254*G1_1_1_1_0 - 0.0126984127*G1_1_1_2_1 + 0.0380952381*G1_1_1_3_1 + 0.0126984127*G1_1_1_4_1 - 0.0253968254*G1_1_1_5_0 - 0.0380952381*G1_1_1_5_1 - 0.0126984127*G1_2_0_1_0 + 0.0253968254*G1_2_0_2_1 + 0.0380952381*G1_2_0_3_0 - 0.0380952381*G1_2_0_4_0 - 0.0253968254*G1_2_0_4_1 + 0.0126984127*G1_2_0_5_0 - 0.0126984127*G1_2_1_1_0 + 0.0253968254*G1_2_1_2_1 + 0.0380952381*G1_2_1_3_0 - 0.0380952381*G1_2_1_4_0 - 0.0253968254*G1_2_1_4_1 + 0.0126984127*G1_2_1_5_0 + 0.0761904762*G1_3_0_0_0 + 0.0761904762*G1_3_0_0_1 + 0.0761904762*G1_3_0_1_0 + 0.0761904762*G1_3_0_2_1 + 0.152380952*G1_3_0_3_0 + 0.152380952*G1_3_0_3_1 - 0.152380952*G1_3_0_4_0 - 0.152380952*G1_3_0_4_1 - 0.152380952*G1_3_0_5_0 - 0.152380952*G1_3_0_5_1 + 0.0761904762*G1_3_1_0_0 + 0.0761904762*G1_3_1_0_1 + 0.0761904762*G1_3_1_1_0 + 0.0761904762*G1_3_1_2_1 + 0.152380952*G1_3_1_3_0 + 0.152380952*G1_3_1_3_1 - 0.152380952*G1_3_1_4_0 - 0.152380952*G1_3_1_4_1 - 0.152380952*G1_3_1_5_0 - 0.152380952*G1_3_1_5_1 + 0.0380952381*G1_4_0_0_0 + 0.0380952381*G1_4_0_0_1 + 0.0126984127*G1_4_0_1_0 + 0.0126984127*G1_4_0_2_1 - 0.0507936508*G1_4_0_4_1 - 0.0507936508*G1_4_0_5_0 + 0.0380952381*G1_4_1_0_0 + 0.0380952381*G1_4_1_0_1 + 0.0126984127*G1_4_1_1_0 + 0.0126984127*G1_4_1_2_1 - 0.0507936508*G1_4_1_4_1 - 0.0507936508*G1_4_1_5_0 + 0.0380952381*G1_5_0_0_0 + 0.0380952381*G1_5_0_0_1 + 0.0126984127*G1_5_0_1_0 + 0.0126984127*G1_5_0_2_1 - 0.0507936508*G1_5_0_4_1 - 0.0507936508*G1_5_0_5_0 + 0.0380952381*G1_5_1_0_0 + 0.0380952381*G1_5_1_0_1 + 0.0126984127*G1_5_1_1_0 + 0.0126984127*G1_5_1_2_1 - 0.0507936508*G1_5_1_4_1 - 0.0507936508*G1_5_1_5_0;
    A[4] = 0.0441993017*G0_0_0_0 - 0.359283602*G0_0_0_1 + 0.0441993017*G0_0_1_0 - 0.359283602*G0_0_1_1 + 0.114299972*G0_1_0_0 - 0.0319653256*G0_1_0_1 + 0.114299972*G0_1_1_0 - 0.0319653256*G0_1_1_1 - 0.0150554461*G0_2_0_0 - 0.0114541592*G0_2_0_1 - 0.0150554461*G0_2_1_0 - 0.0114541592*G0_2_1_1 + 0.0260380278*G0_3_0_0 - 0.107904396*G0_3_0_1 + 0.0260380278*G0_3_1_0 - 0.107904396*G0_3_1_1 + 0.0302152568*G0_4_0_0 + 0.00842703425*G0_4_0_1 + 0.0302152568*G0_4_1_0 + 0.00842703425*G0_4_1_1 - 0.0562532846*G0_5_0_0 - 0.0486707865*G0_5_0_1 - 0.0562532846*G0_5_1_0 - 0.0486707865*G0_5_1_1 - 0.011651767*G0_6_0_0 + 0.00185844576*G0_6_0_1 - 0.011651767*G0_6_1_0 + 0.00185844576*G0_6_1_1 - 0.076530901*G0_7_0_0 - 0.0640916948*G0_7_0_1 - 0.076530901*G0_7_1_0 - 0.0640916948*G0_7_1_1 - 0.0552611596*G0_8_0_0 - 0.0535821826*G0_8_0_1 - 0.0552611596*G0_8_1_0 - 0.0535821826*G0_8_1_1 + 0.0761904762*G1_0_0_0_0 + 0.0761904762*G1_0_0_0_1 + 0.0253968254*G1_0_0_1_0 + 0.0126984127*G1_0_0_2_1 - 0.0253968254*G1_0_0_3_0 - 0.0126984127*G1_0_0_3_1 + 0.0253968254*G1_0_0_4_0 - 0.0888888889*G1_0_0_4_1 - 0.101587302*G1_0_0_5_0 + 0.0126984127*G1_0_0_5_1 + 0.0761904762*G1_0_1_0_0 + 0.0761904762*G1_0_1_0_1 + 0.0253968254*G1_0_1_1_0 + 0.0126984127*G1_0_1_2_1 - 0.0253968254*G1_0_1_3_0 - 0.0126984127*G1_0_1_3_1 + 0.0253968254*G1_0_1_4_0 - 0.0888888889*G1_0_1_4_1 - 0.101587302*G1_0_1_5_0 + 0.0126984127*G1_0_1_5_1 - 0.0222222222*G1_1_0_0_0 - 0.0222222222*G1_1_0_0_1 - 0.00317460317*G1_1_0_1_0 - 0.00317460317*G1_1_0_2_1 + 0.0126984127*G1_1_0_3_0 + 0.0126984127*G1_1_0_3_1 - 0.0126984127*G1_1_0_4_0 + 0.0253968254*G1_1_0_4_1 + 0.0253968254*G1_1_0_5_0 - 0.0126984127*G1_1_0_5_1 - 0.0222222222*G1_1_1_0_0 - 0.0222222222*G1_1_1_0_1 - 0.00317460317*G1_1_1_1_0 - 0.00317460317*G1_1_1_2_1 + 0.0126984127*G1_1_1_3_0 + 0.0126984127*G1_1_1_3_1 - 0.0126984127*G1_1_1_4_0 + 0.0253968254*G1_1_1_4_1 + 0.0253968254*G1_1_1_5_0 - 0.0126984127*G1_1_1_5_1 - 0.0253968254*G1_2_0_0_0 - 0.0253968254*G1_2_0_0_1 - 0.0126984127*G1_2_0_1_0 + 0.0126984127*G1_2_0_2_1 + 0.0380952381*G1_2_0_3_0 + 0.0126984127*G1_2_0_3_1 - 0.0380952381*G1_2_0_4_0 + 0.0126984127*G1_2_0_4_1 + 0.0380952381*G1_2_0_5_0 - 0.0126984127*G1_2_0_5_1 - 0.0253968254*G1_2_1_0_0 - 0.0253968254*G1_2_1_0_1 - 0.0126984127*G1_2_1_1_0 + 0.0126984127*G1_2_1_2_1 + 0.0380952381*G1_2_1_3_0 + 0.0126984127*G1_2_1_3_1 - 0.0380952381*G1_2_1_4_0 + 0.0126984127*G1_2_1_4_1 + 0.0380952381*G1_2_1_5_0 - 0.0126984127*G1_2_1_5_1 + 0.0380952381*G1_3_0_0_0 + 0.0380952381*G1_3_0_0_1 + 0.0126984127*G1_3_0_1_0 + 0.0126984127*G1_3_0_2_1 - 0.0507936508*G1_3_0_4_1 - 0.0507936508*G1_3_0_5_0 + 0.0380952381*G1_3_1_0_0 + 0.0380952381*G1_3_1_0_1 + 0.0126984127*G1_3_1_1_0 + 0.0126984127*G1_3_1_2_1 - 0.0507936508*G1_3_1_4_1 - 0.0507936508*G1_3_1_5_0 + 0.177777778*G1_4_0_0_0 + 0.177777778*G1_4_0_0_1 + 0.0761904762*G1_4_0_1_0 - 0.0253968254*G1_4_0_2_1 - 0.152380952*G1_4_0_3_0 - 0.0507936508*G1_4_0_3_1 + 0.152380952*G1_4_0_4_0 - 0.152380952*G1_4_0_4_1 - 0.253968254*G1_4_0_5_0 + 0.0507936508*G1_4_0_5_1 + 0.177777778*G1_4_1_0_0 + 0.177777778*G1_4_1_0_1 + 0.0761904762*G1_4_1_1_0 - 0.0253968254*G1_4_1_2_1 - 0.152380952*G1_4_1_3_0 - 0.0507936508*G1_4_1_3_1 + 0.152380952*G1_4_1_4_0 - 0.152380952*G1_4_1_4_1 - 0.253968254*G1_4_1_5_0 + 0.0507936508*G1_4_1_5_1 + 0.0888888889*G1_5_0_0_0 + 0.0888888889*G1_5_0_0_1 + 0.0126984127*G1_5_0_1_0 + 0.0126984127*G1_5_0_2_1 - 0.0507936508*G1_5_0_3_0 - 0.0507936508*G1_5_0_3_1 + 0.0507936508*G1_5_0_4_0 - 0.101587302*G1_5_0_4_1 - 0.101587302*G1_5_0_5_0 + 0.0507936508*G1_5_0_5_1 + 0.0888888889*G1_5_1_0_0 + 0.0888888889*G1_5_1_0_1 + 0.0126984127*G1_5_1_1_0 + 0.0126984127*G1_5_1_2_1 - 0.0507936508*G1_5_1_3_0 - 0.0507936508*G1_5_1_3_1 + 0.0507936508*G1_5_1_4_0 - 0.101587302*G1_5_1_4_1 - 0.101587302*G1_5_1_5_0 + 0.0507936508*G1_5_1_5_1;
    A[5] = -0.352233855*G0_0_0_0 + 0.0512490483*G0_0_0_1 - 0.352233855*G0_0_1_0 + 0.0512490483*G0_0_1_1 - 0.127687169*G0_1_0_0 + 0.0185781287*G0_1_0_1 - 0.127687169*G0_1_1_0 + 0.0185781287*G0_1_1_1 + 0.00314386343*G0_2_0_0 - 0.000457423404*G0_2_0_1 + 0.00314386343*G0_2_1_0 - 0.000457423404*G0_2_1_1 + 0.133942424*G0_3_0_1 + 0.133942424*G0_3_1_1 + 0.0217882225*G0_4_0_1 + 0.0217882225*G0_4_1_1 - 0.00758249808*G0_5_0_1 - 0.00758249808*G0_5_1_1 - 0.0928554674*G0_6_0_0 - 0.10636568*G0_6_0_1 - 0.0928554674*G0_6_1_0 - 0.10636568*G0_6_1_1 - 0.0854944574*G0_7_0_0 - 0.0979336636*G0_7_0_1 - 0.0854944574*G0_7_1_0 - 0.0979336636*G0_7_1_1 - 0.011539581*G0_8_0_0 - 0.013218558*G0_8_0_1 - 0.011539581*G0_8_1_0 - 0.013218558*G0_8_1_1 + 0.0761904762*G1_0_0_0_0 + 0.0761904762*G1_0_0_0_1 + 0.0126984127*G1_0_0_1_0 + 0.0253968254*G1_0_0_2_1 - 0.0126984127*G1_0_0_3_0 - 0.0253968254*G1_0_0_3_1 + 0.0126984127*G1_0_0_4_0 - 0.101587302*G1_0_0_4_1 - 0.0888888889*G1_0_0_5_0 + 0.0253968254*G1_0_0_5_1 + 0.0761904762*G1_0_1_0_0 + 0.0761904762*G1_0_1_0_1 + 0.0126984127*G1_0_1_1_0 + 0.0253968254*G1_0_1_2_1 - 0.0126984127*G1_0_1_3_0 - 0.0253968254*G1_0_1_3_1 + 0.0126984127*G1_0_1_4_0 - 0.101587302*G1_0_1_4_1 - 0.0888888889*G1_0_1_5_0 + 0.0253968254*G1_0_1_5_1 - 0.0253968254*G1_1_0_0_0 - 0.0253968254*G1_1_0_0_1 + 0.0126984127*G1_1_0_1_0 - 0.0126984127*G1_1_0_2_1 + 0.0126984127*G1_1_0_3_0 + 0.0380952381*G1_1_0_3_1 - 0.0126984127*G1_1_0_4_0 + 0.0380952381*G1_1_0_4_1 + 0.0126984127*G1_1_0_5_0 - 0.0380952381*G1_1_0_5_1 - 0.0253968254*G1_1_1_0_0 - 0.0253968254*G1_1_1_0_1 + 0.0126984127*G1_1_1_1_0 - 0.0126984127*G1_1_1_2_1 + 0.0126984127*G1_1_1_3_0 + 0.0380952381*G1_1_1_3_1 - 0.0126984127*G1_1_1_4_0 + 0.0380952381*G1_1_1_4_1 + 0.0126984127*G1_1_1_5_0 - 0.0380952381*G1_1_1_5_1 - 0.0222222222*G1_2_0_0_0 - 0.0222222222*G1_2_0_0_1 - 0.00317460317*G1_2_0_1_0 - 0.00317460317*G1_2_0_2_1 + 0.0126984127*G1_2_0_3_0 + 0.0126984127*G1_2_0_3_1 - 0.0126984127*G1_2_0_4_0 + 0.0253968254*G1_2_0_4_1 + 0.0253968254*G1_2_0_5_0 - 0.0126984127*G1_2_0_5_1 - 0.0222222222*G1_2_1_0_0 - 0.0222222222*G1_2_1_0_1 - 0.00317460317*G1_2_1_1_0 - 0.00317460317*G1_2_1_2_1 + 0.0126984127*G1_2_1_3_0 + 0.0126984127*G1_2_1_3_1 - 0.0126984127*G1_2_1_4_0 + 0.0253968254*G1_2_1_4_1 + 0.0253968254*G1_2_1_5_0 - 0.0126984127*G1_2_1_5_1 + 0.0380952381*G1_3_0_0_0 + 0.0380952381*G1_3_0_0_1 + 0.0126984127*G1_3_0_1_0 + 0.0126984127*G1_3_0_2_1 - 0.0507936508*G1_3_0_4_1 - 0.0507936508*G1_3_0_5_0 + 0.0380952381*G1_3_1_0_0 + 0.0380952381*G1_3_1_0_1 + 0.0126984127*G1_3_1_1_0 + 0.0126984127*G1_3_1_2_1 - 0.0507936508*G1_3_1_4_1 - 0.0507936508*G1_3_1_5_0 + 0.0888888889*G1_4_0_0_0 + 0.0888888889*G1_4_0_0_1 + 0.0126984127*G1_4_0_1_0 + 0.0126984127*G1_4_0_2_1 - 0.0507936508*G1_4_0_3_0 - 0.0507936508*G1_4_0_3_1 + 0.0507936508*G1_4_0_4_0 - 0.101587302*G1_4_0_4_1 - 0.101587302*G1_4_0_5_0 + 0.0507936508*G1_4_0_5_1 + 0.0888888889*G1_4_1_0_0 + 0.0888888889*G1_4_1_0_1 + 0.0126984127*G1_4_1_1_0 + 0.0126984127*G1_4_1_2_1 - 0.0507936508*G1_4_1_3_0 - 0.0507936508*G1_4_1_3_1 + 0.0507936508*G1_4_1_4_0 - 0.101587302*G1_4_1_4_1 - 0.101587302*G1_4_1_5_0 + 0.0507936508*G1_4_1_5_1 + 0.177777778*G1_5_0_0_0 + 0.177777778*G1_5_0_0_1 - 0.0253968254*G1_5_0_1_0 + 0.0761904762*G1_5_0_2_1 - 0.0507936508*G1_5_0_3_0 - 0.152380952*G1_5_0_3_1 + 0.0507936508*G1_5_0_4_0 - 0.253968254*G1_5_0_4_1 - 0.152380952*G1_5_0_5_0 + 0.152380952*G1_5_0_5_1 + 0.177777778*G1_5_1_0_0 + 0.177777778*G1_5_1_0_1 - 0.0253968254*G1_5_1_1_0 + 0.0761904762*G1_5_1_2_1 - 0.0507936508*G1_5_1_3_0 - 0.152380952*G1_5_1_3_1 + 0.0507936508*G1_5_1_4_0 - 0.253968254*G1_5_1_4_1 - 0.152380952*G1_5_1_5_0 + 0.152380952*G1_5_1_5_1;
    A[6] = 0.0734837652*G0_0_0_0 + 0.0734837652*G0_0_0_1 + 0.0512077209*G0_1_0_0 + 0.0512077209*G0_1_0_1 - 0.00432111568*G0_2_0_0 - 0.00432111568*G0_2_0_1 - 0.0604617049*G0_3_0_0 - 0.0604617049*G0_3_0_1 - 0.00334029708*G0_4_0_0 - 0.00334029708*G0_4_0_1 - 0.0102720721*G0_5_0_0 - 0.0102720721*G0_5_0_1 + 0.0734837652*G0_6_0_0 + 0.0734837652*G0_6_0_1 + 0.0512077209*G0_7_0_0 + 0.0512077209*G0_7_0_1 - 0.00432111568*G0_8_0_0 - 0.00432111568*G0_8_0_1 + 0.0428571429*G1_0_0_0_0 + 0.0428571429*G1_0_0_0_1 + 0.0206349206*G1_0_0_1_0 + 0.00793650794*G1_0_0_2_1 - 0.00634920635*G1_0_0_3_0 + 0.00634920635*G1_0_0_3_1 + 0.00634920635*G1_0_0_4_0 - 0.0507936508*G1_0_0_4_1 - 0.0634920635*G1_0_0_5_0 - 0.00634920635*G1_0_0_5_1 - 0.00714285714*G1_1_0_0_0 - 0.00714285714*G1_1_0_0_1 - 0.0182539683*G1_1_0_1_0 + 0.00396825397*G1_1_0_2_1 - 0.00317460317*G1_1_0_3_0 - 0.0253968254*G1_1_0_3_1 + 0.00317460317*G1_1_0_4_0 + 0.00317460317*G1_1_0_4_1 + 0.0253968254*G1_1_0_5_0 + 0.0253968254*G1_1_0_5_1 - 0.00396825397*G1_2_0_0_0 - 0.00396825397*G1_2_0_0_1 + 0.000793650794*G1_2_0_1_0 + 0.00396825397*G1_2_0_2_1 + 0.0126984127*G1_2_0_3_0 + 0.00952380952*G1_2_0_3_1 - 0.0126984127*G1_2_0_4_0 + 0.00317460317*G1_2_0_5_0 - 0.00952380952*G1_2_0_5_1 - 0.00317460317*G1_3_0_0_0 - 0.00317460317*G1_3_0_0_1 - 0.0222222222*G1_3_0_1_0 + 0.00317460317*G1_3_0_2_1 - 0.0126984127*G1_3_0_3_0 - 0.0380952381*G1_3_0_3_1 + 0.0126984127*G1_3_0_4_0 + 0.0253968254*G1_3_0_5_0 + 0.0380952381*G1_3_0_5_1 + 0.0253968254*G1_4_0_0_0 + 0.0253968254*G1_4_0_0_1 + 0.0126984127*G1_4_0_2_1 - 0.0126984127*G1_4_0_3_1 - 0.0380952381*G1_4_0_4_1 - 0.0253968254*G1_4_0_5_0 + 0.0126984127*G1_4_0_5_1 + 0.0126984127*G1_5_0_0_0 + 0.0126984127*G1_5_0_0_1 - 0.0253968254*G1_5_0_1_0 + 0.0126984127*G1_5_0_2_1 - 0.0126984127*G1_5_0_3_0 - 0.0507936508*G1_5_0_3_1 + 0.0126984127*G1_5_0_4_0 - 0.0253968254*G1_5_0_4_1 + 0.0126984127*G1_5_0_5_0 + 0.0507936508*G1_5_0_5_1;
    A[7] = 0.0193717022*G0_0_0_0 + 0.0342867365*G0_1_0_0 + 0.0158606966*G0_2_0_0 + 0.0604617049*G0_3_0_0 + 0.00334029708*G0_4_0_0 + 0.0102720721*G0_5_0_0 + 0.27875009*G0_6_0_0 + 0.076479448*G0_7_0_0 + 0.00117725224*G0_8_0_0 - 0.00714285714*G1_0_0_0_0 - 0.00714285714*G1_0_0_0_1 - 0.0182539683*G1_0_0_1_0 + 0.00396825397*G1_0_0_2_1 - 0.00317460317*G1_0_0_3_0 - 0.0253968254*G1_0_0_3_1 + 0.00317460317*G1_0_0_4_0 + 0.00317460317*G1_0_0_4_1 + 0.0253968254*G1_0_0_5_0 + 0.0253968254*G1_0_0_5_1 + 0.0428571429*G1_1_0_0_0 + 0.0428571429*G1_1_0_0_1 + 0.147619048*G1_1_0_1_0 - 0.0428571429*G1_1_0_2_1 + 0.019047619*G1_1_0_3_0 + 0.20952381*G1_1_0_3_1 - 0.019047619*G1_1_0_4_0 - 0.19047619*G1_1_0_5_0 - 0.20952381*G1_1_0_5_1 - 0.00396825397*G1_2_0_0_0 - 0.00396825397*G1_2_0_0_1 - 0.0182539683*G1_2_0_1_0 + 0.00714285714*G1_2_0_2_1 - 0.0253968254*G1_2_0_3_1 - 0.00317460317*G1_2_0_4_1 + 0.0222222222*G1_2_0_5_0 + 0.0253968254*G1_2_0_5_1 + 0.0253968254*G1_3_0_0_0 + 0.0253968254*G1_3_0_0_1 + 0.0761904762*G1_3_0_1_0 - 0.0126984127*G1_3_0_2_1 + 0.0253968254*G1_3_0_3_0 + 0.114285714*G1_3_0_3_1 - 0.0253968254*G1_3_0_4_0 - 0.0126984127*G1_3_0_4_1 - 0.101587302*G1_3_0_5_0 - 0.114285714*G1_3_0_5_1 - 0.00317460317*G1_4_0_0_0 - 0.00317460317*G1_4_0_0_1 + 0.00317460317*G1_4_0_1_0 + 0.00317460317*G1_4_0_2_1 + 0.0126984127*G1_4_0_3_0 + 0.0126984127*G1_4_0_3_1 - 0.0126984127*G1_4_0_4_0 - 0.0126984127*G1_4_0_5_1 + 0.0126984127*G1_5_0_0_0 + 0.0126984127*G1_5_0_0_1 + 0.0761904762*G1_5_0_1_0 - 0.0253968254*G1_5_0_2_1 + 0.0126984127*G1_5_0_3_0 + 0.114285714*G1_5_0_3_1 - 0.0126984127*G1_5_0_4_0 + 0.0126984127*G1_5_0_4_1 - 0.0888888889*G1_5_0_5_0 - 0.114285714*G1_5_0_5_1;
    A[8] = 0.021230148*G0_0_0_1 - 0.0298049583*G0_1_0_1 - 0.037721486*G0_2_0_1 - 0.047442691*G0_3_0_1 + 0.0117673313*G0_4_0_1 - 0.0383987144*G0_5_0_1 - 0.0805335117*G0_6_0_1 + 0.0445141224*G0_7_0_1 - 0.010276907*G0_8_0_1 - 0.00396825397*G1_0_0_0_0 - 0.00396825397*G1_0_0_0_1 + 0.000793650794*G1_0_0_1_0 + 0.00396825397*G1_0_0_2_1 + 0.0126984127*G1_0_0_3_0 + 0.00952380952*G1_0_0_3_1 - 0.0126984127*G1_0_0_4_0 + 0.00317460317*G1_0_0_5_0 - 0.00952380952*G1_0_0_5_1 - 0.00396825397*G1_1_0_0_0 - 0.00396825397*G1_1_0_0_1 - 0.0182539683*G1_1_0_1_0 + 0.00714285714*G1_1_0_2_1 - 0.0253968254*G1_1_0_3_1 - 0.00317460317*G1_1_0_4_1 + 0.0222222222*G1_1_0_5_0 + 0.0253968254*G1_1_0_5_1 - 0.00793650794*G1_2_0_0_0 - 0.00793650794*G1_2_0_0_1 + 0.0206349206*G1_2_0_1_0 - 0.0428571429*G1_2_0_2_1 - 0.0571428571*G1_2_0_3_0 + 0.00634920635*G1_2_0_3_1 + 0.0571428571*G1_2_0_4_0 + 0.0507936508*G1_2_0_4_1 - 0.0126984127*G1_2_0_5_0 - 0.00634920635*G1_2_0_5_1 - 0.0126984127*G1_3_0_0_0 - 0.0126984127*G1_3_0_0_1 - 0.0253968254*G1_3_0_1_0 - 0.0126984127*G1_3_0_2_1 - 0.0380952381*G1_3_0_3_0 - 0.0507936508*G1_3_0_3_1 + 0.0380952381*G1_3_0_4_0 + 0.0253968254*G1_3_0_4_1 + 0.0380952381*G1_3_0_5_0 + 0.0507936508*G1_3_0_5_1 - 0.0126984127*G1_4_0_0_0 - 0.0126984127*G1_4_0_0_1 - 0.0253968254*G1_4_0_2_1 - 0.0380952381*G1_4_0_3_0 - 0.0126984127*G1_4_0_3_1 + 0.0380952381*G1_4_0_4_0 + 0.0380952381*G1_4_0_4_1 + 0.0126984127*G1_4_0_5_0 + 0.0126984127*G1_4_0_5_1 - 0.00317460317*G1_5_0_0_0 - 0.00317460317*G1_5_0_0_1 - 0.0222222222*G1_5_0_1_0 + 0.00317460317*G1_5_0_2_1 - 0.0126984127*G1_5_0_3_0 - 0.0380952381*G1_5_0_3_1 + 0.0126984127*G1_5_0_4_0 + 0.0253968254*G1_5_0_5_0 + 0.0380952381*G1_5_0_5_1;
    A[9] = -0.011651767*G0_0_0_0 - 0.0135102128*G0_0_0_1 - 0.076530901*G0_1_0_0 - 0.0124392062*G0_1_0_1 - 0.0552611596*G0_2_0_0 - 0.001678977*G0_2_0_1 + 0.0260380278*G0_3_0_0 + 0.133942424*G0_3_0_1 + 0.0302152568*G0_4_0_0 + 0.0217882225*G0_4_0_1 - 0.0562532846*G0_5_0_0 - 0.00758249808*G0_5_0_1 + 0.0441993017*G0_6_0_0 + 0.403482904*G0_6_0_1 + 0.114299972*G0_7_0_0 + 0.146265298*G0_7_0_1 - 0.0150554461*G0_8_0_0 - 0.00360128684*G0_8_0_1 - 0.00317460317*G1_0_0_0_0 - 0.00317460317*G1_0_0_0_1 - 0.0222222222*G1_0_0_1_0 + 0.00317460317*G1_0_0_2_1 - 0.0126984127*G1_0_0_3_0 - 0.0380952381*G1_0_0_3_1 + 0.0126984127*G1_0_0_4_0 + 0.0253968254*G1_0_0_5_0 + 0.0380952381*G1_0_0_5_1 + 0.0253968254*G1_1_0_0_0 + 0.0253968254*G1_1_0_0_1 + 0.0761904762*G1_1_0_1_0 - 0.0126984127*G1_1_0_2_1 + 0.0253968254*G1_1_0_3_0 + 0.114285714*G1_1_0_3_1 - 0.0253968254*G1_1_0_4_0 - 0.0126984127*G1_1_0_4_1 - 0.101587302*G1_1_0_5_0 - 0.114285714*G1_1_0_5_1 - 0.0126984127*G1_2_0_0_0 - 0.0126984127*G1_2_0_0_1 - 0.0253968254*G1_2_0_1_0 - 0.0126984127*G1_2_0_2_1 - 0.0380952381*G1_2_0_3_0 - 0.0507936508*G1_2_0_3_1 + 0.0380952381*G1_2_0_4_0 + 0.0253968254*G1_2_0_4_1 + 0.0380952381*G1_2_0_5_0 + 0.0507936508*G1_2_0_5_1 + 0.0761904762*G1_3_0_0_0 + 0.0761904762*G1_3_0_0_1 + 0.177777778*G1_3_0_1_0 + 0.0253968254*G1_3_0_2_1 + 0.152380952*G1_3_0_3_0 + 0.304761905*G1_3_0_3_1 - 0.152380952*G1_3_0_4_0 - 0.101587302*G1_3_0_4_1 - 0.253968254*G1_3_0_5_0 - 0.304761905*G1_3_0_5_1 + 0.0126984127*G1_4_0_0_0 + 0.0126984127*G1_4_0_0_1 + 0.0380952381*G1_4_0_1_0 - 0.0126984127*G1_4_0_2_1 + 0.0507936508*G1_4_0_3_1 - 0.0507936508*G1_4_0_5_0 - 0.0507936508*G1_4_0_5_1 + 0.0126984127*G1_5_0_0_0 + 0.0126984127*G1_5_0_0_1 + 0.0888888889*G1_5_0_1_0 - 0.0126984127*G1_5_0_2_1 + 0.0507936508*G1_5_0_3_0 + 0.152380952*G1_5_0_3_1 - 0.0507936508*G1_5_0_4_0 - 0.101587302*G1_5_0_5_0 - 0.152380952*G1_5_0_5_1;
    A[10] = 0.011651767*G0_0_0_0 - 0.0947139131*G0_0_0_1 + 0.076530901*G0_1_0_0 - 0.0214027626*G0_1_0_1 + 0.0552611596*G0_2_0_0 + 0.0420426016*G0_2_0_1 - 0.0260380278*G0_3_0_0 + 0.107904396*G0_3_0_1 - 0.0302152568*G0_4_0_0 - 0.00842703425*G0_4_0_1 + 0.0562532846*G0_5_0_0 + 0.0486707865*G0_5_0_1 - 0.0441993017*G0_6_0_0 + 0.00704974659*G0_6_0_1 - 0.114299972*G0_7_0_0 - 0.0957218433*G0_7_0_1 + 0.0150554461*G0_8_0_0 + 0.0145980227*G0_8_0_1 + 0.0253968254*G1_0_0_0_0 + 0.0253968254*G1_0_0_0_1 + 0.0126984127*G1_0_0_2_1 - 0.0126984127*G1_0_0_3_1 - 0.0380952381*G1_0_0_4_1 - 0.0253968254*G1_0_0_5_0 + 0.0126984127*G1_0_0_5_1 - 0.00317460317*G1_1_0_0_0 - 0.00317460317*G1_1_0_0_1 + 0.00317460317*G1_1_0_1_0 + 0.00317460317*G1_1_0_2_1 + 0.0126984127*G1_1_0_3_0 + 0.0126984127*G1_1_0_3_1 - 0.0126984127*G1_1_0_4_0 - 0.0126984127*G1_1_0_5_1 - 0.0126984127*G1_2_0_0_0 - 0.0126984127*G1_2_0_0_1 - 0.0253968254*G1_2_0_2_1 - 0.0380952381*G1_2_0_3_0 - 0.0126984127*G1_2_0_3_1 + 0.0380952381*G1_2_0_4_0 + 0.0380952381*G1_2_0_4_1 + 0.0126984127*G1_2_0_5_0 + 0.0126984127*G1_2_0_5_1 + 0.0126984127*G1_3_0_0_0 + 0.0126984127*G1_3_0_0_1 + 0.0380952381*G1_3_0_1_0 - 0.0126984127*G1_3_0_2_1 + 0.0507936508*G1_3_0_3_1 - 0.0507936508*G1_3_0_5_0 - 0.0507936508*G1_3_0_5_1 + 0.0761904762*G1_4_0_0_0 + 0.0761904762*G1_4_0_0_1 + 0.0761904762*G1_4_0_1_0 - 0.0761904762*G1_4_0_2_1 - 0.152380952*G1_4_0_3_0 + 0.152380952*G1_4_0_4_0 - 0.152380952*G1_4_0_5_0 + 0.0126984127*G1_5_0_0_0 + 0.0126984127*G1_5_0_0_1 + 0.0380952381*G1_5_0_1_0 - 0.0126984127*G1_5_0_2_1 + 0.0507936508*G1_5_0_3_1 - 0.0507936508*G1_5_0_5_0 - 0.0507936508*G1_5_0_5_1;
    A[11] = -0.0928554674*G0_0_0_0 + 0.0135102128*G0_0_0_1 - 0.0854944574*G0_1_0_0 + 0.0124392062*G0_1_0_1 - 0.011539581*G0_2_0_0 + 0.001678977*G0_2_0_1 - 0.133942424*G0_3_0_1 - 0.0217882225*G0_4_0_1 + 0.00758249808*G0_5_0_1 - 0.352233855*G0_6_0_0 - 0.403482904*G0_6_0_1 - 0.127687169*G0_7_0_0 - 0.146265298*G0_7_0_1 + 0.00314386343*G0_8_0_0 + 0.00360128684*G0_8_0_1 + 0.0126984127*G1_0_0_0_0 + 0.0126984127*G1_0_0_0_1 - 0.0253968254*G1_0_0_1_0 + 0.0126984127*G1_0_0_2_1 - 0.0126984127*G1_0_0_3_0 - 0.0507936508*G1_0_0_3_1 + 0.0126984127*G1_0_0_4_0 - 0.0253968254*G1_0_0_4_1 + 0.0126984127*G1_0_0_5_0 + 0.0507936508*G1_0_0_5_1 + 0.0126984127*G1_1_0_0_0 + 0.0126984127*G1_1_0_0_1 + 0.0761904762*G1_1_0_1_0 - 0.0253968254*G1_1_0_2_1 + 0.0126984127*G1_1_0_3_0 + 0.114285714*G1_1_0_3_1 - 0.0126984127*G1_1_0_4_0 + 0.0126984127*G1_1_0_4_1 - 0.0888888889*G1_1_0_5_0 - 0.114285714*G1_1_0_5_1 - 0.00317460317*G1_2_0_0_0 - 0.00317460317*G1_2_0_0_1 - 0.0222222222*G1_2_0_1_0 + 0.00317460317*G1_2_0_2_1 - 0.0126984127*G1_2_0_3_0 - 0.0380952381*G1_2_0_3_1 + 0.0126984127*G1_2_0_4_0 + 0.0253968254*G1_2_0_5_0 + 0.0380952381*G1_2_0_5_1 + 0.0126984127*G1_3_0_0_0 + 0.0126984127*G1_3_0_0_1 + 0.0888888889*G1_3_0_1_0 - 0.0126984127*G1_3_0_2_1 + 0.0507936508*G1_3_0_3_0 + 0.152380952*G1_3_0_3_1 - 0.0507936508*G1_3_0_4_0 - 0.101587302*G1_3_0_5_0 - 0.152380952*G1_3_0_5_1 + 0.0126984127*G1_4_0_0_0 + 0.0126984127*G1_4_0_0_1 + 0.0380952381*G1_4_0_1_0 - 0.0126984127*G1_4_0_2_1 + 0.0507936508*G1_4_0_3_1 - 0.0507936508*G1_4_0_5_0 - 0.0507936508*G1_4_0_5_1 - 0.0253968254*G1_5_0_0_0 - 0.0253968254*G1_5_0_0_1 + 0.177777778*G1_5_0_1_0 - 0.0761904762*G1_5_0_2_1 + 0.0507936508*G1_5_0_3_0 + 0.304761905*G1_5_0_3_1 - 0.0507936508*G1_5_0_4_0 + 0.101587302*G1_5_0_4_1 - 0.152380952*G1_5_0_5_0 - 0.304761905*G1_5_0_5_1;
    A[12] = 0.0805335117*G0_0_1_0 + 0.0805335117*G0_0_1_1 - 0.0445141224*G0_1_1_0 - 0.0445141224*G0_1_1_1 + 0.010276907*G0_2_1_0 + 0.010276907*G0_2_1_1 + 0.047442691*G0_3_1_0 + 0.047442691*G0_3_1_1 - 0.0117673313*G0_4_1_0 - 0.0117673313*G0_4_1_1 + 0.0383987144*G0_5_1_0 + 0.0383987144*G0_5_1_1 - 0.021230148*G0_6_1_0 - 0.021230148*G0_6_1_1 + 0.0298049583*G0_7_1_0 + 0.0298049583*G0_7_1_1 + 0.037721486*G0_8_1_0 + 0.037721486*G0_8_1_1 + 0.0428571429*G1_0_1_0_0 + 0.0428571429*G1_0_1_0_1 + 0.00793650794*G1_0_1_1_0 + 0.0206349206*G1_0_1_2_1 + 0.00634920635*G1_0_1_3_0 - 0.00634920635*G1_0_1_3_1 - 0.00634920635*G1_0_1_4_0 - 0.0634920635*G1_0_1_4_1 - 0.0507936508*G1_0_1_5_0 + 0.00634920635*G1_0_1_5_1 - 0.00396825397*G1_1_1_0_0 - 0.00396825397*G1_1_1_0_1 + 0.00396825397*G1_1_1_1_0 + 0.000793650794*G1_1_1_2_1 + 0.00952380952*G1_1_1_3_0 + 0.0126984127*G1_1_1_3_1 - 0.00952380952*G1_1_1_4_0 + 0.00317460317*G1_1_1_4_1 - 0.0126984127*G1_1_1_5_1 - 0.00714285714*G1_2_1_0_0 - 0.00714285714*G1_2_1_0_1 + 0.00396825397*G1_2_1_1_0 - 0.0182539683*G1_2_1_2_1 - 0.0253968254*G1_2_1_3_0 - 0.00317460317*G1_2_1_3_1 + 0.0253968254*G1_2_1_4_0 + 0.0253968254*G1_2_1_4_1 + 0.00317460317*G1_2_1_5_0 + 0.00317460317*G1_2_1_5_1 - 0.00317460317*G1_3_1_0_0 - 0.00317460317*G1_3_1_0_1 + 0.00317460317*G1_3_1_1_0 - 0.0222222222*G1_3_1_2_1 - 0.0380952381*G1_3_1_3_0 - 0.0126984127*G1_3_1_3_1 + 0.0380952381*G1_3_1_4_0 + 0.0253968254*G1_3_1_4_1 + 0.0126984127*G1_3_1_5_1 + 0.0126984127*G1_4_1_0_0 + 0.0126984127*G1_4_1_0_1 + 0.0126984127*G1_4_1_1_0 - 0.0253968254*G1_4_1_2_1 - 0.0507936508*G1_4_1_3_0 - 0.0126984127*G1_4_1_3_1 + 0.0507936508*G1_4_1_4_0 + 0.0126984127*G1_4_1_4_1 - 0.0253968254*G1_4_1_5_0 + 0.0126984127*G1_4_1_5_1 + 0.0253968254*G1_5_1_0_0 + 0.0253968254*G1_5_1_0_1 + 0.0126984127*G1_5_1_1_0 - 0.0126984127*G1_5_1_3_0 + 0.0126984127*G1_5_1_4_0 - 0.0253968254*G1_5_1_4_1 - 0.0380952381*G1_5_1_5_0;
    A[13] = 0.021230148*G0_0_1_0 - 0.0298049583*G0_1_1_0 - 0.037721486*G0_2_1_0 - 0.047442691*G0_3_1_0 + 0.0117673313*G0_4_1_0 - 0.0383987144*G0_5_1_0 - 0.0805335117*G0_6_1_0 + 0.0445141224*G0_7_1_0 - 0.010276907*G0_8_1_0 - 0.00396825397*G1_0_1_0_0 - 0.00396825397*G1_0_1_0_1 + 0.00396825397*G1_0_1_1_0 + 0.000793650794*G1_0_1_2_1 + 0.00952380952*G1_0_1_3_0 + 0.0126984127*G1_0_1_3_1 - 0.00952380952*G1_0_1_4_0 + 0.00317460317*G1_0_1_4_1 - 0.0126984127*G1_0_1_5_1 - 0.00793650794*G1_1_1_0_0 - 0.00793650794*G1_1_1_0_1 - 0.0428571429*G1_1_1_1_0 + 0.0206349206*G1_1_1_2_1 + 0.00634920635*G1_1_1_3_0 - 0.0571428571*G1_1_1_3_1 - 0.00634920635*G1_1_1_4_0 - 0.0126984127*G1_1_1_4_1 + 0.0507936508*G1_1_1_5_0 + 0.0571428571*G1_1_1_5_1 - 0.00396825397*G1_2_1_0_0 - 0.00396825397*G1_2_1_0_1 + 0.00714285714*G1_2_1_1_0 - 0.0182539683*G1_2_1_2_1 - 0.0253968254*G1_2_1_3_0 + 0.0253968254*G1_2_1_4_0 + 0.0222222222*G1_2_1_4_1 - 0.00317460317*G1_2_1_5_0 - 0.0126984127*G1_3_1_0_0 - 0.0126984127*G1_3_1_0_1 - 0.0126984127*G1_3_1_1_0 - 0.0253968254*G1_3_1_2_1 - 0.0507936508*G1_3_1_3_0 - 0.0380952381*G1_3_1_3_1 + 0.0507936508*G1_3_1_4_0 + 0.0380952381*G1_3_1_4_1 + 0.0253968254*G1_3_1_5_0 + 0.0380952381*G1_3_1_5_1 - 0.00317460317*G1_4_1_0_0 - 0.00317460317*G1_4_1_0_1 + 0.00317460317*G1_4_1_1_0 - 0.0222222222*G1_4_1_2_1 - 0.0380952381*G1_4_1_3_0 - 0.0126984127*G1_4_1_3_1 + 0.0380952381*G1_4_1_4_0 + 0.0253968254*G1_4_1_4_1 + 0.0126984127*G1_4_1_5_1 - 0.0126984127*G1_5_1_0_0 - 0.0126984127*G1_5_1_0_1 - 0.0253968254*G1_5_1_1_0 - 0.0126984127*G1_5_1_3_0 - 0.0380952381*G1_5_1_3_1 + 0.0126984127*G1_5_1_4_0 + 0.0126984127*G1_5_1_4_1 + 0.0380952381*G1_5_1_5_0 + 0.0380952381*G1_5_1_5_1;
    A[14] = 0.0232668858*G0_0_1_1 + 0.0259090141*G0_1_1_1 + 0.089712989*G0_2_1_1 + 0.0372270172*G0_3_1_1 + 0.0414544226*G0_4_1_1 + 0.143540782*G0_5_1_1 + 0.0232668858*G0_6_1_1 + 0.0259090141*G0_7_1_1 + 0.089712989*G0_8_1_1 - 0.00714285714*G1_0_1_0_0 - 0.00714285714*G1_0_1_0_1 + 0.00396825397*G1_0_1_1_0 - 0.0182539683*G1_0_1_2_1 - 0.0253968254*G1_0_1_3_0 - 0.00317460317*G1_0_1_3_1 + 0.0253968254*G1_0_1_4_0 + 0.0253968254*G1_0_1_4_1 + 0.00317460317*G1_0_1_5_0 + 0.00317460317*G1_0_1_5_1 - 0.00396825397*G1_1_1_0_0 - 0.00396825397*G1_1_1_0_1 + 0.00714285714*G1_1_1_1_0 - 0.0182539683*G1_1_1_2_1 - 0.0253968254*G1_1_1_3_0 + 0.0253968254*G1_1_1_4_0 + 0.0222222222*G1_1_1_4_1 - 0.00317460317*G1_1_1_5_0 + 0.0428571429*G1_2_1_0_0 + 0.0428571429*G1_2_1_0_1 - 0.0428571429*G1_2_1_1_0 + 0.147619048*G1_2_1_2_1 + 0.20952381*G1_2_1_3_0 + 0.019047619*G1_2_1_3_1 - 0.20952381*G1_2_1_4_0 - 0.19047619*G1_2_1_4_1 - 0.019047619*G1_2_1_5_1 + 0.0253968254*G1_3_1_0_0 + 0.0253968254*G1_3_1_0_1 - 0.0126984127*G1_3_1_1_0 + 0.0761904762*G1_3_1_2_1 + 0.114285714*G1_3_1_3_0 + 0.0253968254*G1_3_1_3_1 - 0.114285714*G1_3_1_4_0 - 0.101587302*G1_3_1_4_1 - 0.0126984127*G1_3_1_5_0 - 0.0253968254*G1_3_1_5_1 + 0.0126984127*G1_4_1_0_0 + 0.0126984127*G1_4_1_0_1 - 0.0253968254*G1_4_1_1_0 + 0.0761904762*G1_4_1_2_1 + 0.114285714*G1_4_1_3_0 + 0.0126984127*G1_4_1_3_1 - 0.114285714*G1_4_1_4_0 - 0.0888888889*G1_4_1_4_1 + 0.0126984127*G1_4_1_5_0 - 0.0126984127*G1_4_1_5_1 - 0.00317460317*G1_5_1_0_0 - 0.00317460317*G1_5_1_0_1 + 0.00317460317*G1_5_1_1_0 + 0.00317460317*G1_5_1_2_1 + 0.0126984127*G1_5_1_3_0 + 0.0126984127*G1_5_1_3_1 - 0.0126984127*G1_5_1_4_0 - 0.0126984127*G1_5_1_5_1;
    A[15] = -0.0127695922*G0_0_1_0 - 0.01480633*G0_0_1_1 + 0.0665271924*G0_1_1_0 + 0.01081322*G0_1_1_1 + 0.131427585*G0_2_1_0 + 0.00399311006*G0_2_1_1 - 0.0204313475*G0_3_1_0 - 0.105101056*G0_3_1_1 + 0.106443508*G0_4_1_0 + 0.0767564165*G0_4_1_1 + 0.210284136*G0_5_1_0 + 0.0283446393*G0_5_1_1 - 0.0127695922*G0_6_1_0 - 0.11656999*G0_6_1_1 + 0.0665271924*G0_7_1_0 + 0.0851323007*G0_7_1_1 + 0.131427585*G0_8_1_0 + 0.031437689*G0_8_1_1 - 0.00317460317*G1_0_1_0_0 - 0.00317460317*G1_0_1_0_1 + 0.00317460317*G1_0_1_1_0 - 0.0222222222*G1_0_1_2_1 - 0.0380952381*G1_0_1_3_0 - 0.0126984127*G1_0_1_3_1 + 0.0380952381*G1_0_1_4_0 + 0.0253968254*G1_0_1_4_1 + 0.0126984127*G1_0_1_5_1 - 0.0126984127*G1_1_1_0_0 - 0.0126984127*G1_1_1_0_1 - 0.0126984127*G1_1_1_1_0 - 0.0253968254*G1_1_1_2_1 - 0.0507936508*G1_1_1_3_0 - 0.0380952381*G1_1_1_3_1 + 0.0507936508*G1_1_1_4_0 + 0.0380952381*G1_1_1_4_1 + 0.0253968254*G1_1_1_5_0 + 0.0380952381*G1_1_1_5_1 + 0.0253968254*G1_2_1_0_0 + 0.0253968254*G1_2_1_0_1 - 0.0126984127*G1_2_1_1_0 + 0.0761904762*G1_2_1_2_1 + 0.114285714*G1_2_1_3_0 + 0.0253968254*G1_2_1_3_1 - 0.114285714*G1_2_1_4_0 - 0.101587302*G1_2_1_4_1 - 0.0126984127*G1_2_1_5_0 - 0.0253968254*G1_2_1_5_1 + 0.0761904762*G1_3_1_0_0 + 0.0761904762*G1_3_1_0_1 + 0.0253968254*G1_3_1_1_0 + 0.177777778*G1_3_1_2_1 + 0.304761905*G1_3_1_3_0 + 0.152380952*G1_3_1_3_1 - 0.304761905*G1_3_1_4_0 - 0.253968254*G1_3_1_4_1 - 0.101587302*G1_3_1_5_0 - 0.152380952*G1_3_1_5_1 + 0.0126984127*G1_4_1_0_0 + 0.0126984127*G1_4_1_0_1 - 0.0126984127*G1_4_1_1_0 + 0.0888888889*G1_4_1_2_1 + 0.152380952*G1_4_1_3_0 + 0.0507936508*G1_4_1_3_1 - 0.152380952*G1_4_1_4_0 - 0.101587302*G1_4_1_4_1 - 0.0507936508*G1_4_1_5_1 + 0.0126984127*G1_5_1_0_0 + 0.0126984127*G1_5_1_0_1 - 0.0126984127*G1_5_1_1_0 + 0.0380952381*G1_5_1_2_1 + 0.0507936508*G1_5_1_3_0 - 0.0507936508*G1_5_1_4_0 - 0.0507936508*G1_5_1_4_1;
    A[16] = 0.0127695922*G0_0_1_0 - 0.103800398*G0_0_1_1 - 0.0665271924*G0_1_1_0 + 0.0186051083*G0_1_1_1 - 0.131427585*G0_2_1_0 - 0.099989896*G0_2_1_1 + 0.0204313475*G0_3_1_0 - 0.0846697083*G0_3_1_1 - 0.106443508*G0_4_1_0 - 0.0296870912*G0_4_1_1 - 0.210284136*G0_5_1_0 - 0.181939497*G0_5_1_1 + 0.0127695922*G0_6_1_0 - 0.00203673781*G0_6_1_1 - 0.0665271924*G0_7_1_0 - 0.0557139724*G0_7_1_1 - 0.131427585*G0_8_1_0 - 0.127434475*G0_8_1_1 + 0.0126984127*G1_0_1_0_0 + 0.0126984127*G1_0_1_0_1 + 0.0126984127*G1_0_1_1_0 - 0.0253968254*G1_0_1_2_1 - 0.0507936508*G1_0_1_3_0 - 0.0126984127*G1_0_1_3_1 + 0.0507936508*G1_0_1_4_0 + 0.0126984127*G1_0_1_4_1 - 0.0253968254*G1_0_1_5_0 + 0.0126984127*G1_0_1_5_1 - 0.00317460317*G1_1_1_0_0 - 0.00317460317*G1_1_1_0_1 + 0.00317460317*G1_1_1_1_0 - 0.0222222222*G1_1_1_2_1 - 0.0380952381*G1_1_1_3_0 - 0.0126984127*G1_1_1_3_1 + 0.0380952381*G1_1_1_4_0 + 0.0253968254*G1_1_1_4_1 + 0.0126984127*G1_1_1_5_1 + 0.0126984127*G1_2_1_0_0 + 0.0126984127*G1_2_1_0_1 - 0.0253968254*G1_2_1_1_0 + 0.0761904762*G1_2_1_2_1 + 0.114285714*G1_2_1_3_0 + 0.0126984127*G1_2_1_3_1 - 0.114285714*G1_2_1_4_0 - 0.0888888889*G1_2_1_4_1 + 0.0126984127*G1_2_1_5_0 - 0.0126984127*G1_2_1_5_1 + 0.0126984127*G1_3_1_0_0 + 0.0126984127*G1_3_1_0_1 - 0.0126984127*G1_3_1_1_0 + 0.0888888889*G1_3_1_2_1 + 0.152380952*G1_3_1_3_0 + 0.0507936508*G1_3_1_3_1 - 0.152380952*G1_3_1_4_0 - 0.101587302*G1_3_1_4_1 - 0.0507936508*G1_3_1_5_1 - 0.0253968254*G1_4_1_0_0 - 0.0253968254*G1_4_1_0_1 - 0.0761904762*G1_4_1_1_0 + 0.177777778*G1_4_1_2_1 + 0.304761905*G1_4_1_3_0 + 0.0507936508*G1_4_1_3_1 - 0.304761905*G1_4_1_4_0 - 0.152380952*G1_4_1_4_1 + 0.101587302*G1_4_1_5_0 - 0.0507936508*G1_4_1_5_1 + 0.0126984127*G1_5_1_0_0 + 0.0126984127*G1_5_1_0_1 - 0.0126984127*G1_5_1_1_0 + 0.0380952381*G1_5_1_2_1 + 0.0507936508*G1_5_1_3_0 - 0.0507936508*G1_5_1_4_0 - 0.0507936508*G1_5_1_4_1;
    A[17] = -0.10176366*G0_0_1_0 + 0.01480633*G0_0_1_1 + 0.0743190807*G0_1_1_0 - 0.01081322*G0_1_1_1 + 0.027444579*G0_2_1_0 - 0.00399311006*G0_2_1_1 + 0.105101056*G0_3_1_1 - 0.0767564165*G0_4_1_1 - 0.0283446393*G0_5_1_1 + 0.10176366*G0_6_1_0 + 0.11656999*G0_6_1_1 - 0.0743190807*G0_7_1_0 - 0.0851323007*G0_7_1_1 - 0.027444579*G0_8_1_0 - 0.031437689*G0_8_1_1 + 0.0253968254*G1_0_1_0_0 + 0.0253968254*G1_0_1_0_1 + 0.0126984127*G1_0_1_1_0 - 0.0126984127*G1_0_1_3_0 + 0.0126984127*G1_0_1_4_0 - 0.0253968254*G1_0_1_4_1 - 0.0380952381*G1_0_1_5_0 - 0.0126984127*G1_1_1_0_0 - 0.0126984127*G1_1_1_0_1 - 0.0253968254*G1_1_1_1_0 - 0.0126984127*G1_1_1_3_0 - 0.0380952381*G1_1_1_3_1 + 0.0126984127*G1_1_1_4_0 + 0.0126984127*G1_1_1_4_1 + 0.0380952381*G1_1_1_5_0 + 0.0380952381*G1_1_1_5_1 - 0.00317460317*G1_2_1_0_0 - 0.00317460317*G1_2_1_0_1 + 0.00317460317*G1_2_1_1_0 + 0.00317460317*G1_2_1_2_1 + 0.0126984127*G1_2_1_3_0 + 0.0126984127*G1_2_1_3_1 - 0.0126984127*G1_2_1_4_0 - 0.0126984127*G1_2_1_5_1 + 0.0126984127*G1_3_1_0_0 + 0.0126984127*G1_3_1_0_1 - 0.0126984127*G1_3_1_1_0 + 0.0380952381*G1_3_1_2_1 + 0.0507936508*G1_3_1_3_0 - 0.0507936508*G1_3_1_4_0 - 0.0507936508*G1_3_1_4_1 + 0.0126984127*G1_4_1_0_0 + 0.0126984127*G1_4_1_0_1 - 0.0126984127*G1_4_1_1_0 + 0.0380952381*G1_4_1_2_1 + 0.0507936508*G1_4_1_3_0 - 0.0507936508*G1_4_1_4_0 - 0.0507936508*G1_4_1_4_1 + 0.0761904762*G1_5_1_0_0 + 0.0761904762*G1_5_1_0_1 - 0.0761904762*G1_5_1_1_0 + 0.0761904762*G1_5_1_2_1 - 0.152380952*G1_5_1_3_1 - 0.152380952*G1_5_1_4_1 + 0.152380952*G1_5_1_5_1;
    A[18] = -0.0441993017*G0_0_0_0 - 0.0441993017*G0_0_0_1 - 0.0512490483*G0_0_1_0 - 0.0512490483*G0_0_1_1 - 0.114299972*G0_1_0_0 - 0.114299972*G0_1_0_1 - 0.0185781287*G0_1_1_0 - 0.0185781287*G0_1_1_1 + 0.0150554461*G0_2_0_0 + 0.0150554461*G0_2_0_1 + 0.000457423404*G0_2_1_0 + 0.000457423404*G0_2_1_1 - 0.0260380278*G0_3_0_0 - 0.0260380278*G0_3_0_1 - 0.133942424*G0_3_1_0 - 0.133942424*G0_3_1_1 - 0.0302152568*G0_4_0_0 - 0.0302152568*G0_4_0_1 - 0.0217882225*G0_4_1_0 - 0.0217882225*G0_4_1_1 + 0.0562532846*G0_5_0_0 + 0.0562532846*G0_5_0_1 + 0.00758249808*G0_5_1_0 + 0.00758249808*G0_5_1_1 + 0.011651767*G0_6_0_0 + 0.011651767*G0_6_0_1 + 0.10636568*G0_6_1_0 + 0.10636568*G0_6_1_1 + 0.076530901*G0_7_0_0 + 0.076530901*G0_7_0_1 + 0.0979336636*G0_7_1_0 + 0.0979336636*G0_7_1_1 + 0.0552611596*G0_8_0_0 + 0.0552611596*G0_8_0_1 + 0.013218558*G0_8_1_0 + 0.013218558*G0_8_1_1 - 0.019047619*G1_0_0_0_0 - 0.019047619*G1_0_0_0_1 - 0.00634920635*G1_0_0_1_0 + 0.00634920635*G1_0_0_2_1 + 0.0253968254*G1_0_0_3_0 + 0.0126984127*G1_0_0_3_1 - 0.0253968254*G1_0_0_4_0 + 0.0126984127*G1_0_0_4_1 + 0.0253968254*G1_0_0_5_0 - 0.0126984127*G1_0_0_5_1 - 0.019047619*G1_0_1_0_0 - 0.019047619*G1_0_1_0_1 + 0.00634920635*G1_0_1_1_0 - 0.00634920635*G1_0_1_2_1 + 0.0126984127*G1_0_1_3_0 + 0.0253968254*G1_0_1_3_1 - 0.0126984127*G1_0_1_4_0 + 0.0253968254*G1_0_1_4_1 + 0.0126984127*G1_0_1_5_0 - 0.0253968254*G1_0_1_5_1 + 0.00317460317*G1_1_0_0_0 + 0.00317460317*G1_1_0_0_1 - 0.00317460317*G1_1_0_1_0 + 0.00952380952*G1_1_0_2_1 + 0.0126984127*G1_1_0_3_0 - 0.0126984127*G1_1_0_4_0 - 0.0126984127*G1_1_0_4_1 - 0.0253968254*G1_1_1_1_0 + 0.0126984127*G1_1_1_2_1 - 0.0380952381*G1_1_1_3_1 - 0.0126984127*G1_1_1_4_1 + 0.0253968254*G1_1_1_5_0 + 0.0380952381*G1_1_1_5_1 + 0.0126984127*G1_2_0_1_0 - 0.0253968254*G1_2_0_2_1 - 0.0380952381*G1_2_0_3_0 + 0.0380952381*G1_2_0_4_0 + 0.0253968254*G1_2_0_4_1 - 0.0126984127*G1_2_0_5_0 + 0.00317460317*G1_2_1_0_0 + 0.00317460317*G1_2_1_0_1 + 0.00952380952*G1_2_1_1_0 - 0.00317460317*G1_2_1_2_1 + 0.0126984127*G1_2_1_3_1 - 0.0126984127*G1_2_1_5_0 - 0.0126984127*G1_2_1_5_1 - 0.0126984127*G1_3_0_0_0 - 0.0126984127*G1_3_0_0_1 - 0.0126984127*G1_3_0_1_0 - 0.0380952381*G1_3_0_2_1 - 0.0761904762*G1_3_0_3_0 - 0.0507936508*G1_3_0_3_1 + 0.0761904762*G1_3_0_4_0 + 0.0507936508*G1_3_0_4_1 + 0.0253968254*G1_3_0_5_0 + 0.0507936508*G1_3_0_5_1 - 0.0126984127*G1_3_1_0_0 - 0.0126984127*G1_3_1_0_1 - 0.0380952381*G1_3_1_1_0 - 0.0126984127*G1_3_1_2_1 - 0.0507936508*G1_3_1_3_0 - 0.0761904762*G1_3_1_3_1 + 0.0507936508*G1_3_1_4_0 + 0.0253968254*G1_3_1_4_1 + 0.0507936508*G1_3_1_5_0 + 0.0761904762*G1_3_1_5_1 - 0.0253968254*G1_4_0_0_0 - 0.0253968254*G1_4_0_0_1 - 0.0507936508*G1_4_0_2_1 - 0.0761904762*G1_4_0_3_0 - 0.0253968254*G1_4_0_3_1 + 0.0761904762*G1_4_0_4_0 + 0.0761904762*G1_4_0_4_1 + 0.0253968254*G1_4_0_5_0 + 0.0253968254*G1_4_0_5_1 - 0.0126984127*G1_4_1_0_0 - 0.0126984127*G1_4_1_0_1 - 0.0126984127*G1_4_1_1_0 - 0.0126984127*G1_4_1_2_1 - 0.0253968254*G1_4_1_3_0 - 0.0253968254*G1_4_1_3_1 + 0.0253968254*G1_4_1_4_0 + 0.0253968254*G1_4_1_4_1 + 0.0253968254*G1_4_1_5_0 + 0.0253968254*G1_4_1_5_1 - 0.0126984127*G1_5_0_0_0 - 0.0126984127*G1_5_0_0_1 - 0.0126984127*G1_5_0_1_0 - 0.0126984127*G1_5_0_2_1 - 0.0253968254*G1_5_0_3_0 - 0.0253968254*G1_5_0_3_1 + 0.0253968254*G1_5_0_4_0 + 0.0253968254*G1_5_0_4_1 + 0.0253968254*G1_5_0_5_0 + 0.0253968254*G1_5_0_5_1 - 0.0253968254*G1_5_1_0_0 - 0.0253968254*G1_5_1_0_1 - 0.0507936508*G1_5_1_1_0 - 0.0253968254*G1_5_1_3_0 - 0.0761904762*G1_5_1_3_1 + 0.0253968254*G1_5_1_4_0 + 0.0253968254*G1_5_1_4_1 + 0.0761904762*G1_5_1_5_0 + 0.0761904762*G1_5_1_5_1;
    A[19] = -0.011651767*G0_0_0_0 - 0.0135102128*G0_0_1_0 - 0.076530901*G0_1_0_0 - 0.0124392062*G0_1_1_0 - 0.0552611596*G0_2_0_0 - 0.001678977*G0_2_1_0 + 0.0260380278*G0_3_0_0 + 0.133942424*G0_3_1_0 + 0.0302152568*G0_4_0_0 + 0.0217882225*G0_4_1_0 - 0.0562532846*G0_5_0_0 - 0.00758249808*G0_5_1_0 + 0.0441993017*G0_6_0_0 + 0.403482904*G0_6_1_0 + 0.114299972*G0_7_0_0 + 0.146265298*G0_7_1_0 - 0.0150554461*G0_8_0_0 - 0.00360128684*G0_8_1_0 + 0.00317460317*G1_0_0_0_0 + 0.00317460317*G1_0_0_0_1 - 0.00317460317*G1_0_0_1_0 + 0.00952380952*G1_0_0_2_1 + 0.0126984127*G1_0_0_3_0 - 0.0126984127*G1_0_0_4_0 - 0.0126984127*G1_0_0_4_1 - 0.0253968254*G1_0_1_1_0 + 0.0126984127*G1_0_1_2_1 - 0.0380952381*G1_0_1_3_1 - 0.0126984127*G1_0_1_4_1 + 0.0253968254*G1_0_1_5_0 + 0.0380952381*G1_0_1_5_1 + 0.00634920635*G1_1_0_0_0 + 0.00634920635*G1_1_0_0_1 + 0.019047619*G1_1_0_1_0 + 0.00634920635*G1_1_0_2_1 + 0.0253968254*G1_1_0_3_0 + 0.0380952381*G1_1_0_3_1 - 0.0253968254*G1_1_0_4_0 - 0.0126984127*G1_1_0_4_1 - 0.0253968254*G1_1_0_5_0 - 0.0380952381*G1_1_0_5_1 + 0.0571428571*G1_1_1_0_0 + 0.0571428571*G1_1_1_0_1 + 0.20952381*G1_1_1_1_0 - 0.0571428571*G1_1_1_2_1 + 0.0380952381*G1_1_1_3_0 + 0.304761905*G1_1_1_3_1 - 0.0380952381*G1_1_1_4_0 - 0.266666667*G1_1_1_5_0 - 0.304761905*G1_1_1_5_1 - 0.0126984127*G1_2_0_0_0 - 0.0126984127*G1_2_0_0_1 - 0.0253968254*G1_2_0_2_1 - 0.0380952381*G1_2_0_3_0 - 0.0126984127*G1_2_0_3_1 + 0.0380952381*G1_2_0_4_0 + 0.0380952381*G1_2_0_4_1 + 0.0126984127*G1_2_0_5_0 + 0.0126984127*G1_2_0_5_1 - 0.0126984127*G1_2_1_0_0 - 0.0126984127*G1_2_1_0_1 - 0.0253968254*G1_2_1_1_0 - 0.0126984127*G1_2_1_3_0 - 0.0380952381*G1_2_1_3_1 + 0.0126984127*G1_2_1_4_0 + 0.0126984127*G1_2_1_4_1 + 0.0380952381*G1_2_1_5_0 + 0.0380952381*G1_2_1_5_1 + 0.0253968254*G1_3_0_1_0 - 0.0507936508*G1_3_0_2_1 - 0.0761904762*G1_3_0_3_0 + 0.0761904762*G1_3_0_4_0 + 0.0507936508*G1_3_0_4_1 - 0.0253968254*G1_3_0_5_0 + 0.0380952381*G1_3_1_0_0 + 0.0380952381*G1_3_1_0_1 + 0.114285714*G1_3_1_1_0 - 0.0380952381*G1_3_1_2_1 + 0.152380952*G1_3_1_3_1 - 0.152380952*G1_3_1_5_0 - 0.152380952*G1_3_1_5_1 + 0.0126984127*G1_4_0_0_0 + 0.0126984127*G1_4_0_0_1 + 0.0126984127*G1_4_0_1_0 - 0.0380952381*G1_4_0_2_1 - 0.0761904762*G1_4_0_3_0 - 0.0253968254*G1_4_0_3_1 + 0.0761904762*G1_4_0_4_0 + 0.0253968254*G1_4_0_4_1 - 0.0253968254*G1_4_0_5_0 + 0.0253968254*G1_4_0_5_1 + 0.0126984127*G1_4_1_0_0 + 0.0126984127*G1_4_1_0_1 + 0.0126984127*G1_4_1_1_0 - 0.0126984127*G1_4_1_2_1 - 0.0253968254*G1_4_1_3_0 + 0.0253968254*G1_4_1_4_0 - 0.0253968254*G1_4_1_5_0 + 0.0126984127*G1_5_0_0_0 + 0.0126984127*G1_5_0_0_1 + 0.0126984127*G1_5_0_1_0 - 0.0126984127*G1_5_0_2_1 - 0.0253968254*G1_5_0_3_0 + 0.0253968254*G1_5_0_4_0 - 0.0253968254*G1_5_0_5_0 + 0.0380952381*G1_5_1_0_0 + 0.0380952381*G1_5_1_0_1 + 0.114285714*G1_5_1_1_0 - 0.0380952381*G1_5_1_2_1 + 0.152380952*G1_5_1_3_1 - 0.152380952*G1_5_1_5_0 - 0.152380952*G1_5_1_5_1;
    A[20] = -0.0127695922*G0_0_0_1 - 0.01480633*G0_0_1_1 + 0.0665271924*G0_1_0_1 + 0.01081322*G0_1_1_1 + 0.131427585*G0_2_0_1 + 0.00399311006*G0_2_1_1 - 0.0204313475*G0_3_0_1 - 0.105101056*G0_3_1_1 + 0.106443508*G0_4_0_1 + 0.0767564165*G0_4_1_1 + 0.210284136*G0_5_0_1 + 0.0283446393*G0_5_1_1 - 0.0127695922*G0_6_0_1 - 0.11656999*G0_6_1_1 + 0.0665271924*G0_7_0_1 + 0.0851323007*G0_7_1_1 + 0.131427585*G0_8_0_1 + 0.031437689*G0_8_1_1 + 0.0126984127*G1_0_0_1_0 - 0.0253968254*G1_0_0_2_1 - 0.0380952381*G1_0_0_3_0 + 0.0380952381*G1_0_0_4_0 + 0.0253968254*G1_0_0_4_1 - 0.0126984127*G1_0_0_5_0 + 0.00317460317*G1_0_1_0_0 + 0.00317460317*G1_0_1_0_1 + 0.00952380952*G1_0_1_1_0 - 0.00317460317*G1_0_1_2_1 + 0.0126984127*G1_0_1_3_1 - 0.0126984127*G1_0_1_5_0 - 0.0126984127*G1_0_1_5_1 - 0.0126984127*G1_1_0_0_0 - 0.0126984127*G1_1_0_0_1 - 0.0253968254*G1_1_0_2_1 - 0.0380952381*G1_1_0_3_0 - 0.0126984127*G1_1_0_3_1 + 0.0380952381*G1_1_0_4_0 + 0.0380952381*G1_1_0_4_1 + 0.0126984127*G1_1_0_5_0 + 0.0126984127*G1_1_0_5_1 - 0.0126984127*G1_1_1_0_0 - 0.0126984127*G1_1_1_0_1 - 0.0253968254*G1_1_1_1_0 - 0.0126984127*G1_1_1_3_0 - 0.0380952381*G1_1_1_3_1 + 0.0126984127*G1_1_1_4_0 + 0.0126984127*G1_1_1_4_1 + 0.0380952381*G1_1_1_5_0 + 0.0380952381*G1_1_1_5_1 + 0.0571428571*G1_2_0_0_0 + 0.0571428571*G1_2_0_0_1 - 0.0571428571*G1_2_0_1_0 + 0.20952381*G1_2_0_2_1 + 0.304761905*G1_2_0_3_0 + 0.0380952381*G1_2_0_3_1 - 0.304761905*G1_2_0_4_0 - 0.266666667*G1_2_0_4_1 - 0.0380952381*G1_2_0_5_1 + 0.00634920635*G1_2_1_0_0 + 0.00634920635*G1_2_1_0_1 + 0.00634920635*G1_2_1_1_0 + 0.019047619*G1_2_1_2_1 + 0.0380952381*G1_2_1_3_0 + 0.0253968254*G1_2_1_3_1 - 0.0380952381*G1_2_1_4_0 - 0.0253968254*G1_2_1_4_1 - 0.0126984127*G1_2_1_5_0 - 0.0253968254*G1_2_1_5_1 + 0.0380952381*G1_3_0_0_0 + 0.0380952381*G1_3_0_0_1 - 0.0380952381*G1_3_0_1_0 + 0.114285714*G1_3_0_2_1 + 0.152380952*G1_3_0_3_0 - 0.152380952*G1_3_0_4_0 - 0.152380952*G1_3_0_4_1 - 0.0507936508*G1_3_1_1_0 + 0.0253968254*G1_3_1_2_1 - 0.0761904762*G1_3_1_3_1 - 0.0253968254*G1_3_1_4_1 + 0.0507936508*G1_3_1_5_0 + 0.0761904762*G1_3_1_5_1 + 0.0380952381*G1_4_0_0_0 + 0.0380952381*G1_4_0_0_1 - 0.0380952381*G1_4_0_1_0 + 0.114285714*G1_4_0_2_1 + 0.152380952*G1_4_0_3_0 - 0.152380952*G1_4_0_4_0 - 0.152380952*G1_4_0_4_1 + 0.0126984127*G1_4_1_0_0 + 0.0126984127*G1_4_1_0_1 - 0.0126984127*G1_4_1_1_0 + 0.0126984127*G1_4_1_2_1 - 0.0253968254*G1_4_1_3_1 - 0.0253968254*G1_4_1_4_1 + 0.0253968254*G1_4_1_5_1 + 0.0126984127*G1_5_0_0_0 + 0.0126984127*G1_5_0_0_1 - 0.0126984127*G1_5_0_1_0 + 0.0126984127*G1_5_0_2_1 - 0.0253968254*G1_5_0_3_1 - 0.0253968254*G1_5_0_4_1 + 0.0253968254*G1_5_0_5_1 + 0.0126984127*G1_5_1_0_0 + 0.0126984127*G1_5_1_0_1 - 0.0380952381*G1_5_1_1_0 + 0.0126984127*G1_5_1_2_1 - 0.0253968254*G1_5_1_3_0 - 0.0761904762*G1_5_1_3_1 + 0.0253968254*G1_5_1_4_0 - 0.0253968254*G1_5_1_4_1 + 0.0253968254*G1_5_1_5_0 + 0.0761904762*G1_5_1_5_1;
    A[21] = 0.00700835027*G0_0_0_0 + 0.00812617548*G0_0_0_1 + 0.00812617548*G0_0_1_0 + 0.00942229274*G0_0_1_1 + 0.170823456*G0_1_0_0 + 0.0277653623*G0_1_0_1 + 0.0277653623*G0_1_1_0 + 0.00451293614*G0_1_1_1 + 0.192538564*G0_2_0_0 + 0.00584981972*G0_2_0_1 + 0.00584981972*G0_2_1_0 + 0.000177732658*G0_2_1_1 + 0.0112133604*G0_3_0_0 + 0.0576827357*G0_3_0_1 + 0.0576827357*G0_3_1_0 + 0.296726215*G0_3_1_1 + 0.273317529*G0_4_0_0 + 0.197089278*G0_4_0_1 + 0.197089278*G0_4_1_0 + 0.142121084*G0_4_1_1 + 0.308061703*G0_5_0_0 + 0.0415242824*G0_5_0_1 + 0.0415242824*G0_5_1_0 + 0.00559714502*G0_5_1_1 + 0.00700835027*G0_6_0_0 + 0.0639772442*G0_6_0_1 + 0.0639772442*G0_6_1_0 + 0.584030138*G0_6_1_1 + 0.170823456*G0_7_0_0 + 0.218596235*G0_7_0_1 + 0.218596235*G0_7_1_0 + 0.279729232*G0_7_1_1 + 0.192538564*G0_8_0_0 + 0.0460555333*G0_8_0_1 + 0.0460555333*G0_8_1_0 + 0.0110165574*G0_8_1_1 - 0.0126984127*G1_0_0_0_0 - 0.0126984127*G1_0_0_0_1 - 0.0126984127*G1_0_0_1_0 - 0.0380952381*G1_0_0_2_1 - 0.0761904762*G1_0_0_3_0 - 0.0507936508*G1_0_0_3_1 + 0.0761904762*G1_0_0_4_0 + 0.0507936508*G1_0_0_4_1 + 0.0253968254*G1_0_0_5_0 + 0.0507936508*G1_0_0_5_1 - 0.0126984127*G1_0_1_0_0 - 0.0126984127*G1_0_1_0_1 - 0.0380952381*G1_0_1_1_0 - 0.0126984127*G1_0_1_2_1 - 0.0507936508*G1_0_1_3_0 - 0.0761904762*G1_0_1_3_1 + 0.0507936508*G1_0_1_4_0 + 0.0253968254*G1_0_1_4_1 + 0.0507936508*G1_0_1_5_0 + 0.0761904762*G1_0_1_5_1 + 0.0253968254*G1_1_0_1_0 - 0.0507936508*G1_1_0_2_1 - 0.0761904762*G1_1_0_3_0 + 0.0761904762*G1_1_0_4_0 + 0.0507936508*G1_1_0_4_1 - 0.0253968254*G1_1_0_5_0 + 0.0380952381*G1_1_1_0_0 + 0.0380952381*G1_1_1_0_1 + 0.114285714*G1_1_1_1_0 - 0.0380952381*G1_1_1_2_1 + 0.152380952*G1_1_1_3_1 - 0.152380952*G1_1_1_5_0 - 0.152380952*G1_1_1_5_1 + 0.0380952381*G1_2_0_0_0 + 0.0380952381*G1_2_0_0_1 - 0.0380952381*G1_2_0_1_0 + 0.114285714*G1_2_0_2_1 + 0.152380952*G1_2_0_3_0 - 0.152380952*G1_2_0_4_0 - 0.152380952*G1_2_0_4_1 - 0.0507936508*G1_2_1_1_0 + 0.0253968254*G1_2_1_2_1 - 0.0761904762*G1_2_1_3_1 - 0.0253968254*G1_2_1_4_1 + 0.0507936508*G1_2_1_5_0 + 0.0761904762*G1_2_1_5_1 + 0.152380952*G1_3_0_0_0 + 0.152380952*G1_3_0_0_1 + 0.152380952*G1_3_0_1_0 + 0.304761905*G1_3_0_2_1 + 0.60952381*G1_3_0_3_0 + 0.457142857*G1_3_0_3_1 - 0.60952381*G1_3_0_4_0 - 0.457142857*G1_3_0_4_1 - 0.304761905*G1_3_0_5_0 - 0.457142857*G1_3_0_5_1 + 0.152380952*G1_3_1_0_0 + 0.152380952*G1_3_1_0_1 + 0.304761905*G1_3_1_1_0 + 0.152380952*G1_3_1_2_1 + 0.457142857*G1_3_1_3_0 + 0.60952381*G1_3_1_3_1 - 0.457142857*G1_3_1_4_0 - 0.304761905*G1_3_1_4_1 - 0.457142857*G1_3_1_5_0 - 0.60952381*G1_3_1_5_1 + 0.152380952*G1_4_0_2_1 + 0.304761905*G1_4_0_3_0 + 0.152380952*G1_4_0_3_1 - 0.304761905*G1_4_0_4_0 - 0.152380952*G1_4_0_4_1 - 0.152380952*G1_4_0_5_1 + 0.0507936508*G1_4_1_1_0 + 0.0507936508*G1_4_1_2_1 + 0.152380952*G1_4_1_3_0 + 0.152380952*G1_4_1_3_1 - 0.152380952*G1_4_1_4_0 - 0.0507936508*G1_4_1_4_1 - 0.0507936508*G1_4_1_5_0 - 0.152380952*G1_4_1_5_1 + 0.0507936508*G1_5_0_1_0 + 0.0507936508*G1_5_0_2_1 + 0.152380952*G1_5_0_3_0 + 0.152380952*G1_5_0_3_1 - 0.152380952*G1_5_0_4_0 - 0.0507936508*G1_5_0_4_1 - 0.0507936508*G1_5_0_5_0 - 0.152380952*G1_5_0_5_1 + 0.152380952*G1_5_1_1_0 + 0.152380952*G1_5_1_3_0 + 0.304761905*G1_5_1_3_1 - 0.152380952*G1_5_1_4_0 - 0.152380952*G1_5_1_5_0 - 0.304761905*G1_5_1_5_1;
    A[22] = -0.00700835027*G0_0_0_0 + 0.0569688939*G0_0_0_1 - 0.00812617548*G0_0_1_0 + 0.0660553783*G0_0_1_1 - 0.170823456*G0_1_0_0 + 0.0477727796*G0_1_0_1 - 0.0277653623*G0_1_1_0 + 0.0077649087*G0_1_1_1 - 0.192538564*G0_2_0_0 - 0.146483031*G0_2_0_1 - 0.00584981972*G0_2_1_0 - 0.00445053347*G0_2_1_1 - 0.0112133604*G0_3_0_0 + 0.0464693753*G0_3_0_1 - 0.0576827357*G0_3_1_0 + 0.239043479*G0_3_1_1 - 0.273317529*G0_4_0_0 - 0.076228251*G0_4_0_1 - 0.197089278*G0_4_1_0 - 0.054968194*G0_4_1_1 - 0.308061703*G0_5_0_0 - 0.266537421*G0_5_0_1 - 0.0415242824*G0_5_1_0 - 0.0359271374*G0_5_1_1 - 0.00700835027*G0_6_0_0 + 0.0011178252*G0_6_0_1 - 0.0639772442*G0_6_1_0 + 0.0102043096*G0_6_1_1 - 0.170823456*G0_7_0_0 - 0.143058093*G0_7_0_1 - 0.218596235*G0_7_1_0 - 0.183065964*G0_7_1_1 - 0.192538564*G0_8_0_0 - 0.186688745*G0_8_0_1 - 0.0460555333*G0_8_1_0 - 0.044656247*G0_8_1_1 - 0.0253968254*G1_0_0_0_0 - 0.0253968254*G1_0_0_0_1 - 0.0507936508*G1_0_0_2_1 - 0.0761904762*G1_0_0_3_0 - 0.0253968254*G1_0_0_3_1 + 0.0761904762*G1_0_0_4_0 + 0.0761904762*G1_0_0_4_1 + 0.0253968254*G1_0_0_5_0 + 0.0253968254*G1_0_0_5_1 - 0.0126984127*G1_0_1_0_0 - 0.0126984127*G1_0_1_0_1 - 0.0126984127*G1_0_1_1_0 - 0.0126984127*G1_0_1_2_1 - 0.0253968254*G1_0_1_3_0 - 0.0253968254*G1_0_1_3_1 + 0.0253968254*G1_0_1_4_0 + 0.0253968254*G1_0_1_4_1 + 0.0253968254*G1_0_1_5_0 + 0.0253968254*G1_0_1_5_1 + 0.0126984127*G1_1_0_0_0 + 0.0126984127*G1_1_0_0_1 + 0.0126984127*G1_1_0_1_0 - 0.0380952381*G1_1_0_2_1 - 0.0761904762*G1_1_0_3_0 - 0.0253968254*G1_1_0_3_1 + 0.0761904762*G1_1_0_4_0 + 0.0253968254*G1_1_0_4_1 - 0.0253968254*G1_1_0_5_0 + 0.0253968254*G1_1_0_5_1 + 0.0126984127*G1_1_1_0_0 + 0.0126984127*G1_1_1_0_1 + 0.0126984127*G1_1_1_1_0 - 0.0126984127*G1_1_1_2_1 - 0.0253968254*G1_1_1_3_0 + 0.0253968254*G1_1_1_4_0 - 0.0253968254*G1_1_1_5_0 + 0.0380952381*G1_2_0_0_0 + 0.0380952381*G1_2_0_0_1 - 0.0380952381*G1_2_0_1_0 + 0.114285714*G1_2_0_2_1 + 0.152380952*G1_2_0_3_0 - 0.152380952*G1_2_0_4_0 - 0.152380952*G1_2_0_4_1 + 0.0126984127*G1_2_1_0_0 + 0.0126984127*G1_2_1_0_1 - 0.0126984127*G1_2_1_1_0 + 0.0126984127*G1_2_1_2_1 - 0.0253968254*G1_2_1_3_1 - 0.0253968254*G1_2_1_4_1 + 0.0253968254*G1_2_1_5_1 + 0.152380952*G1_3_0_2_1 + 0.304761905*G1_3_0_3_0 + 0.152380952*G1_3_0_3_1 - 0.304761905*G1_3_0_4_0 - 0.152380952*G1_3_0_4_1 - 0.152380952*G1_3_0_5_1 + 0.0507936508*G1_3_1_1_0 + 0.0507936508*G1_3_1_2_1 + 0.152380952*G1_3_1_3_0 + 0.152380952*G1_3_1_3_1 - 0.152380952*G1_3_1_4_0 - 0.0507936508*G1_3_1_4_1 - 0.0507936508*G1_3_1_5_0 - 0.152380952*G1_3_1_5_1 - 0.152380952*G1_4_0_0_0 - 0.152380952*G1_4_0_0_1 - 0.152380952*G1_4_0_1_0 + 0.304761905*G1_4_0_2_1 + 0.60952381*G1_4_0_3_0 + 0.152380952*G1_4_0_3_1 - 0.60952381*G1_4_0_4_0 - 0.152380952*G1_4_0_4_1 + 0.304761905*G1_4_0_5_0 - 0.152380952*G1_4_0_5_1 - 0.0507936508*G1_4_1_0_0 - 0.0507936508*G1_4_1_0_1 + 0.0507936508*G1_4_1_2_1 + 0.152380952*G1_4_1_3_0 + 0.101587302*G1_4_1_3_1 - 0.152380952*G1_4_1_4_0 + 0.0507936508*G1_4_1_5_0 - 0.101587302*G1_4_1_5_1 - 0.0507936508*G1_5_0_0_0 - 0.0507936508*G1_5_0_0_1 + 0.0507936508*G1_5_0_2_1 + 0.152380952*G1_5_0_3_0 + 0.101587302*G1_5_0_3_1 - 0.152380952*G1_5_0_4_0 + 0.0507936508*G1_5_0_5_0 - 0.101587302*G1_5_0_5_1 - 0.0507936508*G1_5_1_0_0 - 0.0507936508*G1_5_1_0_1 + 0.0507936508*G1_5_1_1_0 + 0.101587302*G1_5_1_3_0 + 0.152380952*G1_5_1_3_1 - 0.101587302*G1_5_1_4_0 + 0.0507936508*G1_5_1_4_1 - 0.152380952*G1_5_1_5_1;
    A[23] = 0.0558510687*G0_0_0_0 - 0.00812617548*G0_0_0_1 + 0.0647592611*G0_0_1_0 - 0.00942229274*G0_0_1_1 + 0.190830873*G0_1_0_0 - 0.0277653623*G0_1_0_1 + 0.0310173349*G0_1_1_0 - 0.00451293614*G0_1_1_1 + 0.0402057135*G0_2_0_0 - 0.00584981972*G0_2_0_1 + 0.0012215536*G0_2_1_0 - 0.000177732658*G0_2_1_1 - 0.0576827357*G0_3_0_1 - 0.296726215*G0_3_1_1 - 0.197089278*G0_4_0_1 - 0.142121084*G0_4_1_1 - 0.0415242824*G0_5_0_1 - 0.00559714502*G0_5_1_1 - 0.0558510687*G0_6_0_0 - 0.0639772442*G0_6_0_1 - 0.509848584*G0_6_1_0 - 0.584030138*G0_6_1_1 - 0.190830873*G0_7_0_0 - 0.218596235*G0_7_0_1 - 0.244198961*G0_7_1_0 - 0.279729232*G0_7_1_1 - 0.0402057135*G0_8_0_0 - 0.0460555333*G0_8_0_1 - 0.00961727113*G0_8_1_0 - 0.0110165574*G0_8_1_1 - 0.0126984127*G1_0_0_0_0 - 0.0126984127*G1_0_0_0_1 - 0.0126984127*G1_0_0_1_0 - 0.0126984127*G1_0_0_2_1 - 0.0253968254*G1_0_0_3_0 - 0.0253968254*G1_0_0_3_1 + 0.0253968254*G1_0_0_4_0 + 0.0253968254*G1_0_0_4_1 + 0.0253968254*G1_0_0_5_0 + 0.0253968254*G1_0_0_5_1 - 0.0253968254*G1_0_1_0_0 - 0.0253968254*G1_0_1_0_1 - 0.0507936508*G1_0_1_1_0 - 0.0253968254*G1_0_1_3_0 - 0.0761904762*G1_0_1_3_1 + 0.0253968254*G1_0_1_4_0 + 0.0253968254*G1_0_1_4_1 + 0.0761904762*G1_0_1_5_0 + 0.0761904762*G1_0_1_5_1 + 0.0126984127*G1_1_0_0_0 + 0.0126984127*G1_1_0_0_1 + 0.0126984127*G1_1_0_1_0 - 0.0126984127*G1_1_0_2_1 - 0.0253968254*G1_1_0_3_0 + 0.0253968254*G1_1_0_4_0 - 0.0253968254*G1_1_0_5_0 + 0.0380952381*G1_1_1_0_0 + 0.0380952381*G1_1_1_0_1 + 0.114285714*G1_1_1_1_0 - 0.0380952381*G1_1_1_2_1 + 0.152380952*G1_1_1_3_1 - 0.152380952*G1_1_1_5_0 - 0.152380952*G1_1_1_5_1 + 0.0126984127*G1_2_0_0_0 + 0.0126984127*G1_2_0_0_1 - 0.0126984127*G1_2_0_1_0 + 0.0126984127*G1_2_0_2_1 - 0.0253968254*G1_2_0_3_1 - 0.0253968254*G1_2_0_4_1 + 0.0253968254*G1_2_0_5_1 + 0.0126984127*G1_2_1_0_0 + 0.0126984127*G1_2_1_0_1 - 0.0380952381*G1_2_1_1_0 + 0.0126984127*G1_2_1_2_1 - 0.0253968254*G1_2_1_3_0 - 0.0761904762*G1_2_1_3_1 + 0.0253968254*G1_2_1_4_0 - 0.0253968254*G1_2_1_4_1 + 0.0253968254*G1_2_1_5_0 + 0.0761904762*G1_2_1_5_1 + 0.0507936508*G1_3_0_1_0 + 0.0507936508*G1_3_0_2_1 + 0.152380952*G1_3_0_3_0 + 0.152380952*G1_3_0_3_1 - 0.152380952*G1_3_0_4_0 - 0.0507936508*G1_3_0_4_1 - 0.0507936508*G1_3_0_5_0 - 0.152380952*G1_3_0_5_1 + 0.152380952*G1_3_1_1_0 + 0.152380952*G1_3_1_3_0 + 0.304761905*G1_3_1_3_1 - 0.152380952*G1_3_1_4_0 - 0.152380952*G1_3_1_5_0 - 0.304761905*G1_3_1_5_1 - 0.0507936508*G1_4_0_0_0 - 0.0507936508*G1_4_0_0_1 + 0.0507936508*G1_4_0_2_1 + 0.152380952*G1_4_0_3_0 + 0.101587302*G1_4_0_3_1 - 0.152380952*G1_4_0_4_0 + 0.0507936508*G1_4_0_5_0 - 0.101587302*G1_4_0_5_1 - 0.0507936508*G1_4_1_0_0 - 0.0507936508*G1_4_1_0_1 + 0.0507936508*G1_4_1_1_0 + 0.101587302*G1_4_1_3_0 + 0.152380952*G1_4_1_3_1 - 0.101587302*G1_4_1_4_0 + 0.0507936508*G1_4_1_4_1 - 0.152380952*G1_4_1_5_1 - 0.0507936508*G1_5_0_0_0 - 0.0507936508*G1_5_0_0_1 + 0.0507936508*G1_5_0_1_0 + 0.101587302*G1_5_0_3_0 + 0.152380952*G1_5_0_3_1 - 0.101587302*G1_5_0_4_0 + 0.0507936508*G1_5_0_4_1 - 0.152380952*G1_5_0_5_1 - 0.152380952*G1_5_1_0_0 - 0.152380952*G1_5_1_0_1 + 0.304761905*G1_5_1_1_0 - 0.152380952*G1_5_1_2_1 + 0.152380952*G1_5_1_3_0 + 0.60952381*G1_5_1_3_1 - 0.152380952*G1_5_1_4_0 + 0.304761905*G1_5_1_4_1 - 0.152380952*G1_5_1_5_0 - 0.60952381*G1_5_1_5_1;
    A[24] = 0.0441993017*G0_0_0_0 + 0.0441993017*G0_0_0_1 - 0.359283602*G0_0_1_0 - 0.359283602*G0_0_1_1 + 0.114299972*G0_1_0_0 + 0.114299972*G0_1_0_1 - 0.0319653256*G0_1_1_0 - 0.0319653256*G0_1_1_1 - 0.0150554461*G0_2_0_0 - 0.0150554461*G0_2_0_1 - 0.0114541592*G0_2_1_0 - 0.0114541592*G0_2_1_1 + 0.0260380278*G0_3_0_0 + 0.0260380278*G0_3_0_1 - 0.107904396*G0_3_1_0 - 0.107904396*G0_3_1_1 + 0.0302152568*G0_4_0_0 + 0.0302152568*G0_4_0_1 + 0.00842703425*G0_4_1_0 + 0.00842703425*G0_4_1_1 - 0.0562532846*G0_5_0_0 - 0.0562532846*G0_5_0_1 - 0.0486707865*G0_5_1_0 - 0.0486707865*G0_5_1_1 - 0.011651767*G0_6_0_0 - 0.011651767*G0_6_0_1 + 0.00185844576*G0_6_1_0 + 0.00185844576*G0_6_1_1 - 0.076530901*G0_7_0_0 - 0.076530901*G0_7_0_1 - 0.0640916948*G0_7_1_0 - 0.0640916948*G0_7_1_1 - 0.0552611596*G0_8_0_0 - 0.0552611596*G0_8_0_1 - 0.0535821826*G0_8_1_0 - 0.0535821826*G0_8_1_1 + 0.019047619*G1_0_0_0_0 + 0.019047619*G1_0_0_0_1 + 0.00634920635*G1_0_0_1_0 - 0.00634920635*G1_0_0_2_1 - 0.0253968254*G1_0_0_3_0 - 0.0126984127*G1_0_0_3_1 + 0.0253968254*G1_0_0_4_0 - 0.0126984127*G1_0_0_4_1 - 0.0253968254*G1_0_0_5_0 + 0.0126984127*G1_0_0_5_1 - 0.19047619*G1_0_1_0_0 - 0.19047619*G1_0_1_0_1 - 0.0507936508*G1_0_1_1_0 - 0.0634920635*G1_0_1_2_1 + 0.0126984127*G1_0_1_3_0 + 0.0253968254*G1_0_1_3_1 - 0.0126984127*G1_0_1_4_0 + 0.253968254*G1_0_1_4_1 + 0.241269841*G1_0_1_5_0 - 0.0253968254*G1_0_1_5_1 - 0.00317460317*G1_1_0_0_0 - 0.00317460317*G1_1_0_0_1 + 0.00317460317*G1_1_0_1_0 - 0.00952380952*G1_1_0_2_1 - 0.0126984127*G1_1_0_3_0 + 0.0126984127*G1_1_0_4_0 + 0.0126984127*G1_1_0_4_1 + 0.0222222222*G1_1_1_0_0 + 0.0222222222*G1_1_1_0_1 + 0.00317460317*G1_1_1_1_0 + 0.00317460317*G1_1_1_2_1 - 0.0126984127*G1_1_1_3_0 - 0.0126984127*G1_1_1_3_1 + 0.0126984127*G1_1_1_4_0 - 0.0253968254*G1_1_1_4_1 - 0.0253968254*G1_1_1_5_0 + 0.0126984127*G1_1_1_5_1 - 0.0126984127*G1_2_0_1_0 + 0.0253968254*G1_2_0_2_1 + 0.0380952381*G1_2_0_3_0 - 0.0380952381*G1_2_0_4_0 - 0.0253968254*G1_2_0_4_1 + 0.0126984127*G1_2_0_5_0 + 0.0253968254*G1_2_1_0_0 + 0.0253968254*G1_2_1_0_1 + 0.0253968254*G1_2_1_2_1 + 0.0253968254*G1_2_1_3_0 - 0.0253968254*G1_2_1_4_0 - 0.0507936508*G1_2_1_4_1 - 0.0253968254*G1_2_1_5_0 + 0.0126984127*G1_3_0_0_0 + 0.0126984127*G1_3_0_0_1 + 0.0126984127*G1_3_0_1_0 + 0.0380952381*G1_3_0_2_1 + 0.0761904762*G1_3_0_3_0 + 0.0507936508*G1_3_0_3_1 - 0.0761904762*G1_3_0_4_0 - 0.0507936508*G1_3_0_4_1 - 0.0253968254*G1_3_0_5_0 - 0.0507936508*G1_3_0_5_1 + 0.0253968254*G1_3_1_2_1 + 0.0507936508*G1_3_1_3_0 + 0.0253968254*G1_3_1_3_1 - 0.0507936508*G1_3_1_4_0 - 0.0253968254*G1_3_1_4_1 - 0.0253968254*G1_3_1_5_1 + 0.0253968254*G1_4_0_0_0 + 0.0253968254*G1_4_0_0_1 + 0.0507936508*G1_4_0_2_1 + 0.0761904762*G1_4_0_3_0 + 0.0253968254*G1_4_0_3_1 - 0.0761904762*G1_4_0_4_0 - 0.0761904762*G1_4_0_4_1 - 0.0253968254*G1_4_0_5_0 - 0.0253968254*G1_4_0_5_1 - 0.0888888889*G1_4_1_0_0 - 0.0888888889*G1_4_1_0_1 - 0.0380952381*G1_4_1_1_0 + 0.0126984127*G1_4_1_2_1 + 0.0761904762*G1_4_1_3_0 + 0.0253968254*G1_4_1_3_1 - 0.0761904762*G1_4_1_4_0 + 0.0761904762*G1_4_1_4_1 + 0.126984127*G1_4_1_5_0 - 0.0253968254*G1_4_1_5_1 + 0.0126984127*G1_5_0_0_0 + 0.0126984127*G1_5_0_0_1 + 0.0126984127*G1_5_0_1_0 + 0.0126984127*G1_5_0_2_1 + 0.0253968254*G1_5_0_3_0 + 0.0253968254*G1_5_0_3_1 - 0.0253968254*G1_5_0_4_0 - 0.0253968254*G1_5_0_4_1 - 0.0253968254*G1_5_0_5_0 - 0.0253968254*G1_5_0_5_1 - 0.101587302*G1_5_1_0_0 - 0.101587302*G1_5_1_0_1 - 0.0253968254*G1_5_1_1_0 - 0.0253968254*G1_5_1_2_1 + 0.0253968254*G1_5_1_3_0 + 0.0253968254*G1_5_1_3_1 - 0.0253968254*G1_5_1_4_0 + 0.126984127*G1_5_1_4_1 + 0.126984127*G1_5_1_5_0 - 0.0253968254*G1_5_1_5_1;
    A[25] = 0.011651767*G0_0_0_0 - 0.0947139131*G0_0_1_0 + 0.076530901*G0_1_0_0 - 0.0214027626*G0_1_1_0 + 0.0552611596*G0_2_0_0 + 0.0420426016*G0_2_1_0 - 0.0260380278*G0_3_0_0 + 0.107904396*G0_3_1_0 - 0.0302152568*G0_4_0_0 - 0.00842703425*G0_4_1_0 + 0.0562532846*G0_5_0_0 + 0.0486707865*G0_5_1_0 - 0.0441993017*G0_6_0_0 + 0.00704974659*G0_6_1_0 - 0.114299972*G0_7_0_0 - 0.0957218433*G0_7_1_0 + 0.0150554461*G0_8_0_0 + 0.0145980227*G0_8_1_0 - 0.00317460317*G1_0_0_0_0 - 0.00317460317*G1_0_0_0_1 + 0.00317460317*G1_0_0_1_0 - 0.00952380952*G1_0_0_2_1 - 0.0126984127*G1_0_0_3_0 + 0.0126984127*G1_0_0_4_0 + 0.0126984127*G1_0_0_4_1 + 0.0222222222*G1_0_1_0_0 + 0.0222222222*G1_0_1_0_1 + 0.00317460317*G1_0_1_1_0 + 0.00317460317*G1_0_1_2_1 - 0.0126984127*G1_0_1_3_0 - 0.0126984127*G1_0_1_3_1 + 0.0126984127*G1_0_1_4_0 - 0.0253968254*G1_0_1_4_1 - 0.0253968254*G1_0_1_5_0 + 0.0126984127*G1_0_1_5_1 - 0.00634920635*G1_1_0_0_0 - 0.00634920635*G1_1_0_0_1 - 0.019047619*G1_1_0_1_0 - 0.00634920635*G1_1_0_2_1 - 0.0253968254*G1_1_0_3_0 - 0.0380952381*G1_1_0_3_1 + 0.0253968254*G1_1_0_4_0 + 0.0126984127*G1_1_0_4_1 + 0.0253968254*G1_1_0_5_0 + 0.0380952381*G1_1_0_5_1 - 0.0126984127*G1_1_1_0_0 - 0.0126984127*G1_1_1_0_1 - 0.0126984127*G1_1_1_2_1 - 0.0126984127*G1_1_1_3_0 + 0.0126984127*G1_1_1_4_0 + 0.0253968254*G1_1_1_4_1 + 0.0126984127*G1_1_1_5_0 + 0.0126984127*G1_2_0_0_0 + 0.0126984127*G1_2_0_0_1 + 0.0253968254*G1_2_0_2_1 + 0.0380952381*G1_2_0_3_0 + 0.0126984127*G1_2_0_3_1 - 0.0380952381*G1_2_0_4_0 - 0.0380952381*G1_2_0_4_1 - 0.0126984127*G1_2_0_5_0 - 0.0126984127*G1_2_0_5_1 + 0.00317460317*G1_2_1_0_0 + 0.00317460317*G1_2_1_0_1 - 0.00317460317*G1_2_1_1_0 + 0.0222222222*G1_2_1_2_1 + 0.0380952381*G1_2_1_3_0 + 0.0126984127*G1_2_1_3_1 - 0.0380952381*G1_2_1_4_0 - 0.0253968254*G1_2_1_4_1 - 0.0126984127*G1_2_1_5_1 - 0.0253968254*G1_3_0_1_0 + 0.0507936508*G1_3_0_2_1 + 0.0761904762*G1_3_0_3_0 - 0.0761904762*G1_3_0_4_0 - 0.0507936508*G1_3_0_4_1 + 0.0253968254*G1_3_0_5_0 + 0.0126984127*G1_3_1_0_0 + 0.0126984127*G1_3_1_0_1 - 0.0126984127*G1_3_1_1_0 + 0.0380952381*G1_3_1_2_1 + 0.0507936508*G1_3_1_3_0 - 0.0507936508*G1_3_1_4_0 - 0.0507936508*G1_3_1_4_1 - 0.0126984127*G1_4_0_0_0 - 0.0126984127*G1_4_0_0_1 - 0.0126984127*G1_4_0_1_0 + 0.0380952381*G1_4_0_2_1 + 0.0761904762*G1_4_0_3_0 + 0.0253968254*G1_4_0_3_1 - 0.0761904762*G1_4_0_4_0 - 0.0253968254*G1_4_0_4_1 + 0.0253968254*G1_4_0_5_0 - 0.0253968254*G1_4_0_5_1 + 0.0253968254*G1_4_1_0_0 + 0.0253968254*G1_4_1_0_1 + 0.0253968254*G1_4_1_2_1 + 0.0253968254*G1_4_1_3_0 - 0.0253968254*G1_4_1_4_0 - 0.0507936508*G1_4_1_4_1 - 0.0253968254*G1_4_1_5_0 - 0.0126984127*G1_5_0_0_0 - 0.0126984127*G1_5_0_0_1 - 0.0126984127*G1_5_0_1_0 + 0.0126984127*G1_5_0_2_1 + 0.0253968254*G1_5_0_3_0 - 0.0253968254*G1_5_0_4_0 + 0.0253968254*G1_5_0_5_0 + 0.0380952381*G1_5_1_0_0 + 0.0380952381*G1_5_1_0_1 + 0.0126984127*G1_5_1_1_0 + 0.0126984127*G1_5_1_2_1 - 0.0507936508*G1_5_1_4_1 - 0.0507936508*G1_5_1_5_0;
    A[26] = 0.0127695922*G0_0_0_1 - 0.103800398*G0_0_1_1 - 0.0665271924*G0_1_0_1 + 0.0186051083*G0_1_1_1 - 0.131427585*G0_2_0_1 - 0.099989896*G0_2_1_1 + 0.0204313475*G0_3_0_1 - 0.0846697083*G0_3_1_1 - 0.106443508*G0_4_0_1 - 0.0296870912*G0_4_1_1 - 0.210284136*G0_5_0_1 - 0.181939497*G0_5_1_1 + 0.0127695922*G0_6_0_1 - 0.00203673781*G0_6_1_1 - 0.0665271924*G0_7_0_1 - 0.0557139724*G0_7_1_1 - 0.131427585*G0_8_0_1 - 0.127434475*G0_8_1_1 - 0.0126984127*G1_0_0_1_0 + 0.0253968254*G1_0_0_2_1 + 0.0380952381*G1_0_0_3_0 - 0.0380952381*G1_0_0_4_0 - 0.0253968254*G1_0_0_4_1 + 0.0126984127*G1_0_0_5_0 + 0.0253968254*G1_0_1_0_0 + 0.0253968254*G1_0_1_0_1 + 0.0253968254*G1_0_1_2_1 + 0.0253968254*G1_0_1_3_0 - 0.0253968254*G1_0_1_4_0 - 0.0507936508*G1_0_1_4_1 - 0.0253968254*G1_0_1_5_0 + 0.0126984127*G1_1_0_0_0 + 0.0126984127*G1_1_0_0_1 + 0.0253968254*G1_1_0_2_1 + 0.0380952381*G1_1_0_3_0 + 0.0126984127*G1_1_0_3_1 - 0.0380952381*G1_1_0_4_0 - 0.0380952381*G1_1_0_4_1 - 0.0126984127*G1_1_0_5_0 - 0.0126984127*G1_1_0_5_1 + 0.00317460317*G1_1_1_0_0 + 0.00317460317*G1_1_1_0_1 - 0.00317460317*G1_1_1_1_0 + 0.0222222222*G1_1_1_2_1 + 0.0380952381*G1_1_1_3_0 + 0.0126984127*G1_1_1_3_1 - 0.0380952381*G1_1_1_4_0 - 0.0253968254*G1_1_1_4_1 - 0.0126984127*G1_1_1_5_1 - 0.0571428571*G1_2_0_0_0 - 0.0571428571*G1_2_0_0_1 + 0.0571428571*G1_2_0_1_0 - 0.20952381*G1_2_0_2_1 - 0.304761905*G1_2_0_3_0 - 0.0380952381*G1_2_0_3_1 + 0.304761905*G1_2_0_4_0 + 0.266666667*G1_2_0_4_1 + 0.0380952381*G1_2_0_5_1 - 0.0634920635*G1_2_1_0_0 - 0.0634920635*G1_2_1_0_1 + 0.0507936508*G1_2_1_1_0 - 0.19047619*G1_2_1_2_1 - 0.266666667*G1_2_1_3_0 - 0.0253968254*G1_2_1_3_1 + 0.266666667*G1_2_1_4_0 + 0.253968254*G1_2_1_4_1 + 0.0126984127*G1_2_1_5_0 + 0.0253968254*G1_2_1_5_1 - 0.0380952381*G1_3_0_0_0 - 0.0380952381*G1_3_0_0_1 + 0.0380952381*G1_3_0_1_0 - 0.114285714*G1_3_0_2_1 - 0.152380952*G1_3_0_3_0 + 0.152380952*G1_3_0_4_0 + 0.152380952*G1_3_0_4_1 - 0.0253968254*G1_3_1_0_0 - 0.0253968254*G1_3_1_0_1 + 0.0253968254*G1_3_1_1_0 - 0.101587302*G1_3_1_2_1 - 0.152380952*G1_3_1_3_0 - 0.0253968254*G1_3_1_3_1 + 0.152380952*G1_3_1_4_0 + 0.126984127*G1_3_1_4_1 + 0.0253968254*G1_3_1_5_1 - 0.0380952381*G1_4_0_0_0 - 0.0380952381*G1_4_0_0_1 + 0.0380952381*G1_4_0_1_0 - 0.114285714*G1_4_0_2_1 - 0.152380952*G1_4_0_3_0 + 0.152380952*G1_4_0_4_0 + 0.152380952*G1_4_0_4_1 + 0.0126984127*G1_4_1_0_0 + 0.0126984127*G1_4_1_0_1 + 0.0380952381*G1_4_1_1_0 - 0.0888888889*G1_4_1_2_1 - 0.152380952*G1_4_1_3_0 - 0.0253968254*G1_4_1_3_1 + 0.152380952*G1_4_1_4_0 + 0.0761904762*G1_4_1_4_1 - 0.0507936508*G1_4_1_5_0 + 0.0253968254*G1_4_1_5_1 - 0.0126984127*G1_5_0_0_0 - 0.0126984127*G1_5_0_0_1 + 0.0126984127*G1_5_0_1_0 - 0.0126984127*G1_5_0_2_1 + 0.0253968254*G1_5_0_3_1 + 0.0253968254*G1_5_0_4_1 - 0.0253968254*G1_5_0_5_1 + 0.0253968254*G1_5_1_0_0 + 0.0253968254*G1_5_1_0_1 - 0.0253968254*G1_5_1_3_0 - 0.0253968254*G1_5_1_3_1 + 0.0253968254*G1_5_1_4_0 - 0.0253968254*G1_5_1_4_1 - 0.0253968254*G1_5_1_5_0 + 0.0253968254*G1_5_1_5_1;
    A[27] = -0.00700835027*G0_0_0_0 - 0.00812617548*G0_0_0_1 + 0.0569688939*G0_0_1_0 + 0.0660553783*G0_0_1_1 - 0.170823456*G0_1_0_0 - 0.0277653623*G0_1_0_1 + 0.0477727796*G0_1_1_0 + 0.0077649087*G0_1_1_1 - 0.192538564*G0_2_0_0 - 0.00584981972*G0_2_0_1 - 0.146483031*G0_2_1_0 - 0.00445053347*G0_2_1_1 - 0.0112133604*G0_3_0_0 - 0.0576827357*G0_3_0_1 + 0.0464693753*G0_3_1_0 + 0.239043479*G0_3_1_1 - 0.273317529*G0_4_0_0 - 0.197089278*G0_4_0_1 - 0.076228251*G0_4_1_0 - 0.054968194*G0_4_1_1 - 0.308061703*G0_5_0_0 - 0.0415242824*G0_5_0_1 - 0.266537421*G0_5_1_0 - 0.0359271374*G0_5_1_1 - 0.00700835027*G0_6_0_0 - 0.0639772442*G0_6_0_1 + 0.0011178252*G0_6_1_0 + 0.0102043096*G0_6_1_1 - 0.170823456*G0_7_0_0 - 0.218596235*G0_7_0_1 - 0.143058093*G0_7_1_0 - 0.183065964*G0_7_1_1 - 0.192538564*G0_8_0_0 - 0.0460555333*G0_8_0_1 - 0.186688745*G0_8_1_0 - 0.044656247*G0_8_1_1 + 0.0126984127*G1_0_0_0_0 + 0.0126984127*G1_0_0_0_1 + 0.0126984127*G1_0_0_1_0 + 0.0380952381*G1_0_0_2_1 + 0.0761904762*G1_0_0_3_0 + 0.0507936508*G1_0_0_3_1 - 0.0761904762*G1_0_0_4_0 - 0.0507936508*G1_0_0_4_1 - 0.0253968254*G1_0_0_5_0 - 0.0507936508*G1_0_0_5_1 + 0.0253968254*G1_0_1_2_1 + 0.0507936508*G1_0_1_3_0 + 0.0253968254*G1_0_1_3_1 - 0.0507936508*G1_0_1_4_0 - 0.0253968254*G1_0_1_4_1 - 0.0253968254*G1_0_1_5_1 - 0.0253968254*G1_1_0_1_0 + 0.0507936508*G1_1_0_2_1 + 0.0761904762*G1_1_0_3_0 - 0.0761904762*G1_1_0_4_0 - 0.0507936508*G1_1_0_4_1 + 0.0253968254*G1_1_0_5_0 + 0.0126984127*G1_1_1_0_0 + 0.0126984127*G1_1_1_0_1 - 0.0126984127*G1_1_1_1_0 + 0.0380952381*G1_1_1_2_1 + 0.0507936508*G1_1_1_3_0 - 0.0507936508*G1_1_1_4_0 - 0.0507936508*G1_1_1_4_1 - 0.0380952381*G1_2_0_0_0 - 0.0380952381*G1_2_0_0_1 + 0.0380952381*G1_2_0_1_0 - 0.114285714*G1_2_0_2_1 - 0.152380952*G1_2_0_3_0 + 0.152380952*G1_2_0_4_0 + 0.152380952*G1_2_0_4_1 - 0.0253968254*G1_2_1_0_0 - 0.0253968254*G1_2_1_0_1 + 0.0253968254*G1_2_1_1_0 - 0.101587302*G1_2_1_2_1 - 0.152380952*G1_2_1_3_0 - 0.0253968254*G1_2_1_3_1 + 0.152380952*G1_2_1_4_0 + 0.126984127*G1_2_1_4_1 + 0.0253968254*G1_2_1_5_1 - 0.152380952*G1_3_0_0_0 - 0.152380952*G1_3_0_0_1 - 0.152380952*G1_3_0_1_0 - 0.304761905*G1_3_0_2_1 - 0.60952381*G1_3_0_3_0 - 0.457142857*G1_3_0_3_1 + 0.60952381*G1_3_0_4_0 + 0.457142857*G1_3_0_4_1 + 0.304761905*G1_3_0_5_0 + 0.457142857*G1_3_0_5_1 - 0.152380952*G1_3_1_0_0 - 0.152380952*G1_3_1_0_1 - 0.101587302*G1_3_1_1_0 - 0.253968254*G1_3_1_2_1 - 0.457142857*G1_3_1_3_0 - 0.304761905*G1_3_1_3_1 + 0.457142857*G1_3_1_4_0 + 0.406349206*G1_3_1_4_1 + 0.253968254*G1_3_1_5_0 + 0.304761905*G1_3_1_5_1 - 0.152380952*G1_4_0_2_1 - 0.304761905*G1_4_0_3_0 - 0.152380952*G1_4_0_3_1 + 0.304761905*G1_4_0_4_0 + 0.152380952*G1_4_0_4_1 + 0.152380952*G1_4_0_5_1 - 0.0507936508*G1_4_1_0_0 - 0.0507936508*G1_4_1_0_1 - 0.101587302*G1_4_1_2_1 - 0.152380952*G1_4_1_3_0 - 0.0507936508*G1_4_1_3_1 + 0.152380952*G1_4_1_4_0 + 0.152380952*G1_4_1_4_1 + 0.0507936508*G1_4_1_5_0 + 0.0507936508*G1_4_1_5_1 - 0.0507936508*G1_5_0_1_0 - 0.0507936508*G1_5_0_2_1 - 0.152380952*G1_5_0_3_0 - 0.152380952*G1_5_0_3_1 + 0.152380952*G1_5_0_4_0 + 0.0507936508*G1_5_0_4_1 + 0.0507936508*G1_5_0_5_0 + 0.152380952*G1_5_0_5_1 - 0.0507936508*G1_5_1_0_0 - 0.0507936508*G1_5_1_0_1 - 0.0507936508*G1_5_1_2_1 - 0.0507936508*G1_5_1_3_0 + 0.0507936508*G1_5_1_4_0 + 0.101587302*G1_5_1_4_1 + 0.0507936508*G1_5_1_5_0;
    A[28] = 0.00700835027*G0_0_0_0 - 0.0569688939*G0_0_0_1 - 0.0569688939*G0_0_1_0 + 0.463084*G0_0_1_1 + 0.170823456*G0_1_0_0 - 0.0477727796*G0_1_0_1 - 0.0477727796*G0_1_1_0 + 0.0133602172*G0_1_1_1 + 0.192538564*G0_2_0_0 + 0.146483031*G0_2_0_1 + 0.146483031*G0_2_1_0 + 0.111444055*G0_2_1_1 + 0.0112133604*G0_3_0_0 - 0.0464693753*G0_3_0_1 - 0.0464693753*G0_3_1_0 + 0.192574104*G0_3_1_1 + 0.273317529*G0_4_0_0 + 0.076228251*G0_4_0_1 + 0.076228251*G0_4_1_0 + 0.021260057*G0_4_1_1 + 0.308061703*G0_5_0_0 + 0.266537421*G0_5_0_1 + 0.266537421*G0_5_1_0 + 0.230610283*G0_5_1_1 + 0.00700835027*G0_6_0_0 - 0.0011178252*G0_6_0_1 - 0.0011178252*G0_6_1_0 + 0.000178292057*G0_6_1_1 + 0.170823456*G0_7_0_0 + 0.143058093*G0_7_0_1 + 0.143058093*G0_7_1_0 + 0.119805667*G0_7_1_1 + 0.192538564*G0_8_0_0 + 0.186688745*G0_8_0_1 + 0.186688745*G0_8_1_0 + 0.181016658*G0_8_1_1 + 0.0253968254*G1_0_0_0_0 + 0.0253968254*G1_0_0_0_1 + 0.0507936508*G1_0_0_2_1 + 0.0761904762*G1_0_0_3_0 + 0.0253968254*G1_0_0_3_1 - 0.0761904762*G1_0_0_4_0 - 0.0761904762*G1_0_0_4_1 - 0.0253968254*G1_0_0_5_0 - 0.0253968254*G1_0_0_5_1 - 0.0888888889*G1_0_1_0_0 - 0.0888888889*G1_0_1_0_1 - 0.0380952381*G1_0_1_1_0 + 0.0126984127*G1_0_1_2_1 + 0.0761904762*G1_0_1_3_0 + 0.0253968254*G1_0_1_3_1 - 0.0761904762*G1_0_1_4_0 + 0.0761904762*G1_0_1_4_1 + 0.126984127*G1_0_1_5_0 - 0.0253968254*G1_0_1_5_1 - 0.0126984127*G1_1_0_0_0 - 0.0126984127*G1_1_0_0_1 - 0.0126984127*G1_1_0_1_0 + 0.0380952381*G1_1_0_2_1 + 0.0761904762*G1_1_0_3_0 + 0.0253968254*G1_1_0_3_1 - 0.0761904762*G1_1_0_4_0 - 0.0253968254*G1_1_0_4_1 + 0.0253968254*G1_1_0_5_0 - 0.0253968254*G1_1_0_5_1 + 0.0253968254*G1_1_1_0_0 + 0.0253968254*G1_1_1_0_1 + 0.0253968254*G1_1_1_2_1 + 0.0253968254*G1_1_1_3_0 - 0.0253968254*G1_1_1_4_0 - 0.0507936508*G1_1_1_4_1 - 0.0253968254*G1_1_1_5_0 - 0.0380952381*G1_2_0_0_0 - 0.0380952381*G1_2_0_0_1 + 0.0380952381*G1_2_0_1_0 - 0.114285714*G1_2_0_2_1 - 0.152380952*G1_2_0_3_0 + 0.152380952*G1_2_0_4_0 + 0.152380952*G1_2_0_4_1 + 0.0126984127*G1_2_1_0_0 + 0.0126984127*G1_2_1_0_1 + 0.0380952381*G1_2_1_1_0 - 0.0888888889*G1_2_1_2_1 - 0.152380952*G1_2_1_3_0 - 0.0253968254*G1_2_1_3_1 + 0.152380952*G1_2_1_4_0 + 0.0761904762*G1_2_1_4_1 - 0.0507936508*G1_2_1_5_0 + 0.0253968254*G1_2_1_5_1 - 0.152380952*G1_3_0_2_1 - 0.304761905*G1_3_0_3_0 - 0.152380952*G1_3_0_3_1 + 0.304761905*G1_3_0_4_0 + 0.152380952*G1_3_0_4_1 + 0.152380952*G1_3_0_5_1 - 0.0507936508*G1_3_1_0_0 - 0.0507936508*G1_3_1_0_1 - 0.101587302*G1_3_1_2_1 - 0.152380952*G1_3_1_3_0 - 0.0507936508*G1_3_1_3_1 + 0.152380952*G1_3_1_4_0 + 0.152380952*G1_3_1_4_1 + 0.0507936508*G1_3_1_5_0 + 0.0507936508*G1_3_1_5_1 + 0.152380952*G1_4_0_0_0 + 0.152380952*G1_4_0_0_1 + 0.152380952*G1_4_0_1_0 - 0.304761905*G1_4_0_2_1 - 0.60952381*G1_4_0_3_0 - 0.152380952*G1_4_0_3_1 + 0.60952381*G1_4_0_4_0 + 0.152380952*G1_4_0_4_1 - 0.304761905*G1_4_0_5_0 + 0.152380952*G1_4_0_5_1 - 0.152380952*G1_4_1_0_0 - 0.152380952*G1_4_1_0_1 - 0.152380952*G1_4_1_2_1 - 0.152380952*G1_4_1_3_0 + 0.152380952*G1_4_1_4_0 + 0.304761905*G1_4_1_4_1 + 0.152380952*G1_4_1_5_0 + 0.0507936508*G1_5_0_0_0 + 0.0507936508*G1_5_0_0_1 - 0.0507936508*G1_5_0_2_1 - 0.152380952*G1_5_0_3_0 - 0.101587302*G1_5_0_3_1 + 0.152380952*G1_5_0_4_0 - 0.0507936508*G1_5_0_5_0 + 0.101587302*G1_5_0_5_1 - 0.101587302*G1_5_1_0_0 - 0.101587302*G1_5_1_0_1 - 0.0507936508*G1_5_1_2_1 + 0.0507936508*G1_5_1_3_1 + 0.152380952*G1_5_1_4_1 + 0.101587302*G1_5_1_5_0 - 0.0507936508*G1_5_1_5_1;
    A[29] = -0.0558510687*G0_0_0_0 + 0.00812617548*G0_0_0_1 + 0.453997515*G0_0_1_0 - 0.0660553783*G0_0_1_1 - 0.190830873*G0_1_0_0 + 0.0277653623*G0_1_0_1 + 0.0533680882*G0_1_1_0 - 0.0077649087*G0_1_1_1 - 0.0402057135*G0_2_0_0 + 0.00584981972*G0_2_0_1 - 0.0305884424*G0_2_1_0 + 0.00445053347*G0_2_1_1 + 0.0576827357*G0_3_0_1 - 0.239043479*G0_3_1_1 + 0.197089278*G0_4_0_1 + 0.054968194*G0_4_1_1 + 0.0415242824*G0_5_0_1 + 0.0359271374*G0_5_1_1 + 0.0558510687*G0_6_0_0 + 0.0639772442*G0_6_0_1 - 0.00890819235*G0_6_1_0 - 0.0102043096*G0_6_1_1 + 0.190830873*G0_7_0_0 + 0.218596235*G0_7_0_1 + 0.159813538*G0_7_1_0 + 0.183065964*G0_7_1_1 + 0.0402057135*G0_8_0_0 + 0.0460555333*G0_8_0_1 + 0.0389841599*G0_8_1_0 + 0.044656247*G0_8_1_1 + 0.0126984127*G1_0_0_0_0 + 0.0126984127*G1_0_0_0_1 + 0.0126984127*G1_0_0_1_0 + 0.0126984127*G1_0_0_2_1 + 0.0253968254*G1_0_0_3_0 + 0.0253968254*G1_0_0_3_1 - 0.0253968254*G1_0_0_4_0 - 0.0253968254*G1_0_0_4_1 - 0.0253968254*G1_0_0_5_0 - 0.0253968254*G1_0_0_5_1 - 0.101587302*G1_0_1_0_0 - 0.101587302*G1_0_1_0_1 - 0.0253968254*G1_0_1_1_0 - 0.0253968254*G1_0_1_2_1 + 0.0253968254*G1_0_1_3_0 + 0.0253968254*G1_0_1_3_1 - 0.0253968254*G1_0_1_4_0 + 0.126984127*G1_0_1_4_1 + 0.126984127*G1_0_1_5_0 - 0.0253968254*G1_0_1_5_1 - 0.0126984127*G1_1_0_0_0 - 0.0126984127*G1_1_0_0_1 - 0.0126984127*G1_1_0_1_0 + 0.0126984127*G1_1_0_2_1 + 0.0253968254*G1_1_0_3_0 - 0.0253968254*G1_1_0_4_0 + 0.0253968254*G1_1_0_5_0 + 0.0380952381*G1_1_1_0_0 + 0.0380952381*G1_1_1_0_1 + 0.0126984127*G1_1_1_1_0 + 0.0126984127*G1_1_1_2_1 - 0.0507936508*G1_1_1_4_1 - 0.0507936508*G1_1_1_5_0 - 0.0126984127*G1_2_0_0_0 - 0.0126984127*G1_2_0_0_1 + 0.0126984127*G1_2_0_1_0 - 0.0126984127*G1_2_0_2_1 + 0.0253968254*G1_2_0_3_1 + 0.0253968254*G1_2_0_4_1 - 0.0253968254*G1_2_0_5_1 + 0.0253968254*G1_2_1_0_0 + 0.0253968254*G1_2_1_0_1 - 0.0253968254*G1_2_1_3_0 - 0.0253968254*G1_2_1_3_1 + 0.0253968254*G1_2_1_4_0 - 0.0253968254*G1_2_1_4_1 - 0.0253968254*G1_2_1_5_0 + 0.0253968254*G1_2_1_5_1 - 0.0507936508*G1_3_0_1_0 - 0.0507936508*G1_3_0_2_1 - 0.152380952*G1_3_0_3_0 - 0.152380952*G1_3_0_3_1 + 0.152380952*G1_3_0_4_0 + 0.0507936508*G1_3_0_4_1 + 0.0507936508*G1_3_0_5_0 + 0.152380952*G1_3_0_5_1 - 0.0507936508*G1_3_1_0_0 - 0.0507936508*G1_3_1_0_1 - 0.0507936508*G1_3_1_2_1 - 0.0507936508*G1_3_1_3_0 + 0.0507936508*G1_3_1_4_0 + 0.101587302*G1_3_1_4_1 + 0.0507936508*G1_3_1_5_0 + 0.0507936508*G1_4_0_0_0 + 0.0507936508*G1_4_0_0_1 - 0.0507936508*G1_4_0_2_1 - 0.152380952*G1_4_0_3_0 - 0.101587302*G1_4_0_3_1 + 0.152380952*G1_4_0_4_0 - 0.0507936508*G1_4_0_5_0 + 0.101587302*G1_4_0_5_1 - 0.101587302*G1_4_1_0_0 - 0.101587302*G1_4_1_0_1 - 0.0507936508*G1_4_1_2_1 + 0.0507936508*G1_4_1_3_1 + 0.152380952*G1_4_1_4_1 + 0.101587302*G1_4_1_5_0 - 0.0507936508*G1_4_1_5_1 + 0.0507936508*G1_5_0_0_0 + 0.0507936508*G1_5_0_0_1 - 0.0507936508*G1_5_0_1_0 - 0.101587302*G1_5_0_3_0 - 0.152380952*G1_5_0_3_1 + 0.101587302*G1_5_0_4_0 - 0.0507936508*G1_5_0_4_1 + 0.152380952*G1_5_0_5_1 - 0.253968254*G1_5_1_0_0 - 0.253968254*G1_5_1_0_1 + 0.101587302*G1_5_1_1_0 - 0.152380952*G1_5_1_2_1 + 0.0507936508*G1_5_1_3_0 + 0.304761905*G1_5_1_3_1 - 0.0507936508*G1_5_1_4_0 + 0.406349206*G1_5_1_4_1 + 0.152380952*G1_5_1_5_0 - 0.304761905*G1_5_1_5_1;
    A[30] = -0.352233855*G0_0_0_0 - 0.352233855*G0_0_0_1 + 0.0512490483*G0_0_1_0 + 0.0512490483*G0_0_1_1 - 0.127687169*G0_1_0_0 - 0.127687169*G0_1_0_1 + 0.0185781287*G0_1_1_0 + 0.0185781287*G0_1_1_1 + 0.00314386343*G0_2_0_0 + 0.00314386343*G0_2_0_1 - 0.000457423404*G0_2_1_0 - 0.000457423404*G0_2_1_1 + 0.133942424*G0_3_1_0 + 0.133942424*G0_3_1_1 + 0.0217882225*G0_4_1_0 + 0.0217882225*G0_4_1_1 - 0.00758249808*G0_5_1_0 - 0.00758249808*G0_5_1_1 - 0.0928554674*G0_6_0_0 - 0.0928554674*G0_6_0_1 - 0.10636568*G0_6_1_0 - 0.10636568*G0_6_1_1 - 0.0854944574*G0_7_0_0 - 0.0854944574*G0_7_0_1 - 0.0979336636*G0_7_1_0 - 0.0979336636*G0_7_1_1 - 0.011539581*G0_8_0_0 - 0.011539581*G0_8_0_1 - 0.013218558*G0_8_1_0 - 0.013218558*G0_8_1_1 - 0.19047619*G1_0_0_0_0 - 0.19047619*G1_0_0_0_1 - 0.0634920635*G1_0_0_1_0 - 0.0507936508*G1_0_0_2_1 + 0.0253968254*G1_0_0_3_0 + 0.0126984127*G1_0_0_3_1 - 0.0253968254*G1_0_0_4_0 + 0.241269841*G1_0_0_4_1 + 0.253968254*G1_0_0_5_0 - 0.0126984127*G1_0_0_5_1 + 0.019047619*G1_0_1_0_0 + 0.019047619*G1_0_1_0_1 - 0.00634920635*G1_0_1_1_0 + 0.00634920635*G1_0_1_2_1 - 0.0126984127*G1_0_1_3_0 - 0.0253968254*G1_0_1_3_1 + 0.0126984127*G1_0_1_4_0 - 0.0253968254*G1_0_1_4_1 - 0.0126984127*G1_0_1_5_0 + 0.0253968254*G1_0_1_5_1 + 0.0253968254*G1_1_0_0_0 + 0.0253968254*G1_1_0_0_1 + 0.0253968254*G1_1_0_1_0 + 0.0253968254*G1_1_0_3_1 - 0.0253968254*G1_1_0_4_1 - 0.0507936508*G1_1_0_5_0 - 0.0253968254*G1_1_0_5_1 + 0.0253968254*G1_1_1_1_0 - 0.0126984127*G1_1_1_2_1 + 0.0380952381*G1_1_1_3_1 + 0.0126984127*G1_1_1_4_1 - 0.0253968254*G1_1_1_5_0 - 0.0380952381*G1_1_1_5_1 + 0.0222222222*G1_2_0_0_0 + 0.0222222222*G1_2_0_0_1 + 0.00317460317*G1_2_0_1_0 + 0.00317460317*G1_2_0_2_1 - 0.0126984127*G1_2_0_3_0 - 0.0126984127*G1_2_0_3_1 + 0.0126984127*G1_2_0_4_0 - 0.0253968254*G1_2_0_4_1 - 0.0253968254*G1_2_0_5_0 + 0.0126984127*G1_2_0_5_1 - 0.00317460317*G1_2_1_0_0 - 0.00317460317*G1_2_1_0_1 - 0.00952380952*G1_2_1_1_0 + 0.00317460317*G1_2_1_2_1 - 0.0126984127*G1_2_1_3_1 + 0.0126984127*G1_2_1_5_0 + 0.0126984127*G1_2_1_5_1 + 0.0253968254*G1_3_0_1_0 + 0.0253968254*G1_3_0_3_0 + 0.0507936508*G1_3_0_3_1 - 0.0253968254*G1_3_0_4_0 - 0.0253968254*G1_3_0_5_0 - 0.0507936508*G1_3_0_5_1 + 0.0126984127*G1_3_1_0_0 + 0.0126984127*G1_3_1_0_1 + 0.0380952381*G1_3_1_1_0 + 0.0126984127*G1_3_1_2_1 + 0.0507936508*G1_3_1_3_0 + 0.0761904762*G1_3_1_3_1 - 0.0507936508*G1_3_1_4_0 - 0.0253968254*G1_3_1_4_1 - 0.0507936508*G1_3_1_5_0 - 0.0761904762*G1_3_1_5_1 - 0.101587302*G1_4_0_0_0 - 0.101587302*G1_4_0_0_1 - 0.0253968254*G1_4_0_1_0 - 0.0253968254*G1_4_0_2_1 + 0.0253968254*G1_4_0_3_0 + 0.0253968254*G1_4_0_3_1 - 0.0253968254*G1_4_0_4_0 + 0.126984127*G1_4_0_4_1 + 0.126984127*G1_4_0_5_0 - 0.0253968254*G1_4_0_5_1 + 0.0126984127*G1_4_1_0_0 + 0.0126984127*G1_4_1_0_1 + 0.0126984127*G1_4_1_1_0 + 0.0126984127*G1_4_1_2_1 + 0.0253968254*G1_4_1_3_0 + 0.0253968254*G1_4_1_3_1 - 0.0253968254*G1_4_1_4_0 - 0.0253968254*G1_4_1_4_1 - 0.0253968254*G1_4_1_5_0 - 0.0253968254*G1_4_1_5_1 - 0.0888888889*G1_5_0_0_0 - 0.0888888889*G1_5_0_0_1 + 0.0126984127*G1_5_0_1_0 - 0.0380952381*G1_5_0_2_1 + 0.0253968254*G1_5_0_3_0 + 0.0761904762*G1_5_0_3_1 - 0.0253968254*G1_5_0_4_0 + 0.126984127*G1_5_0_4_1 + 0.0761904762*G1_5_0_5_0 - 0.0761904762*G1_5_0_5_1 + 0.0253968254*G1_5_1_0_0 + 0.0253968254*G1_5_1_0_1 + 0.0507936508*G1_5_1_1_0 + 0.0253968254*G1_5_1_3_0 + 0.0761904762*G1_5_1_3_1 - 0.0253968254*G1_5_1_4_0 - 0.0253968254*G1_5_1_4_1 - 0.0761904762*G1_5_1_5_0 - 0.0761904762*G1_5_1_5_1;
    A[31] = -0.0928554674*G0_0_0_0 + 0.0135102128*G0_0_1_0 - 0.0854944574*G0_1_0_0 + 0.0124392062*G0_1_1_0 - 0.011539581*G0_2_0_0 + 0.001678977*G0_2_1_0 - 0.133942424*G0_3_1_0 - 0.0217882225*G0_4_1_0 + 0.00758249808*G0_5_1_0 - 0.352233855*G0_6_0_0 - 0.403482904*G0_6_1_0 - 0.127687169*G0_7_0_0 - 0.146265298*G0_7_1_0 + 0.00314386343*G0_8_0_0 + 0.00360128684*G0_8_1_0 + 0.0253968254*G1_0_0_0_0 + 0.0253968254*G1_0_0_0_1 + 0.0253968254*G1_0_0_1_0 + 0.0253968254*G1_0_0_3_1 - 0.0253968254*G1_0_0_4_1 - 0.0507936508*G1_0_0_5_0 - 0.0253968254*G1_0_0_5_1 + 0.0253968254*G1_0_1_1_0 - 0.0126984127*G1_0_1_2_1 + 0.0380952381*G1_0_1_3_1 + 0.0126984127*G1_0_1_4_1 - 0.0253968254*G1_0_1_5_0 - 0.0380952381*G1_0_1_5_1 - 0.0634920635*G1_1_0_0_0 - 0.0634920635*G1_1_0_0_1 - 0.19047619*G1_1_0_1_0 + 0.0507936508*G1_1_0_2_1 - 0.0253968254*G1_1_0_3_0 - 0.266666667*G1_1_0_3_1 + 0.0253968254*G1_1_0_4_0 + 0.0126984127*G1_1_0_4_1 + 0.253968254*G1_1_0_5_0 + 0.266666667*G1_1_0_5_1 - 0.0571428571*G1_1_1_0_0 - 0.0571428571*G1_1_1_0_1 - 0.20952381*G1_1_1_1_0 + 0.0571428571*G1_1_1_2_1 - 0.0380952381*G1_1_1_3_0 - 0.304761905*G1_1_1_3_1 + 0.0380952381*G1_1_1_4_0 + 0.266666667*G1_1_1_5_0 + 0.304761905*G1_1_1_5_1 + 0.00317460317*G1_2_0_0_0 + 0.00317460317*G1_2_0_0_1 + 0.0222222222*G1_2_0_1_0 - 0.00317460317*G1_2_0_2_1 + 0.0126984127*G1_2_0_3_0 + 0.0380952381*G1_2_0_3_1 - 0.0126984127*G1_2_0_4_0 - 0.0253968254*G1_2_0_5_0 - 0.0380952381*G1_2_0_5_1 + 0.0126984127*G1_2_1_0_0 + 0.0126984127*G1_2_1_0_1 + 0.0253968254*G1_2_1_1_0 + 0.0126984127*G1_2_1_3_0 + 0.0380952381*G1_2_1_3_1 - 0.0126984127*G1_2_1_4_0 - 0.0126984127*G1_2_1_4_1 - 0.0380952381*G1_2_1_5_0 - 0.0380952381*G1_2_1_5_1 - 0.0253968254*G1_3_0_0_0 - 0.0253968254*G1_3_0_0_1 - 0.101587302*G1_3_0_1_0 + 0.0253968254*G1_3_0_2_1 - 0.0253968254*G1_3_0_3_0 - 0.152380952*G1_3_0_3_1 + 0.0253968254*G1_3_0_4_0 + 0.126984127*G1_3_0_5_0 + 0.152380952*G1_3_0_5_1 - 0.0380952381*G1_3_1_0_0 - 0.0380952381*G1_3_1_0_1 - 0.114285714*G1_3_1_1_0 + 0.0380952381*G1_3_1_2_1 - 0.152380952*G1_3_1_3_1 + 0.152380952*G1_3_1_5_0 + 0.152380952*G1_3_1_5_1 + 0.0253968254*G1_4_0_0_0 + 0.0253968254*G1_4_0_0_1 - 0.0253968254*G1_4_0_3_0 - 0.0253968254*G1_4_0_3_1 + 0.0253968254*G1_4_0_4_0 - 0.0253968254*G1_4_0_4_1 - 0.0253968254*G1_4_0_5_0 + 0.0253968254*G1_4_0_5_1 - 0.0126984127*G1_4_1_0_0 - 0.0126984127*G1_4_1_0_1 - 0.0126984127*G1_4_1_1_0 + 0.0126984127*G1_4_1_2_1 + 0.0253968254*G1_4_1_3_0 - 0.0253968254*G1_4_1_4_0 + 0.0253968254*G1_4_1_5_0 + 0.0126984127*G1_5_0_0_0 + 0.0126984127*G1_5_0_0_1 - 0.0888888889*G1_5_0_1_0 + 0.0380952381*G1_5_0_2_1 - 0.0253968254*G1_5_0_3_0 - 0.152380952*G1_5_0_3_1 + 0.0253968254*G1_5_0_4_0 - 0.0507936508*G1_5_0_4_1 + 0.0761904762*G1_5_0_5_0 + 0.152380952*G1_5_0_5_1 - 0.0380952381*G1_5_1_0_0 - 0.0380952381*G1_5_1_0_1 - 0.114285714*G1_5_1_1_0 + 0.0380952381*G1_5_1_2_1 - 0.152380952*G1_5_1_3_1 + 0.152380952*G1_5_1_5_0 + 0.152380952*G1_5_1_5_1;
    A[32] = -0.10176366*G0_0_0_1 + 0.01480633*G0_0_1_1 + 0.0743190807*G0_1_0_1 - 0.01081322*G0_1_1_1 + 0.027444579*G0_2_0_1 - 0.00399311006*G0_2_1_1 + 0.105101056*G0_3_1_1 - 0.0767564165*G0_4_1_1 - 0.0283446393*G0_5_1_1 + 0.10176366*G0_6_0_1 + 0.11656999*G0_6_1_1 - 0.0743190807*G0_7_0_1 - 0.0851323007*G0_7_1_1 - 0.027444579*G0_8_0_1 - 0.031437689*G0_8_1_1 + 0.0222222222*G1_0_0_0_0 + 0.0222222222*G1_0_0_0_1 + 0.00317460317*G1_0_0_1_0 + 0.00317460317*G1_0_0_2_1 - 0.0126984127*G1_0_0_3_0 - 0.0126984127*G1_0_0_3_1 + 0.0126984127*G1_0_0_4_0 - 0.0253968254*G1_0_0_4_1 - 0.0253968254*G1_0_0_5_0 + 0.0126984127*G1_0_0_5_1 - 0.00317460317*G1_0_1_0_0 - 0.00317460317*G1_0_1_0_1 - 0.00952380952*G1_0_1_1_0 + 0.00317460317*G1_0_1_2_1 - 0.0126984127*G1_0_1_3_1 + 0.0126984127*G1_0_1_5_0 + 0.0126984127*G1_0_1_5_1 + 0.00317460317*G1_1_0_0_0 + 0.00317460317*G1_1_0_0_1 + 0.0222222222*G1_1_0_1_0 - 0.00317460317*G1_1_0_2_1 + 0.0126984127*G1_1_0_3_0 + 0.0380952381*G1_1_0_3_1 - 0.0126984127*G1_1_0_4_0 - 0.0253968254*G1_1_0_5_0 - 0.0380952381*G1_1_0_5_1 + 0.0126984127*G1_1_1_0_0 + 0.0126984127*G1_1_1_0_1 + 0.0253968254*G1_1_1_1_0 + 0.0126984127*G1_1_1_3_0 + 0.0380952381*G1_1_1_3_1 - 0.0126984127*G1_1_1_4_0 - 0.0126984127*G1_1_1_4_1 - 0.0380952381*G1_1_1_5_0 - 0.0380952381*G1_1_1_5_1 - 0.0126984127*G1_2_0_0_0 - 0.0126984127*G1_2_0_0_1 - 0.0126984127*G1_2_0_1_0 - 0.0126984127*G1_2_0_3_1 + 0.0126984127*G1_2_0_4_1 + 0.0253968254*G1_2_0_5_0 + 0.0126984127*G1_2_0_5_1 - 0.00634920635*G1_2_1_0_0 - 0.00634920635*G1_2_1_0_1 - 0.00634920635*G1_2_1_1_0 - 0.019047619*G1_2_1_2_1 - 0.0380952381*G1_2_1_3_0 - 0.0253968254*G1_2_1_3_1 + 0.0380952381*G1_2_1_4_0 + 0.0253968254*G1_2_1_4_1 + 0.0126984127*G1_2_1_5_0 + 0.0253968254*G1_2_1_5_1 + 0.0126984127*G1_3_0_0_0 + 0.0126984127*G1_3_0_0_1 + 0.0380952381*G1_3_0_1_0 - 0.0126984127*G1_3_0_2_1 + 0.0507936508*G1_3_0_3_1 - 0.0507936508*G1_3_0_5_0 - 0.0507936508*G1_3_0_5_1 + 0.0507936508*G1_3_1_1_0 - 0.0253968254*G1_3_1_2_1 + 0.0761904762*G1_3_1_3_1 + 0.0253968254*G1_3_1_4_1 - 0.0507936508*G1_3_1_5_0 - 0.0761904762*G1_3_1_5_1 + 0.0380952381*G1_4_0_0_0 + 0.0380952381*G1_4_0_0_1 + 0.0126984127*G1_4_0_1_0 + 0.0126984127*G1_4_0_2_1 - 0.0507936508*G1_4_0_4_1 - 0.0507936508*G1_4_0_5_0 - 0.0126984127*G1_4_1_0_0 - 0.0126984127*G1_4_1_0_1 + 0.0126984127*G1_4_1_1_0 - 0.0126984127*G1_4_1_2_1 + 0.0253968254*G1_4_1_3_1 + 0.0253968254*G1_4_1_4_1 - 0.0253968254*G1_4_1_5_1 + 0.0253968254*G1_5_0_0_0 + 0.0253968254*G1_5_0_0_1 + 0.0253968254*G1_5_0_1_0 + 0.0253968254*G1_5_0_3_1 - 0.0253968254*G1_5_0_4_1 - 0.0507936508*G1_5_0_5_0 - 0.0253968254*G1_5_0_5_1 - 0.0126984127*G1_5_1_0_0 - 0.0126984127*G1_5_1_0_1 + 0.0380952381*G1_5_1_1_0 - 0.0126984127*G1_5_1_2_1 + 0.0253968254*G1_5_1_3_0 + 0.0761904762*G1_5_1_3_1 - 0.0253968254*G1_5_1_4_0 + 0.0253968254*G1_5_1_4_1 - 0.0253968254*G1_5_1_5_0 - 0.0761904762*G1_5_1_5_1;
    A[33] = 0.0558510687*G0_0_0_0 + 0.0647592611*G0_0_0_1 - 0.00812617548*G0_0_1_0 - 0.00942229274*G0_0_1_1 + 0.190830873*G0_1_0_0 + 0.0310173349*G0_1_0_1 - 0.0277653623*G0_1_1_0 - 0.00451293614*G0_1_1_1 + 0.0402057135*G0_2_0_0 + 0.0012215536*G0_2_0_1 - 0.00584981972*G0_2_1_0 - 0.000177732658*G0_2_1_1 - 0.0576827357*G0_3_1_0 - 0.296726215*G0_3_1_1 - 0.197089278*G0_4_1_0 - 0.142121084*G0_4_1_1 - 0.0415242824*G0_5_1_0 - 0.00559714502*G0_5_1_1 - 0.0558510687*G0_6_0_0 - 0.509848584*G0_6_0_1 - 0.0639772442*G0_6_1_0 - 0.584030138*G0_6_1_1 - 0.190830873*G0_7_0_0 - 0.244198961*G0_7_0_1 - 0.218596235*G0_7_1_0 - 0.279729232*G0_7_1_1 - 0.0402057135*G0_8_0_0 - 0.00961727113*G0_8_0_1 - 0.0460555333*G0_8_1_0 - 0.0110165574*G0_8_1_1 + 0.0253968254*G1_0_0_1_0 + 0.0253968254*G1_0_0_3_0 + 0.0507936508*G1_0_0_3_1 - 0.0253968254*G1_0_0_4_0 - 0.0253968254*G1_0_0_5_0 - 0.0507936508*G1_0_0_5_1 + 0.0126984127*G1_0_1_0_0 + 0.0126984127*G1_0_1_0_1 + 0.0380952381*G1_0_1_1_0 + 0.0126984127*G1_0_1_2_1 + 0.0507936508*G1_0_1_3_0 + 0.0761904762*G1_0_1_3_1 - 0.0507936508*G1_0_1_4_0 - 0.0253968254*G1_0_1_4_1 - 0.0507936508*G1_0_1_5_0 - 0.0761904762*G1_0_1_5_1 - 0.0253968254*G1_1_0_0_0 - 0.0253968254*G1_1_0_0_1 - 0.101587302*G1_1_0_1_0 + 0.0253968254*G1_1_0_2_1 - 0.0253968254*G1_1_0_3_0 - 0.152380952*G1_1_0_3_1 + 0.0253968254*G1_1_0_4_0 + 0.126984127*G1_1_0_5_0 + 0.152380952*G1_1_0_5_1 - 0.0380952381*G1_1_1_0_0 - 0.0380952381*G1_1_1_0_1 - 0.114285714*G1_1_1_1_0 + 0.0380952381*G1_1_1_2_1 - 0.152380952*G1_1_1_3_1 + 0.152380952*G1_1_1_5_0 + 0.152380952*G1_1_1_5_1 + 0.0126984127*G1_2_0_0_0 + 0.0126984127*G1_2_0_0_1 + 0.0380952381*G1_2_0_1_0 - 0.0126984127*G1_2_0_2_1 + 0.0507936508*G1_2_0_3_1 - 0.0507936508*G1_2_0_5_0 - 0.0507936508*G1_2_0_5_1 + 0.0507936508*G1_2_1_1_0 - 0.0253968254*G1_2_1_2_1 + 0.0761904762*G1_2_1_3_1 + 0.0253968254*G1_2_1_4_1 - 0.0507936508*G1_2_1_5_0 - 0.0761904762*G1_2_1_5_1 - 0.152380952*G1_3_0_0_0 - 0.152380952*G1_3_0_0_1 - 0.253968254*G1_3_0_1_0 - 0.101587302*G1_3_0_2_1 - 0.304761905*G1_3_0_3_0 - 0.457142857*G1_3_0_3_1 + 0.304761905*G1_3_0_4_0 + 0.253968254*G1_3_0_4_1 + 0.406349206*G1_3_0_5_0 + 0.457142857*G1_3_0_5_1 - 0.152380952*G1_3_1_0_0 - 0.152380952*G1_3_1_0_1 - 0.304761905*G1_3_1_1_0 - 0.152380952*G1_3_1_2_1 - 0.457142857*G1_3_1_3_0 - 0.60952381*G1_3_1_3_1 + 0.457142857*G1_3_1_4_0 + 0.304761905*G1_3_1_4_1 + 0.457142857*G1_3_1_5_0 + 0.60952381*G1_3_1_5_1 - 0.0507936508*G1_4_0_0_0 - 0.0507936508*G1_4_0_0_1 - 0.0507936508*G1_4_0_1_0 - 0.0507936508*G1_4_0_3_1 + 0.0507936508*G1_4_0_4_1 + 0.101587302*G1_4_0_5_0 + 0.0507936508*G1_4_0_5_1 - 0.0507936508*G1_4_1_1_0 - 0.0507936508*G1_4_1_2_1 - 0.152380952*G1_4_1_3_0 - 0.152380952*G1_4_1_3_1 + 0.152380952*G1_4_1_4_0 + 0.0507936508*G1_4_1_4_1 + 0.0507936508*G1_4_1_5_0 + 0.152380952*G1_4_1_5_1 - 0.0507936508*G1_5_0_0_0 - 0.0507936508*G1_5_0_0_1 - 0.101587302*G1_5_0_1_0 - 0.0507936508*G1_5_0_3_0 - 0.152380952*G1_5_0_3_1 + 0.0507936508*G1_5_0_4_0 + 0.0507936508*G1_5_0_4_1 + 0.152380952*G1_5_0_5_0 + 0.152380952*G1_5_0_5_1 - 0.152380952*G1_5_1_1_0 - 0.152380952*G1_5_1_3_0 - 0.304761905*G1_5_1_3_1 + 0.152380952*G1_5_1_4_0 + 0.152380952*G1_5_1_5_0 + 0.304761905*G1_5_1_5_1;
    A[34] = -0.0558510687*G0_0_0_0 + 0.453997515*G0_0_0_1 + 0.00812617548*G0_0_1_0 - 0.0660553783*G0_0_1_1 - 0.190830873*G0_1_0_0 + 0.0533680882*G0_1_0_1 + 0.0277653623*G0_1_1_0 - 0.0077649087*G0_1_1_1 - 0.0402057135*G0_2_0_0 - 0.0305884424*G0_2_0_1 + 0.00584981972*G0_2_1_0 + 0.00445053347*G0_2_1_1 + 0.0576827357*G0_3_1_0 - 0.239043479*G0_3_1_1 + 0.197089278*G0_4_1_0 + 0.054968194*G0_4_1_1 + 0.0415242824*G0_5_1_0 + 0.0359271374*G0_5_1_1 + 0.0558510687*G0_6_0_0 - 0.00890819235*G0_6_0_1 + 0.0639772442*G0_6_1_0 - 0.0102043096*G0_6_1_1 + 0.190830873*G0_7_0_0 + 0.159813538*G0_7_0_1 + 0.218596235*G0_7_1_0 + 0.183065964*G0_7_1_1 + 0.0402057135*G0_8_0_0 + 0.0389841599*G0_8_0_1 + 0.0460555333*G0_8_1_0 + 0.044656247*G0_8_1_1 - 0.101587302*G1_0_0_0_0 - 0.101587302*G1_0_0_0_1 - 0.0253968254*G1_0_0_1_0 - 0.0253968254*G1_0_0_2_1 + 0.0253968254*G1_0_0_3_0 + 0.0253968254*G1_0_0_3_1 - 0.0253968254*G1_0_0_4_0 + 0.126984127*G1_0_0_4_1 + 0.126984127*G1_0_0_5_0 - 0.0253968254*G1_0_0_5_1 + 0.0126984127*G1_0_1_0_0 + 0.0126984127*G1_0_1_0_1 + 0.0126984127*G1_0_1_1_0 + 0.0126984127*G1_0_1_2_1 + 0.0253968254*G1_0_1_3_0 + 0.0253968254*G1_0_1_3_1 - 0.0253968254*G1_0_1_4_0 - 0.0253968254*G1_0_1_4_1 - 0.0253968254*G1_0_1_5_0 - 0.0253968254*G1_0_1_5_1 + 0.0253968254*G1_1_0_0_0 + 0.0253968254*G1_1_0_0_1 - 0.0253968254*G1_1_0_3_0 - 0.0253968254*G1_1_0_3_1 + 0.0253968254*G1_1_0_4_0 - 0.0253968254*G1_1_0_4_1 - 0.0253968254*G1_1_0_5_0 + 0.0253968254*G1_1_0_5_1 - 0.0126984127*G1_1_1_0_0 - 0.0126984127*G1_1_1_0_1 - 0.0126984127*G1_1_1_1_0 + 0.0126984127*G1_1_1_2_1 + 0.0253968254*G1_1_1_3_0 - 0.0253968254*G1_1_1_4_0 + 0.0253968254*G1_1_1_5_0 + 0.0380952381*G1_2_0_0_0 + 0.0380952381*G1_2_0_0_1 + 0.0126984127*G1_2_0_1_0 + 0.0126984127*G1_2_0_2_1 - 0.0507936508*G1_2_0_4_1 - 0.0507936508*G1_2_0_5_0 - 0.0126984127*G1_2_1_0_0 - 0.0126984127*G1_2_1_0_1 + 0.0126984127*G1_2_1_1_0 - 0.0126984127*G1_2_1_2_1 + 0.0253968254*G1_2_1_3_1 + 0.0253968254*G1_2_1_4_1 - 0.0253968254*G1_2_1_5_1 - 0.0507936508*G1_3_0_0_0 - 0.0507936508*G1_3_0_0_1 - 0.0507936508*G1_3_0_1_0 - 0.0507936508*G1_3_0_3_1 + 0.0507936508*G1_3_0_4_1 + 0.101587302*G1_3_0_5_0 + 0.0507936508*G1_3_0_5_1 - 0.0507936508*G1_3_1_1_0 - 0.0507936508*G1_3_1_2_1 - 0.152380952*G1_3_1_3_0 - 0.152380952*G1_3_1_3_1 + 0.152380952*G1_3_1_4_0 + 0.0507936508*G1_3_1_4_1 + 0.0507936508*G1_3_1_5_0 + 0.152380952*G1_3_1_5_1 - 0.253968254*G1_4_0_0_0 - 0.253968254*G1_4_0_0_1 - 0.152380952*G1_4_0_1_0 + 0.101587302*G1_4_0_2_1 + 0.304761905*G1_4_0_3_0 + 0.0507936508*G1_4_0_3_1 - 0.304761905*G1_4_0_4_0 + 0.152380952*G1_4_0_4_1 + 0.406349206*G1_4_0_5_0 - 0.0507936508*G1_4_0_5_1 + 0.0507936508*G1_4_1_0_0 + 0.0507936508*G1_4_1_0_1 - 0.0507936508*G1_4_1_2_1 - 0.152380952*G1_4_1_3_0 - 0.101587302*G1_4_1_3_1 + 0.152380952*G1_4_1_4_0 - 0.0507936508*G1_4_1_5_0 + 0.101587302*G1_4_1_5_1 - 0.101587302*G1_5_0_0_0 - 0.101587302*G1_5_0_0_1 - 0.0507936508*G1_5_0_1_0 + 0.0507936508*G1_5_0_3_0 - 0.0507936508*G1_5_0_4_0 + 0.101587302*G1_5_0_4_1 + 0.152380952*G1_5_0_5_0 + 0.0507936508*G1_5_1_0_0 + 0.0507936508*G1_5_1_0_1 - 0.0507936508*G1_5_1_1_0 - 0.101587302*G1_5_1_3_0 - 0.152380952*G1_5_1_3_1 + 0.101587302*G1_5_1_4_0 - 0.0507936508*G1_5_1_4_1 + 0.152380952*G1_5_1_5_1;
    A[35] = 0.445089323*G0_0_0_0 - 0.0647592611*G0_0_0_1 - 0.0647592611*G0_0_1_0 + 0.00942229274*G0_0_1_1 + 0.213181626*G0_1_0_0 - 0.0310173349*G0_1_0_1 - 0.0310173349*G0_1_1_0 + 0.00451293614*G0_1_1_1 + 0.00839571753*G0_2_0_0 - 0.0012215536*G0_2_0_1 - 0.0012215536*G0_2_1_0 + 0.000177732658*G0_2_1_1 + 0.296726215*G0_3_1_1 + 0.142121084*G0_4_1_1 + 0.00559714502*G0_5_1_1 + 0.445089323*G0_6_0_0 + 0.509848584*G0_6_0_1 + 0.509848584*G0_6_1_0 + 0.584030138*G0_6_1_1 + 0.213181626*G0_7_0_0 + 0.244198961*G0_7_0_1 + 0.244198961*G0_7_1_0 + 0.279729232*G0_7_1_1 + 0.00839571753*G0_8_0_0 + 0.00961727113*G0_8_0_1 + 0.00961727113*G0_8_1_0 + 0.0110165574*G0_8_1_1 - 0.0888888889*G1_0_0_0_0 - 0.0888888889*G1_0_0_0_1 + 0.0126984127*G1_0_0_1_0 - 0.0380952381*G1_0_0_2_1 + 0.0253968254*G1_0_0_3_0 + 0.0761904762*G1_0_0_3_1 - 0.0253968254*G1_0_0_4_0 + 0.126984127*G1_0_0_4_1 + 0.0761904762*G1_0_0_5_0 - 0.0761904762*G1_0_0_5_1 + 0.0253968254*G1_0_1_0_0 + 0.0253968254*G1_0_1_0_1 + 0.0507936508*G1_0_1_1_0 + 0.0253968254*G1_0_1_3_0 + 0.0761904762*G1_0_1_3_1 - 0.0253968254*G1_0_1_4_0 - 0.0253968254*G1_0_1_4_1 - 0.0761904762*G1_0_1_5_0 - 0.0761904762*G1_0_1_5_1 + 0.0126984127*G1_1_0_0_0 + 0.0126984127*G1_1_0_0_1 - 0.0888888889*G1_1_0_1_0 + 0.0380952381*G1_1_0_2_1 - 0.0253968254*G1_1_0_3_0 - 0.152380952*G1_1_0_3_1 + 0.0253968254*G1_1_0_4_0 - 0.0507936508*G1_1_0_4_1 + 0.0761904762*G1_1_0_5_0 + 0.152380952*G1_1_0_5_1 - 0.0380952381*G1_1_1_0_0 - 0.0380952381*G1_1_1_0_1 - 0.114285714*G1_1_1_1_0 + 0.0380952381*G1_1_1_2_1 - 0.152380952*G1_1_1_3_1 + 0.152380952*G1_1_1_5_0 + 0.152380952*G1_1_1_5_1 + 0.0253968254*G1_2_0_0_0 + 0.0253968254*G1_2_0_0_1 + 0.0253968254*G1_2_0_1_0 + 0.0253968254*G1_2_0_3_1 - 0.0253968254*G1_2_0_4_1 - 0.0507936508*G1_2_0_5_0 - 0.0253968254*G1_2_0_5_1 - 0.0126984127*G1_2_1_0_0 - 0.0126984127*G1_2_1_0_1 + 0.0380952381*G1_2_1_1_0 - 0.0126984127*G1_2_1_2_1 + 0.0253968254*G1_2_1_3_0 + 0.0761904762*G1_2_1_3_1 - 0.0253968254*G1_2_1_4_0 + 0.0253968254*G1_2_1_4_1 - 0.0253968254*G1_2_1_5_0 - 0.0761904762*G1_2_1_5_1 - 0.0507936508*G1_3_0_0_0 - 0.0507936508*G1_3_0_0_1 - 0.101587302*G1_3_0_1_0 - 0.0507936508*G1_3_0_3_0 - 0.152380952*G1_3_0_3_1 + 0.0507936508*G1_3_0_4_0 + 0.0507936508*G1_3_0_4_1 + 0.152380952*G1_3_0_5_0 + 0.152380952*G1_3_0_5_1 - 0.152380952*G1_3_1_1_0 - 0.152380952*G1_3_1_3_0 - 0.304761905*G1_3_1_3_1 + 0.152380952*G1_3_1_4_0 + 0.152380952*G1_3_1_5_0 + 0.304761905*G1_3_1_5_1 - 0.101587302*G1_4_0_0_0 - 0.101587302*G1_4_0_0_1 - 0.0507936508*G1_4_0_1_0 + 0.0507936508*G1_4_0_3_0 - 0.0507936508*G1_4_0_4_0 + 0.101587302*G1_4_0_4_1 + 0.152380952*G1_4_0_5_0 + 0.0507936508*G1_4_1_0_0 + 0.0507936508*G1_4_1_0_1 - 0.0507936508*G1_4_1_1_0 - 0.101587302*G1_4_1_3_0 - 0.152380952*G1_4_1_3_1 + 0.101587302*G1_4_1_4_0 - 0.0507936508*G1_4_1_4_1 + 0.152380952*G1_4_1_5_1 - 0.152380952*G1_5_0_0_0 - 0.152380952*G1_5_0_0_1 - 0.152380952*G1_5_0_1_0 - 0.152380952*G1_5_0_3_1 + 0.152380952*G1_5_0_4_1 + 0.304761905*G1_5_0_5_0 + 0.152380952*G1_5_0_5_1 + 0.152380952*G1_5_1_0_0 + 0.152380952*G1_5_1_0_1 - 0.304761905*G1_5_1_1_0 + 0.152380952*G1_5_1_2_1 - 0.152380952*G1_5_1_3_0 - 0.60952381*G1_5_1_3_1 + 0.152380952*G1_5_1_4_0 - 0.304761905*G1_5_1_4_1 + 0.152380952*G1_5_1_5_0 + 0.60952381*G1_5_1_5_1;
  }

};

/// This class defines the interface for the assembly of the global
/// tensor corresponding to a form with r + n arguments, that is, a
/// mapping
///
///     a : V1 x V2 x ... Vr x W1 x W2 x ... x Wn -> R
///
/// with arguments v1, v2, ..., vr, w1, w2, ..., wn. The rank r
/// global tensor A is defined by
///
///     A = a(V1, V2, ..., Vr, w1, w2, ..., wn),
///
/// where each argument Vj represents the application to the
/// sequence of basis functions of Vj and w1, w2, ..., wn are given
/// fixed functions (coefficients).

class QuadratureElementBilinearForm: public ufc::form
{
public:

  /// Constructor
  QuadratureElementBilinearForm() : ufc::form()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~QuadratureElementBilinearForm()
  {
    // Do nothing
  }

  /// Return a string identifying the form
  virtual const char* signature() const
  {
    return "w1_a0[0, 1, 2, 3, 4, 5, 6, 7, 8](dXa1[0, 1]/dxb0[0, 1])(dXa2[0, 1]/dxb0[0, 1]) | ((d/dXa1[0, 1])vi0[0, 1, 2, 3, 4, 5])*va0[0, 1, 2, 3, 4, 5, 6, 7, 8]*((d/dXa2[0, 1])vi1[0, 1, 2, 3, 4, 5])*dX(0) + 2.0w0_a0[0, 1, 2, 3, 4, 5]w0_a2[0, 1, 2, 3, 4, 5](dXa1[0, 1]/dxb0[0, 1])(dXa3[0, 1]/dxb0[0, 1]) | ((d/dXa1[0, 1])vi0[0, 1, 2, 3, 4, 5])*va0[0, 1, 2, 3, 4, 5]*vi1[0, 1, 2, 3, 4, 5]*((d/dXa3[0, 1])va2[0, 1, 2, 3, 4, 5])*dX(0)";
  }

  /// Return the rank of the global tensor (r)
  virtual unsigned int rank() const
  {
    return 2;
  }

  /// Return the number of coefficients (n)
  virtual unsigned int num_coefficients() const
  {
    return 2;
  }

  /// Return the number of cell integrals
  virtual unsigned int num_cell_integrals() const
  {
    return 1;
  }
  
  /// Return the number of exterior facet integrals
  virtual unsigned int num_exterior_facet_integrals() const
  {
    return 0;
  }
  
  /// Return the number of interior facet integrals
  virtual unsigned int num_interior_facet_integrals() const
  {
    return 0;
  }
    
  /// Create a new finite element for argument function i
  virtual ufc::finite_element* create_finite_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new QuadratureElementBilinearForm_finite_element_0();
      break;
    case 1:
      return new QuadratureElementBilinearForm_finite_element_1();
      break;
    case 2:
      return new QuadratureElementBilinearForm_finite_element_2();
      break;
    case 3:
      return new QuadratureElementBilinearForm_finite_element_3();
      break;
    }
    return 0;
  }
  
  /// Create a new dof map for argument function i
  virtual ufc::dof_map* create_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new QuadratureElementBilinearForm_dof_map_0();
      break;
    case 1:
      return new QuadratureElementBilinearForm_dof_map_1();
      break;
    case 2:
      return new QuadratureElementBilinearForm_dof_map_2();
      break;
    case 3:
      return new QuadratureElementBilinearForm_dof_map_3();
      break;
    }
    return 0;
  }

  /// Create a new cell integral on sub domain i
  virtual ufc::cell_integral* create_cell_integral(unsigned int i) const
  {
    return new QuadratureElementBilinearForm_cell_integral_0();
  }

  /// Create a new exterior facet integral on sub domain i
  virtual ufc::exterior_facet_integral* create_exterior_facet_integral(unsigned int i) const
  {
    return 0;
  }

  /// Create a new interior facet integral on sub domain i
  virtual ufc::interior_facet_integral* create_interior_facet_integral(unsigned int i) const
  {
    return 0;
  }

};

/// This class defines the interface for a finite element.

class QuadratureElementLinearForm_finite_element_0: public ufc::finite_element
{
public:

  /// Constructor
  QuadratureElementLinearForm_finite_element_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~QuadratureElementLinearForm_finite_element_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 2 on a triangle";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::triangle;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 6;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute inverse of Jacobian
    
    // Get coordinates and map to the reference (UFC) element
    double x = (element_coordinates[0][1]*element_coordinates[2][0] -\
                element_coordinates[0][0]*element_coordinates[2][1] +\
                J_11*coordinates[0] - J_01*coordinates[1]) / detJ;
    double y = (element_coordinates[1][1]*element_coordinates[0][0] -\
                element_coordinates[1][0]*element_coordinates[0][1] -\
                J_10*coordinates[0] + J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-08)
      x = -1.0;
    else
      x = 2.0 *x/(1.0 - y) - 1.0;
    y = 2.0*y - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111*psitilde_bs_0_1 + 1.66666667*y*psitilde_bs_0_1 - 0.555555556*psitilde_bs_0_0;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_2_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.707106781*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    const double basisvalue1 = 1.73205081*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
    const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
    const double basisvalue3 = 2.73861279*psitilde_a_2*scalings_y_2*psitilde_bs_2_0;
    const double basisvalue4 = 2.12132034*psitilde_a_1*scalings_y_1*psitilde_bs_1_1;
    const double basisvalue5 = 1.22474487*psitilde_a_0*scalings_y_0*psitilde_bs_0_2;
    
    // Table(s) of coefficients
    const static double coefficients0[6][6] = \
    {{0, -0.173205081, -0.1, 0.121716124, 0.0942809042, 0.0544331054},
    {0, 0.173205081, -0.1, 0.121716124, -0.0942809042, 0.0544331054},
    {0, 0, 0.2, 0, 0, 0.163299316},
    {0.471404521, 0.230940108, 0.133333333, 0, 0.188561808, -0.163299316},
    {0.471404521, -0.230940108, 0.133333333, 0, -0.188561808, -0.163299316},
    {0.471404521, 0, -0.266666667, -0.243432248, 0, 0.0544331054}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    const double coeff0_4 = coefficients0[dof][4];
    const double coeff0_5 = coefficients0[dof][5];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5;
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute inverse of Jacobian
    
    // Get coordinates and map to the reference (UFC) element
    double x = (element_coordinates[0][1]*element_coordinates[2][0] -\
                element_coordinates[0][0]*element_coordinates[2][1] +\
                J_11*coordinates[0] - J_01*coordinates[1]) / detJ;
    double y = (element_coordinates[1][1]*element_coordinates[0][0] -\
                element_coordinates[1][0]*element_coordinates[0][1] -\
                J_10*coordinates[0] + J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-08)
      x = -1.0;
    else
      x = 2.0 *x/(1.0 - y) - 1.0;
    y = 2.0*y - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 2;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 1)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[2][2] =  {{J_11 / detJ, -J_01 / detJ}, {-J_10 / detJ, J_00 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111*psitilde_bs_0_1 + 1.66666667*y*psitilde_bs_0_1 - 0.555555556*psitilde_bs_0_0;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_2_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.707106781*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    const double basisvalue1 = 1.73205081*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
    const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
    const double basisvalue3 = 2.73861279*psitilde_a_2*scalings_y_2*psitilde_bs_2_0;
    const double basisvalue4 = 2.12132034*psitilde_a_1*scalings_y_1*psitilde_bs_1_1;
    const double basisvalue5 = 1.22474487*psitilde_a_0*scalings_y_0*psitilde_bs_0_2;
    
    // Table(s) of coefficients
    const static double coefficients0[6][6] = \
    {{0, -0.173205081, -0.1, 0.121716124, 0.0942809042, 0.0544331054},
    {0, 0.173205081, -0.1, 0.121716124, -0.0942809042, 0.0544331054},
    {0, 0, 0.2, 0, 0, 0.163299316},
    {0.471404521, 0.230940108, 0.133333333, 0, 0.188561808, -0.163299316},
    {0.471404521, -0.230940108, 0.133333333, 0, -0.188561808, -0.163299316},
    {0.471404521, 0, -0.266666667, -0.243432248, 0, 0.0544331054}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[6][6] = \
    {{0, 0, 0, 0, 0, 0},
    {4.89897949, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0},
    {0, 9.48683298, 0, 0, 0, 0},
    {4, 0, 7.07106781, 0, 0, 0},
    {0, 0, 0, 0, 0, 0}};
    
    const static double dmats1[6][6] = \
    {{0, 0, 0, 0, 0, 0},
    {2.44948974, 0, 0, 0, 0, 0},
    {4.24264069, 0, 0, 0, 0, 0},
    {2.5819889, 4.74341649, -0.912870929, 0, 0, 0},
    {2, 6.12372436, 3.53553391, 0, 0, 0},
    {-2.30940108, 0, 8.16496581, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    double coeff0_4 = 0;
    double coeff0_5 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    double new_coeff0_4 = 0;
    double new_coeff0_5 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
      new_coeff0_4 = coefficients0[dof][4];
      new_coeff0_5 = coefficients0[dof][5];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
        coeff0_4 = new_coeff0_4;
        coeff0_5 = new_coeff0_5;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3];
          new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4];
          new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3];
          new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4];
          new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    const static double X[6][1][2] = {{{0, 0}}, {{1, 0}}, {{0, 1}}, {{0.5, 0.5}}, {{0, 0.5}}, {{0.5, 0}}};
    const static double W[6][1] = {{1}, {1}, {1}, {1}, {1}, {1}};
    const static double D[6][1][1] = {{{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}};
    
    const double * const * x = c.coordinates;
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    
    // Compute affine mapping y = F(X)
    double y[2];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1];
    
    // Evaluate function at physical points
    double values[1];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Affine map: Do nothing
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 1; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new QuadratureElementLinearForm_finite_element_0();
  }

};

/// This class defines the interface for a finite element.

class QuadratureElementLinearForm_finite_element_1_0: public ufc::finite_element
{
public:

  /// Constructor
  QuadratureElementLinearForm_finite_element_1_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~QuadratureElementLinearForm_finite_element_1_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Quadrature element with 9 quadrature point(s) (3x3) on a triangle";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::triangle;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 9;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    throw std::runtime_error("evaluate_basis() is not supported for QuadratureElement");
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("evaluate_basis_all() is not supported for QuadratureElement.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    throw std::runtime_error("evaluate_basis_derivatives() is not supported for QuadratureElement");
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("evaluate_basis_derivatives_all() is not supported for QuadratureElement.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    const static double X[9][1][2] = {{{0.102717655, 0.0885879595}}, {{0.0665540678, 0.409466864}}, {{0.0239311323, 0.787659462}}, {{0.45570602, 0.0885879595}}, {{0.295266568, 0.409466864}}, {{0.106170269, 0.787659462}}, {{0.808694386, 0.0885879595}}, {{0.523979068, 0.409466864}}, {{0.188409406, 0.787659462}}};
    const static double W[9][1] = {{1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}};
    const static double D[9][1][1] = {{{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}};
    
    const double * const * x = c.coordinates;
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    
    // Compute affine mapping y = F(X)
    double y[2];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1];
    
    // Evaluate function at physical points
    double values[1];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Affine map: Do nothing
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 1; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    throw std::runtime_error("interpolate_vertex_values() is not supported for QuadratureElement");
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new QuadratureElementLinearForm_finite_element_1_0();
  }

};

/// This class defines the interface for a finite element.

class QuadratureElementLinearForm_finite_element_1_1: public ufc::finite_element
{
public:

  /// Constructor
  QuadratureElementLinearForm_finite_element_1_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~QuadratureElementLinearForm_finite_element_1_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Quadrature element with 9 quadrature point(s) (3x3) on a triangle";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::triangle;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 9;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    throw std::runtime_error("evaluate_basis() is not supported for QuadratureElement");
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("evaluate_basis_all() is not supported for QuadratureElement.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    throw std::runtime_error("evaluate_basis_derivatives() is not supported for QuadratureElement");
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("evaluate_basis_derivatives_all() is not supported for QuadratureElement.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    const static double X[9][1][2] = {{{0.102717655, 0.0885879595}}, {{0.0665540678, 0.409466864}}, {{0.0239311323, 0.787659462}}, {{0.45570602, 0.0885879595}}, {{0.295266568, 0.409466864}}, {{0.106170269, 0.787659462}}, {{0.808694386, 0.0885879595}}, {{0.523979068, 0.409466864}}, {{0.188409406, 0.787659462}}};
    const static double W[9][1] = {{1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}};
    const static double D[9][1][1] = {{{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}};
    
    const double * const * x = c.coordinates;
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    
    // Compute affine mapping y = F(X)
    double y[2];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1];
    
    // Evaluate function at physical points
    double values[1];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Affine map: Do nothing
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 1; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    throw std::runtime_error("interpolate_vertex_values() is not supported for QuadratureElement");
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new QuadratureElementLinearForm_finite_element_1_1();
  }

};

/// This class defines the interface for a finite element.

class QuadratureElementLinearForm_finite_element_1: public ufc::finite_element
{
public:

  /// Constructor
  QuadratureElementLinearForm_finite_element_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~QuadratureElementLinearForm_finite_element_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Mixed finite element: [Quadrature element with 9 quadrature point(s) (3x3) on a triangle, Quadrature element with 9 quadrature point(s) (3x3) on a triangle]";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::triangle;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 18;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 1;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 2;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    throw std::runtime_error("evaluate_basis() is not supported for QuadratureElement");
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("evaluate_basis_all() is not supported for QuadratureElement.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    throw std::runtime_error("evaluate_basis_derivatives() is not supported for QuadratureElement");
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("evaluate_basis_derivatives_all() is not supported for QuadratureElement.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    const static double X[18][1][2] = {{{0.102717655, 0.0885879595}}, {{0.0665540678, 0.409466864}}, {{0.0239311323, 0.787659462}}, {{0.45570602, 0.0885879595}}, {{0.295266568, 0.409466864}}, {{0.106170269, 0.787659462}}, {{0.808694386, 0.0885879595}}, {{0.523979068, 0.409466864}}, {{0.188409406, 0.787659462}}, {{0.102717655, 0.0885879595}}, {{0.0665540678, 0.409466864}}, {{0.0239311323, 0.787659462}}, {{0.45570602, 0.0885879595}}, {{0.295266568, 0.409466864}}, {{0.106170269, 0.787659462}}, {{0.808694386, 0.0885879595}}, {{0.523979068, 0.409466864}}, {{0.188409406, 0.787659462}}};
    const static double W[18][1] = {{1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}};
    const static double D[18][1][2] = {{{1, 0}}, {{1, 0}}, {{1, 0}}, {{1, 0}}, {{1, 0}}, {{1, 0}}, {{1, 0}}, {{1, 0}}, {{1, 0}}, {{0, 1}}, {{0, 1}}, {{0, 1}}, {{0, 1}}, {{0, 1}}, {{0, 1}}, {{0, 1}}, {{0, 1}}, {{0, 1}}};
    
    const double * const * x = c.coordinates;
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    
    // Compute affine mapping y = F(X)
    double y[2];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1];
    
    // Evaluate function at physical points
    double values[2];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Affine map: Do nothing
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 2; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    throw std::runtime_error("interpolate_vertex_values() is not supported for QuadratureElement");
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 2;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new QuadratureElementLinearForm_finite_element_1_0();
      break;
    case 1:
      return new QuadratureElementLinearForm_finite_element_1_1();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a finite element.

class QuadratureElementLinearForm_finite_element_2: public ufc::finite_element
{
public:

  /// Constructor
  QuadratureElementLinearForm_finite_element_2() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~QuadratureElementLinearForm_finite_element_2()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 2 on a triangle";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::triangle;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 6;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute inverse of Jacobian
    
    // Get coordinates and map to the reference (UFC) element
    double x = (element_coordinates[0][1]*element_coordinates[2][0] -\
                element_coordinates[0][0]*element_coordinates[2][1] +\
                J_11*coordinates[0] - J_01*coordinates[1]) / detJ;
    double y = (element_coordinates[1][1]*element_coordinates[0][0] -\
                element_coordinates[1][0]*element_coordinates[0][1] -\
                J_10*coordinates[0] + J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-08)
      x = -1.0;
    else
      x = 2.0 *x/(1.0 - y) - 1.0;
    y = 2.0*y - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111*psitilde_bs_0_1 + 1.66666667*y*psitilde_bs_0_1 - 0.555555556*psitilde_bs_0_0;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_2_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.707106781*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    const double basisvalue1 = 1.73205081*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
    const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
    const double basisvalue3 = 2.73861279*psitilde_a_2*scalings_y_2*psitilde_bs_2_0;
    const double basisvalue4 = 2.12132034*psitilde_a_1*scalings_y_1*psitilde_bs_1_1;
    const double basisvalue5 = 1.22474487*psitilde_a_0*scalings_y_0*psitilde_bs_0_2;
    
    // Table(s) of coefficients
    const static double coefficients0[6][6] = \
    {{0, -0.173205081, -0.1, 0.121716124, 0.0942809042, 0.0544331054},
    {0, 0.173205081, -0.1, 0.121716124, -0.0942809042, 0.0544331054},
    {0, 0, 0.2, 0, 0, 0.163299316},
    {0.471404521, 0.230940108, 0.133333333, 0, 0.188561808, -0.163299316},
    {0.471404521, -0.230940108, 0.133333333, 0, -0.188561808, -0.163299316},
    {0.471404521, 0, -0.266666667, -0.243432248, 0, 0.0544331054}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    const double coeff0_4 = coefficients0[dof][4];
    const double coeff0_5 = coefficients0[dof][5];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5;
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute inverse of Jacobian
    
    // Get coordinates and map to the reference (UFC) element
    double x = (element_coordinates[0][1]*element_coordinates[2][0] -\
                element_coordinates[0][0]*element_coordinates[2][1] +\
                J_11*coordinates[0] - J_01*coordinates[1]) / detJ;
    double y = (element_coordinates[1][1]*element_coordinates[0][0] -\
                element_coordinates[1][0]*element_coordinates[0][1] -\
                J_10*coordinates[0] + J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-08)
      x = -1.0;
    else
      x = 2.0 *x/(1.0 - y) - 1.0;
    y = 2.0*y - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 2;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 1)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[2][2] =  {{J_11 / detJ, -J_01 / detJ}, {-J_10 / detJ, J_00 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111*psitilde_bs_0_1 + 1.66666667*y*psitilde_bs_0_1 - 0.555555556*psitilde_bs_0_0;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_2_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.707106781*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    const double basisvalue1 = 1.73205081*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
    const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
    const double basisvalue3 = 2.73861279*psitilde_a_2*scalings_y_2*psitilde_bs_2_0;
    const double basisvalue4 = 2.12132034*psitilde_a_1*scalings_y_1*psitilde_bs_1_1;
    const double basisvalue5 = 1.22474487*psitilde_a_0*scalings_y_0*psitilde_bs_0_2;
    
    // Table(s) of coefficients
    const static double coefficients0[6][6] = \
    {{0, -0.173205081, -0.1, 0.121716124, 0.0942809042, 0.0544331054},
    {0, 0.173205081, -0.1, 0.121716124, -0.0942809042, 0.0544331054},
    {0, 0, 0.2, 0, 0, 0.163299316},
    {0.471404521, 0.230940108, 0.133333333, 0, 0.188561808, -0.163299316},
    {0.471404521, -0.230940108, 0.133333333, 0, -0.188561808, -0.163299316},
    {0.471404521, 0, -0.266666667, -0.243432248, 0, 0.0544331054}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[6][6] = \
    {{0, 0, 0, 0, 0, 0},
    {4.89897949, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0},
    {0, 9.48683298, 0, 0, 0, 0},
    {4, 0, 7.07106781, 0, 0, 0},
    {0, 0, 0, 0, 0, 0}};
    
    const static double dmats1[6][6] = \
    {{0, 0, 0, 0, 0, 0},
    {2.44948974, 0, 0, 0, 0, 0},
    {4.24264069, 0, 0, 0, 0, 0},
    {2.5819889, 4.74341649, -0.912870929, 0, 0, 0},
    {2, 6.12372436, 3.53553391, 0, 0, 0},
    {-2.30940108, 0, 8.16496581, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    double coeff0_4 = 0;
    double coeff0_5 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    double new_coeff0_4 = 0;
    double new_coeff0_5 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
      new_coeff0_4 = coefficients0[dof][4];
      new_coeff0_5 = coefficients0[dof][5];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
        coeff0_4 = new_coeff0_4;
        coeff0_5 = new_coeff0_5;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3];
          new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4];
          new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3];
          new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4];
          new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    const static double X[6][1][2] = {{{0, 0}}, {{1, 0}}, {{0, 1}}, {{0.5, 0.5}}, {{0, 0.5}}, {{0.5, 0}}};
    const static double W[6][1] = {{1}, {1}, {1}, {1}, {1}, {1}};
    const static double D[6][1][1] = {{{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}};
    
    const double * const * x = c.coordinates;
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    
    // Compute affine mapping y = F(X)
    double y[2];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1];
    
    // Evaluate function at physical points
    double values[1];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Affine map: Do nothing
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 1; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new QuadratureElementLinearForm_finite_element_2();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class QuadratureElementLinearForm_dof_map_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  QuadratureElementLinearForm_dof_map_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~QuadratureElementLinearForm_dof_map_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 2 on a triangle";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0] + m.num_entities[1];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 6;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    return 2;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    unsigned int offset = m.num_entities[0];
    dofs[3] = offset + c.entity_indices[1][0];
    dofs[4] = offset + c.entity_indices[1][1];
    dofs[5] = offset + c.entity_indices[1][2];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 4;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 5;
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[3][0] = 0.5*x[1][0] + 0.5*x[2][0];
    coordinates[3][1] = 0.5*x[1][1] + 0.5*x[2][1];
    coordinates[4][0] = 0.5*x[0][0] + 0.5*x[2][0];
    coordinates[4][1] = 0.5*x[0][1] + 0.5*x[2][1];
    coordinates[5][0] = 0.5*x[0][0] + 0.5*x[1][0];
    coordinates[5][1] = 0.5*x[0][1] + 0.5*x[1][1];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new QuadratureElementLinearForm_dof_map_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class QuadratureElementLinearForm_dof_map_1_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  QuadratureElementLinearForm_dof_map_1_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~QuadratureElementLinearForm_dof_map_1_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Quadrature element with 9 quadrature point(s) (3x3) on a triangle";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = 9*m.num_entities[2];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 9;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    return 2;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = 9*c.entity_indices[2][0];
    dofs[1] = 9*c.entity_indices[2][0] + 1;
    dofs[2] = 9*c.entity_indices[2][0] + 2;
    dofs[3] = 9*c.entity_indices[2][0] + 3;
    dofs[4] = 9*c.entity_indices[2][0] + 4;
    dofs[5] = 9*c.entity_indices[2][0] + 5;
    dofs[6] = 9*c.entity_indices[2][0] + 6;
    dofs[7] = 9*c.entity_indices[2][0] + 7;
    dofs[8] = 9*c.entity_indices[2][0] + 8;
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.808694386*x[0][0] + 0.102717655*x[1][0] + 0.0885879595*x[2][0];
    coordinates[0][1] = 0.808694386*x[0][1] + 0.102717655*x[1][1] + 0.0885879595*x[2][1];
    coordinates[1][0] = 0.523979068*x[0][0] + 0.0665540678*x[1][0] + 0.409466864*x[2][0];
    coordinates[1][1] = 0.523979068*x[0][1] + 0.0665540678*x[1][1] + 0.409466864*x[2][1];
    coordinates[2][0] = 0.188409406*x[0][0] + 0.0239311323*x[1][0] + 0.787659462*x[2][0];
    coordinates[2][1] = 0.188409406*x[0][1] + 0.0239311323*x[1][1] + 0.787659462*x[2][1];
    coordinates[3][0] = 0.45570602*x[0][0] + 0.45570602*x[1][0] + 0.0885879595*x[2][0];
    coordinates[3][1] = 0.45570602*x[0][1] + 0.45570602*x[1][1] + 0.0885879595*x[2][1];
    coordinates[4][0] = 0.295266568*x[0][0] + 0.295266568*x[1][0] + 0.409466864*x[2][0];
    coordinates[4][1] = 0.295266568*x[0][1] + 0.295266568*x[1][1] + 0.409466864*x[2][1];
    coordinates[5][0] = 0.106170269*x[0][0] + 0.106170269*x[1][0] + 0.787659462*x[2][0];
    coordinates[5][1] = 0.106170269*x[0][1] + 0.106170269*x[1][1] + 0.787659462*x[2][1];
    coordinates[6][0] = 0.102717655*x[0][0] + 0.808694386*x[1][0] + 0.0885879595*x[2][0];
    coordinates[6][1] = 0.102717655*x[0][1] + 0.808694386*x[1][1] + 0.0885879595*x[2][1];
    coordinates[7][0] = 0.0665540678*x[0][0] + 0.523979068*x[1][0] + 0.409466864*x[2][0];
    coordinates[7][1] = 0.0665540678*x[0][1] + 0.523979068*x[1][1] + 0.409466864*x[2][1];
    coordinates[8][0] = 0.0239311323*x[0][0] + 0.188409406*x[1][0] + 0.787659462*x[2][0];
    coordinates[8][1] = 0.0239311323*x[0][1] + 0.188409406*x[1][1] + 0.787659462*x[2][1];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new QuadratureElementLinearForm_dof_map_1_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class QuadratureElementLinearForm_dof_map_1_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  QuadratureElementLinearForm_dof_map_1_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~QuadratureElementLinearForm_dof_map_1_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Quadrature element with 9 quadrature point(s) (3x3) on a triangle";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = 9*m.num_entities[2];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 9;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    return 2;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = 9*c.entity_indices[2][0];
    dofs[1] = 9*c.entity_indices[2][0] + 1;
    dofs[2] = 9*c.entity_indices[2][0] + 2;
    dofs[3] = 9*c.entity_indices[2][0] + 3;
    dofs[4] = 9*c.entity_indices[2][0] + 4;
    dofs[5] = 9*c.entity_indices[2][0] + 5;
    dofs[6] = 9*c.entity_indices[2][0] + 6;
    dofs[7] = 9*c.entity_indices[2][0] + 7;
    dofs[8] = 9*c.entity_indices[2][0] + 8;
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.808694386*x[0][0] + 0.102717655*x[1][0] + 0.0885879595*x[2][0];
    coordinates[0][1] = 0.808694386*x[0][1] + 0.102717655*x[1][1] + 0.0885879595*x[2][1];
    coordinates[1][0] = 0.523979068*x[0][0] + 0.0665540678*x[1][0] + 0.409466864*x[2][0];
    coordinates[1][1] = 0.523979068*x[0][1] + 0.0665540678*x[1][1] + 0.409466864*x[2][1];
    coordinates[2][0] = 0.188409406*x[0][0] + 0.0239311323*x[1][0] + 0.787659462*x[2][0];
    coordinates[2][1] = 0.188409406*x[0][1] + 0.0239311323*x[1][1] + 0.787659462*x[2][1];
    coordinates[3][0] = 0.45570602*x[0][0] + 0.45570602*x[1][0] + 0.0885879595*x[2][0];
    coordinates[3][1] = 0.45570602*x[0][1] + 0.45570602*x[1][1] + 0.0885879595*x[2][1];
    coordinates[4][0] = 0.295266568*x[0][0] + 0.295266568*x[1][0] + 0.409466864*x[2][0];
    coordinates[4][1] = 0.295266568*x[0][1] + 0.295266568*x[1][1] + 0.409466864*x[2][1];
    coordinates[5][0] = 0.106170269*x[0][0] + 0.106170269*x[1][0] + 0.787659462*x[2][0];
    coordinates[5][1] = 0.106170269*x[0][1] + 0.106170269*x[1][1] + 0.787659462*x[2][1];
    coordinates[6][0] = 0.102717655*x[0][0] + 0.808694386*x[1][0] + 0.0885879595*x[2][0];
    coordinates[6][1] = 0.102717655*x[0][1] + 0.808694386*x[1][1] + 0.0885879595*x[2][1];
    coordinates[7][0] = 0.0665540678*x[0][0] + 0.523979068*x[1][0] + 0.409466864*x[2][0];
    coordinates[7][1] = 0.0665540678*x[0][1] + 0.523979068*x[1][1] + 0.409466864*x[2][1];
    coordinates[8][0] = 0.0239311323*x[0][0] + 0.188409406*x[1][0] + 0.787659462*x[2][0];
    coordinates[8][1] = 0.0239311323*x[0][1] + 0.188409406*x[1][1] + 0.787659462*x[2][1];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new QuadratureElementLinearForm_dof_map_1_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class QuadratureElementLinearForm_dof_map_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  QuadratureElementLinearForm_dof_map_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~QuadratureElementLinearForm_dof_map_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Mixed finite element: [Quadrature element with 9 quadrature point(s) (3x3) on a triangle, Quadrature element with 9 quadrature point(s) (3x3) on a triangle]";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = 18*m.num_entities[2];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 18;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    return 2;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = 9*c.entity_indices[2][0];
    dofs[1] = 9*c.entity_indices[2][0] + 1;
    dofs[2] = 9*c.entity_indices[2][0] + 2;
    dofs[3] = 9*c.entity_indices[2][0] + 3;
    dofs[4] = 9*c.entity_indices[2][0] + 4;
    dofs[5] = 9*c.entity_indices[2][0] + 5;
    dofs[6] = 9*c.entity_indices[2][0] + 6;
    dofs[7] = 9*c.entity_indices[2][0] + 7;
    dofs[8] = 9*c.entity_indices[2][0] + 8;
    unsigned int offset = 9*m.num_entities[2];
    dofs[9] = offset + 9*c.entity_indices[2][0];
    dofs[10] = offset + 9*c.entity_indices[2][0] + 1;
    dofs[11] = offset + 9*c.entity_indices[2][0] + 2;
    dofs[12] = offset + 9*c.entity_indices[2][0] + 3;
    dofs[13] = offset + 9*c.entity_indices[2][0] + 4;
    dofs[14] = offset + 9*c.entity_indices[2][0] + 5;
    dofs[15] = offset + 9*c.entity_indices[2][0] + 6;
    dofs[16] = offset + 9*c.entity_indices[2][0] + 7;
    dofs[17] = offset + 9*c.entity_indices[2][0] + 8;
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.808694386*x[0][0] + 0.102717655*x[1][0] + 0.0885879595*x[2][0];
    coordinates[0][1] = 0.808694386*x[0][1] + 0.102717655*x[1][1] + 0.0885879595*x[2][1];
    coordinates[1][0] = 0.523979068*x[0][0] + 0.0665540678*x[1][0] + 0.409466864*x[2][0];
    coordinates[1][1] = 0.523979068*x[0][1] + 0.0665540678*x[1][1] + 0.409466864*x[2][1];
    coordinates[2][0] = 0.188409406*x[0][0] + 0.0239311323*x[1][0] + 0.787659462*x[2][0];
    coordinates[2][1] = 0.188409406*x[0][1] + 0.0239311323*x[1][1] + 0.787659462*x[2][1];
    coordinates[3][0] = 0.45570602*x[0][0] + 0.45570602*x[1][0] + 0.0885879595*x[2][0];
    coordinates[3][1] = 0.45570602*x[0][1] + 0.45570602*x[1][1] + 0.0885879595*x[2][1];
    coordinates[4][0] = 0.295266568*x[0][0] + 0.295266568*x[1][0] + 0.409466864*x[2][0];
    coordinates[4][1] = 0.295266568*x[0][1] + 0.295266568*x[1][1] + 0.409466864*x[2][1];
    coordinates[5][0] = 0.106170269*x[0][0] + 0.106170269*x[1][0] + 0.787659462*x[2][0];
    coordinates[5][1] = 0.106170269*x[0][1] + 0.106170269*x[1][1] + 0.787659462*x[2][1];
    coordinates[6][0] = 0.102717655*x[0][0] + 0.808694386*x[1][0] + 0.0885879595*x[2][0];
    coordinates[6][1] = 0.102717655*x[0][1] + 0.808694386*x[1][1] + 0.0885879595*x[2][1];
    coordinates[7][0] = 0.0665540678*x[0][0] + 0.523979068*x[1][0] + 0.409466864*x[2][0];
    coordinates[7][1] = 0.0665540678*x[0][1] + 0.523979068*x[1][1] + 0.409466864*x[2][1];
    coordinates[8][0] = 0.0239311323*x[0][0] + 0.188409406*x[1][0] + 0.787659462*x[2][0];
    coordinates[8][1] = 0.0239311323*x[0][1] + 0.188409406*x[1][1] + 0.787659462*x[2][1];
    coordinates[9][0] = 0.808694386*x[0][0] + 0.102717655*x[1][0] + 0.0885879595*x[2][0];
    coordinates[9][1] = 0.808694386*x[0][1] + 0.102717655*x[1][1] + 0.0885879595*x[2][1];
    coordinates[10][0] = 0.523979068*x[0][0] + 0.0665540678*x[1][0] + 0.409466864*x[2][0];
    coordinates[10][1] = 0.523979068*x[0][1] + 0.0665540678*x[1][1] + 0.409466864*x[2][1];
    coordinates[11][0] = 0.188409406*x[0][0] + 0.0239311323*x[1][0] + 0.787659462*x[2][0];
    coordinates[11][1] = 0.188409406*x[0][1] + 0.0239311323*x[1][1] + 0.787659462*x[2][1];
    coordinates[12][0] = 0.45570602*x[0][0] + 0.45570602*x[1][0] + 0.0885879595*x[2][0];
    coordinates[12][1] = 0.45570602*x[0][1] + 0.45570602*x[1][1] + 0.0885879595*x[2][1];
    coordinates[13][0] = 0.295266568*x[0][0] + 0.295266568*x[1][0] + 0.409466864*x[2][0];
    coordinates[13][1] = 0.295266568*x[0][1] + 0.295266568*x[1][1] + 0.409466864*x[2][1];
    coordinates[14][0] = 0.106170269*x[0][0] + 0.106170269*x[1][0] + 0.787659462*x[2][0];
    coordinates[14][1] = 0.106170269*x[0][1] + 0.106170269*x[1][1] + 0.787659462*x[2][1];
    coordinates[15][0] = 0.102717655*x[0][0] + 0.808694386*x[1][0] + 0.0885879595*x[2][0];
    coordinates[15][1] = 0.102717655*x[0][1] + 0.808694386*x[1][1] + 0.0885879595*x[2][1];
    coordinates[16][0] = 0.0665540678*x[0][0] + 0.523979068*x[1][0] + 0.409466864*x[2][0];
    coordinates[16][1] = 0.0665540678*x[0][1] + 0.523979068*x[1][1] + 0.409466864*x[2][1];
    coordinates[17][0] = 0.0239311323*x[0][0] + 0.188409406*x[1][0] + 0.787659462*x[2][0];
    coordinates[17][1] = 0.0239311323*x[0][1] + 0.188409406*x[1][1] + 0.787659462*x[2][1];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 2;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new QuadratureElementLinearForm_dof_map_1_0();
      break;
    case 1:
      return new QuadratureElementLinearForm_dof_map_1_1();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class QuadratureElementLinearForm_dof_map_2: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  QuadratureElementLinearForm_dof_map_2() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~QuadratureElementLinearForm_dof_map_2()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 2 on a triangle";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0] + m.num_entities[1];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 6;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    return 2;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    unsigned int offset = m.num_entities[0];
    dofs[3] = offset + c.entity_indices[1][0];
    dofs[4] = offset + c.entity_indices[1][1];
    dofs[5] = offset + c.entity_indices[1][2];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 4;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 5;
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[3][0] = 0.5*x[1][0] + 0.5*x[2][0];
    coordinates[3][1] = 0.5*x[1][1] + 0.5*x[2][1];
    coordinates[4][0] = 0.5*x[0][0] + 0.5*x[2][0];
    coordinates[4][1] = 0.5*x[0][1] + 0.5*x[2][1];
    coordinates[5][0] = 0.5*x[0][0] + 0.5*x[1][0];
    coordinates[5][1] = 0.5*x[0][1] + 0.5*x[1][1];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new QuadratureElementLinearForm_dof_map_2();
  }

};

/// This class defines the interface for the tabulation of the cell
/// tensor corresponding to the local contribution to a form from
/// the integral over a cell.

class QuadratureElementLinearForm_cell_integral_0: public ufc::cell_integral
{
public:

  /// Constructor
  QuadratureElementLinearForm_cell_integral_0() : ufc::cell_integral()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~QuadratureElementLinearForm_cell_integral_0()
  {
    // Do nothing
  }

  /// Tabulate the tensor for the contribution from a local cell
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
      
    // Compute determinant of Jacobian
    double detJ = J_00*J_11 - J_01*J_10;
      
    // Compute inverse of Jacobian
    const double Jinv_00 =  J_11 / detJ;
    const double Jinv_01 = -J_01 / detJ;
    const double Jinv_10 = -J_10 / detJ;
    const double Jinv_11 =  J_00 / detJ;
    
    // Set scale factor
    const double det = std::abs(detJ);
    
    // Number of operations to compute element tensor = 468
    // Compute coefficients
    const double c1_0_0_0 = w[1][0];
    const double c1_0_0_1 = w[1][1];
    const double c1_0_0_2 = w[1][2];
    const double c1_0_0_3 = w[1][3];
    const double c1_0_0_4 = w[1][4];
    const double c1_0_0_5 = w[1][5];
    const double c0_1_0_0 = w[0][0];
    const double c0_1_0_1 = w[0][1];
    const double c0_1_0_2 = w[0][2];
    const double c0_1_0_3 = w[0][3];
    const double c0_1_0_4 = w[0][4];
    const double c0_1_0_5 = w[0][5];
    const double c0_1_0_6 = w[0][6];
    const double c0_1_0_7 = w[0][7];
    const double c0_1_0_8 = w[0][8];
    const double c0_1_0_9 = w[0][9];
    const double c0_1_0_10 = w[0][10];
    const double c0_1_0_11 = w[0][11];
    const double c0_1_0_12 = w[0][12];
    const double c0_1_0_13 = w[0][13];
    const double c0_1_0_14 = w[0][14];
    const double c0_1_0_15 = w[0][15];
    const double c0_1_0_16 = w[0][16];
    const double c0_1_0_17 = w[0][17];
    
    // Compute geometry tensors
    // Number of operations to compute decalrations = 78
    const double G0_0 = det*c1_0_0_0;
    const double G0_1 = det*c1_0_0_1;
    const double G0_2 = det*c1_0_0_2;
    const double G0_3 = det*c1_0_0_3;
    const double G0_4 = det*c1_0_0_4;
    const double G0_5 = det*c1_0_0_5;
    const double G1_0_0_0 = det*c0_1_0_0*Jinv_00;
    const double G1_0_1_0 = det*c0_1_0_1*Jinv_00;
    const double G1_0_2_0 = det*c0_1_0_2*Jinv_00;
    const double G1_0_3_0 = det*c0_1_0_3*Jinv_00;
    const double G1_0_4_0 = det*c0_1_0_4*Jinv_00;
    const double G1_0_5_0 = det*c0_1_0_5*Jinv_00;
    const double G1_0_6_0 = det*c0_1_0_6*Jinv_00;
    const double G1_0_7_0 = det*c0_1_0_7*Jinv_00;
    const double G1_0_8_0 = det*c0_1_0_8*Jinv_00;
    const double G1_0_9_1 = det*c0_1_0_9*Jinv_01;
    const double G1_0_10_1 = det*c0_1_0_10*Jinv_01;
    const double G1_0_11_1 = det*c0_1_0_11*Jinv_01;
    const double G1_0_12_1 = det*c0_1_0_12*Jinv_01;
    const double G1_0_13_1 = det*c0_1_0_13*Jinv_01;
    const double G1_0_14_1 = det*c0_1_0_14*Jinv_01;
    const double G1_0_15_1 = det*c0_1_0_15*Jinv_01;
    const double G1_0_16_1 = det*c0_1_0_16*Jinv_01;
    const double G1_0_17_1 = det*c0_1_0_17*Jinv_01;
    const double G1_1_0_0 = det*c0_1_0_0*Jinv_10;
    const double G1_1_1_0 = det*c0_1_0_1*Jinv_10;
    const double G1_1_2_0 = det*c0_1_0_2*Jinv_10;
    const double G1_1_3_0 = det*c0_1_0_3*Jinv_10;
    const double G1_1_4_0 = det*c0_1_0_4*Jinv_10;
    const double G1_1_5_0 = det*c0_1_0_5*Jinv_10;
    const double G1_1_6_0 = det*c0_1_0_6*Jinv_10;
    const double G1_1_7_0 = det*c0_1_0_7*Jinv_10;
    const double G1_1_8_0 = det*c0_1_0_8*Jinv_10;
    const double G1_1_9_1 = det*c0_1_0_9*Jinv_11;
    const double G1_1_10_1 = det*c0_1_0_10*Jinv_11;
    const double G1_1_11_1 = det*c0_1_0_11*Jinv_11;
    const double G1_1_12_1 = det*c0_1_0_12*Jinv_11;
    const double G1_1_13_1 = det*c0_1_0_13*Jinv_11;
    const double G1_1_14_1 = det*c0_1_0_14*Jinv_11;
    const double G1_1_15_1 = det*c0_1_0_15*Jinv_11;
    const double G1_1_16_1 = det*c0_1_0_16*Jinv_11;
    const double G1_1_17_1 = det*c0_1_0_17*Jinv_11;
    
    // Compute element tensor
    // Number of operations to compute tensor = 390
    A[0] = 0.0166666667*G0_0 - 0.00277777778*G0_1 - 0.00277777778*G0_2 - 0.0111111111*G0_3 + 0.124732813*G1_0_0_0 + 0.0697858496*G1_0_1_0 - 0.00477853908*G1_0_2_0 + 0.0734807188*G1_0_3_0 + 0.0184479255*G1_0_4_0 - 0.0178545702*G1_0_5_0 - 0.032881915*G1_0_6_0 - 0.0467259427*G1_0_7_0 - 0.0175396736*G1_0_8_0 + 0.124732813*G1_0_9_1 + 0.0697858496*G1_0_10_1 - 0.00477853908*G1_0_11_1 + 0.0734807188*G1_0_12_1 + 0.0184479255*G1_0_13_1 - 0.0178545702*G1_0_14_1 - 0.032881915*G1_0_15_1 - 0.0467259427*G1_0_16_1 - 0.0175396736*G1_0_17_1 + 0.124732813*G1_1_0_0 + 0.0697858496*G1_1_1_0 - 0.00477853908*G1_1_2_0 + 0.0734807188*G1_1_3_0 + 0.0184479255*G1_1_4_0 - 0.0178545702*G1_1_5_0 - 0.032881915*G1_1_6_0 - 0.0467259427*G1_1_7_0 - 0.0175396736*G1_1_8_0 + 0.124732813*G1_1_9_1 + 0.0697858496*G1_1_10_1 - 0.00477853908*G1_1_11_1 + 0.0734807188*G1_1_12_1 + 0.0184479255*G1_1_13_1 - 0.0178545702*G1_1_14_1 - 0.032881915*G1_1_15_1 - 0.0467259427*G1_1_16_1 - 0.0175396736*G1_1_17_1;
    A[1] = -0.00277777778*G0_0 + 0.0166666667*G0_1 - 0.00277777778*G0_2 - 0.0111111111*G0_4 + 0.032881915*G1_0_0_0 + 0.0467259427*G1_0_1_0 + 0.0175396736*G1_0_2_0 - 0.0734807188*G1_0_3_0 - 0.0184479255*G1_0_4_0 + 0.0178545702*G1_0_5_0 - 0.124732813*G1_0_6_0 - 0.0697858496*G1_0_7_0 + 0.00477853908*G1_0_8_0 + 0.032881915*G1_0_9_1 + 0.0467259427*G1_0_10_1 + 0.0175396736*G1_0_11_1 - 0.0734807188*G1_0_12_1 - 0.0184479255*G1_0_13_1 + 0.0178545702*G1_0_14_1 - 0.124732813*G1_0_15_1 - 0.0697858496*G1_0_16_1 + 0.00477853908*G1_0_17_1;
    A[2] = -0.00277777778*G0_0 - 0.00277777778*G0_1 + 0.0166666667*G0_2 - 0.0111111111*G0_5 + 0.036036478*G1_1_0_0 - 0.0406181783*G1_1_1_0 - 0.041714596*G1_1_2_0 + 0.0576583648*G1_1_3_0 - 0.0649890852*G1_1_4_0 - 0.0667433537*G1_1_5_0 + 0.036036478*G1_1_6_0 - 0.0406181783*G1_1_7_0 - 0.041714596*G1_1_8_0 + 0.036036478*G1_1_9_1 - 0.0406181783*G1_1_10_1 - 0.041714596*G1_1_11_1 + 0.0576583648*G1_1_12_1 - 0.0649890852*G1_1_13_1 - 0.0667433537*G1_1_14_1 + 0.036036478*G1_1_15_1 - 0.0406181783*G1_1_16_1 - 0.041714596*G1_1_17_1;
    A[3] = -0.0111111111*G0_0 + 0.0888888889*G0_3 + 0.0444444444*G0_4 + 0.0444444444*G0_5 - 0.0197779425*G1_0_0_0 - 0.104296263*G1_0_1_0 - 0.0611109793*G1_0_2_0 - 0.031644708*G1_0_3_0 - 0.166874021*G1_0_4_0 - 0.0977775669*G1_0_5_0 - 0.0197779425*G1_0_6_0 - 0.104296263*G1_0_7_0 - 0.0611109793*G1_0_8_0 - 0.0197779425*G1_0_9_1 - 0.104296263*G1_0_10_1 - 0.0611109793*G1_0_11_1 - 0.031644708*G1_0_12_1 - 0.166874021*G1_0_13_1 - 0.0977775669*G1_0_14_1 - 0.0197779425*G1_0_15_1 - 0.104296263*G1_0_16_1 - 0.0611109793*G1_0_17_1 - 0.0229325055*G1_1_0_0 - 0.0169521424*G1_1_1_0 - 0.00185670966*G1_1_2_0 - 0.162783792*G1_1_3_0 - 0.120332862*G1_1_4_0 - 0.0131796431*G1_1_5_0 - 0.180547234*G1_1_6_0 - 0.133463935*G1_1_7_0 - 0.0146178442*G1_1_8_0 - 0.0229325055*G1_1_9_1 - 0.0169521424*G1_1_10_1 - 0.00185670966*G1_1_11_1 - 0.162783792*G1_1_12_1 - 0.120332862*G1_1_13_1 - 0.0131796431*G1_1_14_1 - 0.180547234*G1_1_15_1 - 0.133463935*G1_1_16_1 - 0.0146178442*G1_1_17_1;
    A[4] = -0.0111111111*G0_1 + 0.0444444444*G0_3 + 0.0888888889*G0_4 + 0.0444444444*G0_5 + 0.0197779425*G1_0_0_0 + 0.104296263*G1_0_1_0 + 0.0611109793*G1_0_2_0 + 0.031644708*G1_0_3_0 + 0.166874021*G1_0_4_0 + 0.0977775669*G1_0_5_0 + 0.0197779425*G1_0_6_0 + 0.104296263*G1_0_7_0 + 0.0611109793*G1_0_8_0 + 0.0197779425*G1_0_9_1 + 0.104296263*G1_0_10_1 + 0.0611109793*G1_0_11_1 + 0.031644708*G1_0_12_1 + 0.166874021*G1_0_13_1 + 0.0977775669*G1_0_14_1 + 0.0197779425*G1_0_15_1 + 0.104296263*G1_0_16_1 + 0.0611109793*G1_0_17_1 - 0.160769291*G1_1_0_0 - 0.0291676713*G1_1_1_0 + 0.0464931351*G1_1_2_0 - 0.131139084*G1_1_3_0 + 0.0465411597*G1_1_4_0 + 0.0845979238*G1_1_5_0 - 0.00315456302*G1_1_6_0 + 0.087344121*G1_1_7_0 + 0.0592542697*G1_1_8_0 - 0.160769291*G1_1_9_1 - 0.0291676713*G1_1_10_1 + 0.0464931351*G1_1_11_1 - 0.131139084*G1_1_12_1 + 0.0465411597*G1_1_13_1 + 0.0845979238*G1_1_14_1 - 0.00315456302*G1_1_15_1 + 0.087344121*G1_1_16_1 + 0.0592542697*G1_1_17_1;
    A[5] = -0.0111111111*G0_2 + 0.0444444444*G0_3 + 0.0444444444*G0_4 + 0.0888888889*G0_5 - 0.157614728*G1_0_0_0 - 0.116511792*G1_0_1_0 - 0.0127611346*G1_0_2_0 + 0.157614728*G1_0_6_0 + 0.116511792*G1_0_7_0 + 0.0127611346*G1_0_8_0 - 0.157614728*G1_0_9_1 - 0.116511792*G1_0_10_1 - 0.0127611346*G1_0_11_1 + 0.157614728*G1_0_15_1 + 0.116511792*G1_0_16_1 + 0.0127611346*G1_0_17_1 + 0.0229325055*G1_1_0_0 + 0.0169521424*G1_1_1_0 + 0.00185670966*G1_1_2_0 + 0.162783792*G1_1_3_0 + 0.120332862*G1_1_4_0 + 0.0131796431*G1_1_5_0 + 0.180547234*G1_1_6_0 + 0.133463935*G1_1_7_0 + 0.0146178442*G1_1_8_0 + 0.0229325055*G1_1_9_1 + 0.0169521424*G1_1_10_1 + 0.00185670966*G1_1_11_1 + 0.162783792*G1_1_12_1 + 0.120332862*G1_1_13_1 + 0.0131796431*G1_1_14_1 + 0.180547234*G1_1_15_1 + 0.133463935*G1_1_16_1 + 0.0146178442*G1_1_17_1;
  }

};

/// This class defines the interface for the assembly of the global
/// tensor corresponding to a form with r + n arguments, that is, a
/// mapping
///
///     a : V1 x V2 x ... Vr x W1 x W2 x ... x Wn -> R
///
/// with arguments v1, v2, ..., vr, w1, w2, ..., wn. The rank r
/// global tensor A is defined by
///
///     A = a(V1, V2, ..., Vr, w1, w2, ..., wn),
///
/// where each argument Vj represents the application to the
/// sequence of basis functions of Vj and w1, w2, ..., wn are given
/// fixed functions (coefficients).

class QuadratureElementLinearForm: public ufc::form
{
public:

  /// Constructor
  QuadratureElementLinearForm() : ufc::form()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~QuadratureElementLinearForm()
  {
    // Do nothing
  }

  /// Return a string identifying the form
  virtual const char* signature() const
  {
    return "w1_a0[0, 1, 2, 3, 4, 5] | vi0[0, 1, 2, 3, 4, 5]*va0[0, 1, 2, 3, 4, 5]*dX(0) + -w0_a1[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17](dXa0[0, 1]/dxa2[0, 1]) | ((d/dXa0[0, 1])vi0[0, 1, 2, 3, 4, 5])*va1[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17][a2[0, 1]]*dX(0)";
  }

  /// Return the rank of the global tensor (r)
  virtual unsigned int rank() const
  {
    return 1;
  }

  /// Return the number of coefficients (n)
  virtual unsigned int num_coefficients() const
  {
    return 2;
  }

  /// Return the number of cell integrals
  virtual unsigned int num_cell_integrals() const
  {
    return 1;
  }
  
  /// Return the number of exterior facet integrals
  virtual unsigned int num_exterior_facet_integrals() const
  {
    return 0;
  }
  
  /// Return the number of interior facet integrals
  virtual unsigned int num_interior_facet_integrals() const
  {
    return 0;
  }
    
  /// Create a new finite element for argument function i
  virtual ufc::finite_element* create_finite_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new QuadratureElementLinearForm_finite_element_0();
      break;
    case 1:
      return new QuadratureElementLinearForm_finite_element_1();
      break;
    case 2:
      return new QuadratureElementLinearForm_finite_element_2();
      break;
    }
    return 0;
  }
  
  /// Create a new dof map for argument function i
  virtual ufc::dof_map* create_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new QuadratureElementLinearForm_dof_map_0();
      break;
    case 1:
      return new QuadratureElementLinearForm_dof_map_1();
      break;
    case 2:
      return new QuadratureElementLinearForm_dof_map_2();
      break;
    }
    return 0;
  }

  /// Create a new cell integral on sub domain i
  virtual ufc::cell_integral* create_cell_integral(unsigned int i) const
  {
    return new QuadratureElementLinearForm_cell_integral_0();
  }

  /// Create a new exterior facet integral on sub domain i
  virtual ufc::exterior_facet_integral* create_exterior_facet_integral(unsigned int i) const
  {
    return 0;
  }

  /// Create a new interior facet integral on sub domain i
  virtual ufc::interior_facet_integral* create_interior_facet_integral(unsigned int i) const
  {
    return 0;
  }

};

#endif
